from twisted.python import util
from twisted.internet.defer import Deferred

from zope.interface import Interface, implements

from nevow import rend, static
from nevow import tags as t
from nevow.stan import directive
from nevow.loaders import xmlfile, stan

from safe.webmanager.template import *

class ResultsPage(TemplatedPage):
    implements(ITemplatedPage)

    page_title = "Results"

    class Content(rend.Fragment):

        docFactory = stan([
            t.table(data=directive("results"), render=directive("sequence"))[
                t.tr(pattern="header")[
                    t.th["id"],
                    t.th["timestamp"],
                    t.th["object"],
                    t.th["metric"],
                    t.th["value"],
                ],
                t.tr(pattern="item", render=directive("row"))[
                    t.td[t.slot(name="id")],
                    t.td[t.slot(name="timestamp")],
                    t.td[t.slot(name="object")],
                    t.td[t.slot(name="metric")],
                    t.td[t.slot(name="value")],
                ]
            ]
        ])

        def __init__(self, dbpool, design_point_id = None, experiment_id = None):
            """Initialize the content object."""

            self.experiment_id = experiment_id
            self.design_point_id = design_point_id
            self.dbpool = dbpool

        def render_row(self, ctx, data):
            ctx.fillSlots('id',data[0])
            ctx.fillSlots('timestamp',data[1])
            ctx.fillSlots('object',data[2])
            ctx.fillSlots('metric',data[3])
            ctx.fillSlots('value',data[4])
            return ctx

        def data_results(self, ctx, data):
            """Method invoked to retrieve the results for the page."""

            if not self.design_point_id:
                query = """
                SELECT r.id, r.timestamp, r.object, r.metric, r.value
                FROM results AS r
                JOIN design_points AS s ON (r.design_point_id = s.id)
                WHERE s.experiment_id = %d
                """ % (self.experiment_id)
            else:
                query = """
                SELECT id, timestamp, object, metric, value
                FROM results
                WHERE design_point_id = %d
                """ % (self.design_point_id)

            return self.dbpool.runQuery(query)
    
    def __init__(self, dbpool, experiment = None, design_point = None):
        """Constructor for the Results() page."""

        super(TemplatedPage, self).__init__()

        self.dbpool = dbpool
        self.experiment = experiment
        self.design_point = design_point
        if experiment:
            self.experiment_id = experiment.cachedValue("id")
        else:
            self.experiment_id = None
        if design_point:
            self.design_point_id = design_point.cachedValue("id")
        else:
            self.design_point_id = None

    def render_content(self,ctx,data):
        """Return the content to be placed in the global template."""
        
        return ctx.tag[self.Content(self.dbpool,
            experiment_id = self.experiment_id,
            design_point_id = self.design_point_id)]
