#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William Stratton (willstrat@gmail.com)
#
import common
import sys
""" This command queries the SAFE server for the status of an experiment. Experiments exist within an installation, which is available for one or more users. Therefore, to identify a running experiment, we need to know the installation, the user, and the experiment.
"""

def print_data(data):
    dpts = data['dpts']
    reps = data['replications']
    total_runs = dpts*reps
    print('%d simruns: %d design points and %d replications' %(total_runs, dpts, reps))
    print('%d / %d runs completed' % (data['complete'], total_runs))
    errors = data['errors']
    print('%d errors' % len(errors))
    for e in errors:
        print('\nError:')
        print('Design Point:   %d' % e['dpt'])
        print('Simulation Run: %d' % e['simrun'])

def main():
    import argparse
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version

    parser = argparse.ArgumentParser(
            add_help=True,
            description='Inspect the status of a running experiment.',
            epilog=epilog_str)

    parser.add_argument('--expid','-e',
            required=False,
            default=None,
            help='experiment id')
    parser.add_argument('--expname','-n',
            required=False,
            default=None,
            help='experiment name')
    args = parser.parse_args()

    if args.expname is None and args.expid is None:
        sys.stderr.write("One of expid or expname is required\n")
        return 1

    import requests
    config = common.get_config()

    if args.expid is None:
        print(">>> querying status of experiment " + args.expname)
        header = {}
        header['Authorization'] = 'Token ' + config['token']
        r = requests.get(config['base-url'] +"view_experiments/", data = {}, headers=header)
        if int(r.status_code)  != 200:
            sys.stderr.write("Not authenticated, please log in before starting an experiment\n")
            return 1
        for e in r.json():
            if e['exp_name'] == args.expname:
                args.expid = str(e['exp_id'])
                break
        else:
            print "Experiment "+str(args.expname)+" Not found, please specify a valid experiment owned by the current user"
            return 1
    else:
        print(">>> querying status of experiment " + args.expid)

    r = requests.get(config['base-url'] +'exp/'+args.expid+'/status/')

    data = r.json()

    print_data(data)

if __name__ == "__main__":
    exit_code = main()
    if exit_code != 0:
        sys.exit(exit_code)
