#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: Vinicius Felizardo (vdf001@bucknell.edu)
#

import requests
import argparse
import getpass
import json
import os
import sys

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--username', '-u',
            required=True,
            help="User account login")
    parser.add_argument('--hostname', '-s',
            required=False,
            help="Server hostname",
            default="127.0.0.1")
    parser.add_argument('--port', '-r',
            required=False,
            help="Server port",
            default="8000")

    args = parser.parse_args()

    password = getpass.getpass("Password:")
    
    #Ok, now we have all arguments. Lets try to connect
    BASE_URL = 'http://'+args.hostname+':'+args.port+'/safe/'

    auth_data = {'username': args.username, 'password': password}
    r = requests.post(BASE_URL+'api-token-auth/', data=auth_data)
    if r.status_code == 200:
        #Authenticated :)
        token = r.json()["token"]
        #Lets save BASE_URL and the TOKEN in the configuration file
        with open(os.path.abspath(os.path.join(os.path.expanduser('~'), '.safe_cfg')), 'w') as outfile:
            json.dump({"token":token, "base-url":BASE_URL, "username":args.username }, outfile)
        print("Success")
        return 0
    else:
        print('Not possible to connect')
        return 1


if __name__ == "__main__":
    if main() != 0:
        sys.exit(1)
