#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William Stratton (wss012@bucknell.edu)
#

import argparse
import requests
import getpass
import json
import common
import re

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--username', '-u',
            default=None,
            help="User account login")
    parser.add_argument('--email', '-e',
            default='nope',
            help="User's e-mail addresss")
    parser.add_argument('--superuser', '-s',
            action='store_true',
            help="Is the new user a superuser?")
    parser.add_argument('--firstname', '-f',
            default = ' ',
            help="The user's first name")
    parser.add_argument('--lastname', '-l',
            default = ' ',
            help="The user's last name")
    parser.add_argument('--staff', '-t',
            action='store_true',
            help="Is the new user staff?")

    args = parser.parse_args()

    if args.username == None:
        args.username = raw_input('Username: ')

    valid_pass = False
    while not valid_pass:
        password = getpass.getpass('Password: ')
        confirm = getpass.getpass('Confirm password: ')
        valid_pass = (password == confirm)
        if not valid_pass:
            print('Passwords did not match. Try again.')

    while not re.match(r"[^@]+@[^@]+\.[^@]+", args.email):
        args.email = raw_input('Please enter a vaild email address.')

    # Get config information and set header
    config = common.get_config()
    header = {'Authorization': 'Token ' + config['token']}
    header['content-type'] = 'application/json'

    data = {'username':args.username, 'email':args.email, 'password':password, 'is_superuser': args.superuser, 'first_name':args.firstname, 'last_name':args.lastname, 'is_staff': args.staff}

    r = requests.post(config['base-url']+'create-user/', headers = header, data=json.dumps(data))
    
    if r.status_code >= 400:
        print("Error creating user")
        return 1
    print("Success")
    return 0




if __name__ == "__main__":
    if main() != 0:
        sys.exit(1)
