#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William S. Stratton (willstrat@gmail.com)
#

""" This command queries the SAFE server for the results of an experiment. The user must specify the id of the metric and simulation run that they want the results from. The user may also specify the format of the data (default json)
"""

import common

def main():
    import argparse
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version 

    parser = argparse.ArgumentParser(
            add_help=True,
            description='Dump data to a file.',
            epilog=epilog_str)
    
    parser.add_argument('--metric','-m',
            required=True,
            help='metric id')

    parser.add_argument('--simrun','-s',
            required=True,
            help='simulation run id')

    parser.add_argument('--filetype','-f',
            required=False,
            default='json',
            help='file type')

    parser.add_argument('filename',
                        nargs="?",
                        help='name of dump file')
                        

    args = parser.parse_args()
        
    print ">>> dumping results of metric " + args.metric + " from run " + args.simrun + " to "+ ("stdout" if args.filename is None else "a " + args.filetype + " file named " + args.filename )

    json = fetch_data(args.simrun, args.metric)
    
    dumpdata(json, args.filetype, args.filename)

def fetch_data(simrun, metric):
    import requests
    config = common.get_config()
    header = {}
    header['Authorization'] = 'Token ' + config['token']        
    header['content-type'] = 'application/json'
    r = requests.get(config['base-url'] + 'simrun/'+str(simrun)+'/metric/'+str(metric)+'/results/', headers = header)
    return r.json()

def dumpdata(json, filetype, filename):
    if filetype == 'csv':
        dump_csv(json, filename)
    else:
        dump_json(json, filename)

def dump_json(json_data, filename):
    import json
    if filename is None:
        print json.dumps(json_data, indent=4)
    else:
        new_filename = filename
        if new_filename[-5:] != '.json':
            new_filename = new_filename + '.json'
        outfile = open(new_filename, 'w')
        json.dump(json_data, outfile)


def dump_csv(json, filename):
    import csv
    outfile = open(filename+'.csv', 'w')
    header = json[0].keys()
    writer = csv.DictWriter(outfile, header)
    writer.writeheader()
    map(writer.writerow, json)
    outfile.close()
 
if __name__ == "__main__":
    main()
