$(document).ready(function(){
	
	// Sets up the Backbone models
	var Factor = Backbone.Model.extend({
		defaults: {
		    name: "Factor",
		    description: "Details about the factor go here.",
		    type: "double",
		    id: 0,
		    levels: new Array()
		}
	    });

	var Factors = Backbone.Collection.extend({
		model: Factor
	    });

	var Experiment = Backbone.Model.extend({
		defaults: {
		    name: "Experiment Name",
		    description: "A description of the experiment goes here.",
		    replications: 0,
		    exp_description_id: 0,
		    public: false,
		    factors: new Factors()
		}
	    });

	// create global vars
	var workingExperiment = new Experiment();
	var workingFactor = new Factor();
	
	// Call the appropriate function upon arriving at a new page
	// You will likely never call this function outside of changePage()
	var arriveAtPage = {
	    main: function(){},
	    login: function(){},
	    experiment_list: function(){},
	    description_list: function(){
		$.get('/safe/exp-descs/',
		      function(data){
			  var descs = data.results;
			  $("ul#descriptions").html("");
			  for (var i = 0; i < descs.length; i++){
			      var desc = descs[i];
			      $("ul#descriptions").append("<li> <a id="+desc.id+" class=exp_desc title='"+desc.description+"' href=#> "+desc.name+"</a> </li>");
			  }
			});
		
	    },
	    experiment_info: function(){
		$('form input.name').val(workingExperiment.get('name'));
		$('form textarea.description').val(workingExperiment.get('description'));
		$('form input.replications').val(workingExperiment.get('replications'));
		$('form input.public').val(workingExperiment.get('public'));
		
	    },
	    factor_list: function(){
		var factors = workingExperiment.get('factors');
		$('ul#factor_list').html("");
		for (var i = 0; i < factors.length; i++){
		    var fac = factors.at(i);
		    $('ul#factor_list').append("<li> <a id="+i+" class=factor title='"+fac.get('description')+"' href=#> "+fac.get('name')+"</a> </li>");
		}
	    },
	    level_config: function(){
		$('p#factor_name').text(workingFactor.get('name'));
		$('p#factor_description').text(workingFactor.get('description'));
		$('ul#levels').html("");
		var type = workingFactor.get('type');
		for (level in workingFactor.get('levels')){
		    if (type == 'double' || type == 'double_double' || type == 'int'){
		    $('ul#level_list').append("<li> <input type='number' class='level value="+level+"'/> </li>");
		    }else if (type == 'bool'){
			//TODO: THINGS HERE
		    }
		}
	    }
	};

	// change to the given page
	var changePage = function(destination){
	    // css files set all div's but the given one to not display
	    // There is probably a way to do this that does not require files
		var filename = "/static/safe/css/"+ destination +".css";
		$("link").attr("href",filename);
		// No
		arriveAtPage[destination]();
	    }

	// Uses the changePage class on buttons
	// This will probably be obselete when the page is completed
	$(".changePage").click(function(){
		changePage($(this).attr("name"))
		    });

	// Loading a new experiment description into the models
	$('body').on('click', 'a.exp_desc', function(){
		$.get('/safe/exp-descs/'+this.id+'/factors/',
		      function(data){
			  workingExperiment.set({'name': data.name, 'description': data.description, 'replications': data.replications, 'exp_description_id': data.exp_description_id, 'public': data.public});
			  factors = new Factors();
			  for (fac in data.factors){
			      factors.add([{
					  'name': fac.name, 
					      'description': fac.description, 
					      'type': fac.data_type, 
					      'id': fac.id, 
					      'levels': fac.levels
				      }]);
			  }
			  workingExperiment.set({'factors':factors});
			  changePage("experiment_info");
		      });
		
	    });

	// Set the working factor to the chosen factor
	// and directs to the appropriate level config page
	$('body').on('click', 'a.factor', function(){
		workingFactor = workingExperiment.get('factors').at(this.id);
		changePage('level_config');
	    });

	// Stores the experiment info in the Backbone structure
	$("button#submit_exp_info").click(function(){
		//TODO: public always gives false. fix this
		workingExperiment.set({'name':$('form input.name').val(), 'description':$('form textarea.description').val(), 'replications':$('form input.replications').val(), 'public': boolean($('form input.public').val())});
		changePage($(this).attr('name'));
	    });

	// Creates an empty level entry box
	// Note: add level delete functionality
	$("button#add_level").click(function(){
		var type = workingFactor.get('type');
		if (type == 'double' || type == 'double_double' || type == 'int'){
		    $('ul#level_list').append("<li> <input type='number' class='level'/> </li>");
		} else if (type == 'bool'){
		    // TODO: put things here
		    // limit the number of booleans to 2
		    // and force 1 true and 1 false
		}
	    });
	    
	// Stores the entered levels in the backbone model.
	$("button#save_levels").click(function(){
		workingFactor.set({levels: new Array()});
		$("input.level").each(function(index){
			workingFactor.get('levels').push(this.value);
		    });
		changePage('factor_list');
	    });
    });