function BaseMagnitude(){
    //console.log("Initializing BaseMagnitude...");
}

BaseMagnitude.prototype.init = function(margins, width, height, parentDomTag){
    this.xMin = this.yMin = this.xMax = this.yMax = 0;
    this.data = {};
    this.toDisplay = {};
    this.pointAttributes = {};
    this.mouseEvents = {};
    this.jMargin = margins;
    this.height = height - this.jMargin.top - this.jMargin.bottom;
    this.width = width - this.jMargin.left - this.jMargin.right;

    this.dXScale = d3.scale.linear().range([0, this.width]).domain([0, 0]);
    this.dYScale = d3.scale.linear().range([this.height, 0]).domain([0, 0]);

    this.dXAxis = d3.svg.axis().scale(this.dXScale).orient("bottom");
    this.dYAxis = d3.svg.axis().scale(this.dYScale).orient("left");

    this.svg = d3.select("#" + parentDomTag)
        .append("svg")
        .attr("width", this.width + this.jMargin.left + this.jMargin.right)
        .attr("height", this.height + this.jMargin.top + this.jMargin.bottom + 30);

	// Text label for x axis
	this.svg.append("text")
        .attr("class", "x label")
		.attr("x", this.width / 2)
		.attr("y", this.height + this.jMargin.top + this.jMargin.bottom)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	this.svg.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)

		.attr("x", 0 - ((this.height) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	this.svg.append("g")
		.attr("class", "x axis")
		.attr("transform", "translate(" + this.jMargin.left + "," + (this.height + this.jMargin.top) + ")")
		.call(this.dXAxis);

    // Add y axis
    this.svg.append("g")
		.attr("class", "y axis")
		.attr("transform", "translate(" + this.jMargin.left + "," + this.jMargin.top + ")")
		.call(this.dYAxis);

    return this;
}

BaseMagnitude.prototype.displayAll = function(){
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] == true) { this.display(id) }
        }.bind(this));
}

BaseMagnitude.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
    this.data[id] = data;
    this.pointAttributes[id] = pointAttributes;
    this.mouseEvents[id] = mouseEvents;
    this.toDisplay[id] = true;
    this.updateDataFeatures();
}

BaseMagnitude.prototype.setInvisible = function(id){
    this.toDisplay[id] = false;
    this.updateDataFeatures();
    this.display(id);
}

BaseMagnitude.prototype.setVisible = function(id){
    this.toDisplay[id] = true;
    this.updateDataFeatures();
    this.display(id);
}

BaseMagnitude.prototype.remove = function(id){
    delete this.data[id];
    delete this.pointAttributes[id];
    delete this.mouseEvents[id];
    
}

BaseMagnitude.prototype.hasData = function(id){
    if( this.data[id] != undefined ){ return true }
    else { return false }
}

BaseMagnitude.prototype.updateDomains = function(xDomain, yDomain){
    if(xDomain != undefined) { this.dXScale.domain(xDomain) }
    if(yDomain != undefined) { this.dYScale.domain(yDomain) }

    this.svg.select(".x.axis").call(this.dXAxis);
    this.svg.select(".y.axis").call(this.dYAxis);
}

BaseMagnitude.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d.time });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yRange = d3.extent(this.data[id], function(d) { return d.value});
        this.yMin = (this.yMin == undefined) ? yRange[0] : d3.min([this.yMin, yRange[0]]);
        this.yMax = (this.yMax == undefined) ? yRange[1] : d3.max([this.yMax, yRange[1]]);
        }.bind(this));

    this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}

BaseMagnitude.prototype.updateCrossHairs = function(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xleft = this.dXScale.range()[0] + this.jMargin.left; 
    var xright = this.dXScale.range()[1] + this.jMargin.left;
    var ytop = this.dYScale.range()[0] + this.jMargin.top;
    var ybottom = this.dYScale.range()[1] + this.jMargin.top;

    if (x < xleft) { x = xleft }
    if (x > xright) { x = xright }
    if (y < ybottom) { y = ybottom }
    if (y > ytop) { y = ytop }

    horizontal = this.svg.selectAll(".crosshairhorizontal").data([""]);
    vertical = this.svg.selectAll(".crosshairvertical").data([""]);
 
    horizontal.enter().append("line").attr("class", "crosshairhorizontal").style("stroke", "black").style("pointer-events", "none")
    vertical.enter().append("line").attr("class", "crosshairvertical").style("stroke", "black").style("pointer-events", "none")
   
    horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

BaseMagnitude.prototype.enableCrossHairs = function(){
    this.svg.on("mousemove", this.updateCrossHairs.bind(this));
    this.svg.on("mouseout", function(){
        this.svg.selectAll(".crosshairhorizontal").remove();
        this.svg.selectAll(".crosshairvertical").remove();
        }.bind(this));
}

BaseMagnitude.prototype.disableCrossHairs = function(){
    this.svg.selectAll(".crosshairhorizontal").remove();
    this.svg.selectAll(".crosshairvertical").remove();
    this.svg.on("mousemove", undefined);

}
