//----------------------------------------
// Display Elements
// ---------------------------------------

displayElementsNumber = 0;
DisplayElements = function(removeCallBack, parentDomId){
    this.parentDomId = parentDomId;
    this.removeCallBack = removeCallBack;
    this.dParent = d3.selectAll("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.id = "display_elements_" + displayElementsNumber;
    displayElementsNumber += 1;

    this.dSelf = this.dParent.selectAll("#" + this.id).data([""]).enter().append("div")
        .attr("id", this.id)
        .attr("class", "display_elements");

    this.elements = {};
    this.colors = {};
    this.returnAttributes = {};

}

DisplayElements.prototype.addElement = function(key, elementString, elementColor, returnAttributes){
    this.elements[key] = elementString;
    this.colors[key] = elementColor;
    this.returnAttributes[key] = returnAttributes;
    this.updateElements();
}

DisplayElements.prototype.removeElement = function(key){
    delete this.elements[key];
    delete this.colors[key];
    delete this.returnAttributes[key];
    this.updateElements();
}

DisplayElements.prototype.removeSelected = function(key){
    this.removeCallBack(this.returnAttributes[key]);
}

DisplayElements.prototype.updateElements = function(){
	this.dSelf.selectAll(".key_line").remove();
	var displayed = this.dSelf.selectAll(".key_line").data(Object.keys(this.elements))
			.enter()
			.append("div")
			.attr("class", "key_line")

    displayed.append("div")
        .style("float", "left")
        .text("X")
        .on("click", function(d){ this.removeSelected(d) }.bind(this));

	displayed.append("div")
		.attr("class", "key_square")
		.style("background", function(d){ return this.colors[d] }.bind(this));

	displayed.append("div")
		.attr("class", "key_label")
		.text(function(d) { return this.elements[d] }.bind(this));

    displayed.sort(function(a, b){ return this.elements[a].localeCompare(this.elements[b]) }.bind(this));
}

//----------------------------------------
// Selection Parameters
//---------------------------------------

selectionParametersNumber = 0;
SelectionParameters = function(removeCallBack, parentDomId){
    this.parentDomId = parentDomId;
    this.removeCallBack = removeCallBack;
    this.elements = {};
    this.returnAttributes = {};

    this.id = this.parentDomId + "_selection_parameters_" + selectionParametersNumber;
    selectionParametersNumber += 1;

    this.dSelf = d3.select("#" + this.parentDomId)
        .append("div")
        .attr("class", "selection_parameters")
        .attr("id", this.id);
}

SelectionParameters.prototype.addElement = function(key, elementString, returnAttributes){
    this.elements[key] = elementString;
    this.returnAttributes[key] = returnAttributes;
    this.updateElements();
}

SelectionParameters.prototype.removeElement = function(key){
    console.log("Deleting: ", key);
    delete this.elements[key];
    delete this.returnAttributes[key];
    this.updateElements();
}

SelectionParameters.prototype.removeSelected = function(key){
    this.removeCallBack(this.returnAttributes[key]);
}

SelectionParameters.prototype.updateElements = function(){
    console.log(this.elements);
    this.dSelf.selectAll(".select_line").remove();

    var parameters = this.dSelf.selectAll(".select_line").data(Object.keys(this.elements))
        .enter()
        .append("div")
        .attr("class", "select_line")

     parameters
        .append("div")
        .style("float", "left")
        .text("X : ")
        .on("click", function(d){ return this.removeSelected(d); }.bind(this)); 

    parameters
        .append("div")
        .attr("class", "select_label")
        .text(function(d) { return this.elements[d] }.bind(this));
}
