LineMagnitude.prototype = new BaseMagnitude();
LineMagnitude.prototype.constructor = LineMagnitude;

function LineMagnitude(margins, width, height, parentDomTag, pad){
    //console.log("Initializing LineMagnitude...");
    BaseMagnitude.apply(this);
    this.pad = (pad == undefined) ? false : pad;
    this.init(margins, width, height, parentDomTag);

    return this;
}

LineMagnitude.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        //console.log("Removing...", this.toDisplay[id]);
        this.svg.selectAll(".dataGroup_" + id).remove();
        this.svg.selectAll(".dataPath_" + id).remove();
        return;
    }

    currentData = [];
    var groupNextLeft = {}, groupNextRight = {};
    var dataNextLeft = {}, dataNextRight = {};
    
   	this.data[id].forEach(function(dataPoint){
        if(this.toDisplay[id] != true){ return }
       
        if (dataPoint["xValue"] < this.dXScale.domain()[0]){
            if(groupNextLeft["xValue"] != undefined && groupNextLeft["xValue"] > dataPoint["xValue"]){ return }
            else{ groupNextLeft["xValue"] = dataPoint["xValue"]; groupNextLeft["medianValue"] = dataPoint["medianValue"] }

        }else if (dataPoint["xValue"] > this.dXScale.domain()[1]){
            if(groupNextRight["xValue"] != undefined && groupNextRight["xValue"] < dataPoint["xValue"]){ return }
            else{ groupNextRight["xValue"] = dataPoint["xValue"]; groupNextRight["medianValue"] = dataPoint["medianValue"] }

        }else{
            currentData.push(dataPoint);
        }
	}.bind(this));
    
    currentData.sort(function(a, b){ return a["xValue"] - b["xValue"] });
    var leftMost = currentData.length > 0 ? currentData[0] : groupNextRight;
    var rightMost = currentData.length > 0 ? currentData[currentData.length - 1] : groupNextLeft;

    //console.log(this.dXScale.domain());
    if(groupNextLeft["xValue"] != undefined){
        var slope = (groupNextLeft["medianValue"] - leftMost["medianValue"]) / (groupNextLeft["xValue"] - leftMost["xValue"]);
        var dx = this.dXScale.domain()[0] - leftMost["xValue"];
        dataNextLeft["xValue"] = this.dXScale.domain()[0];
        dataNextLeft["medianValue"] = leftMost["medianValue"] + dx * slope;
    }

    if(groupNextRight["xValue"] != undefined){
        var slope = (groupNextRight["medianValue"] - rightMost["medianValue"]) / (groupNextRight["xValue"] - rightMost["xValue"]);
        var dx = this.dXScale.domain()[1] - rightMost["xValue"];
        //console.log("Group:", groupNextRight, ", RightMost:", rightMost, ", Slope:", slope, ", dx:", dx, ", dataNextLeft:", groupNextLeft, ", leftMost:", leftMost);
        dataNextRight["xValue"] = this.dXScale.domain()[1];
        dataNextRight["medianValue"] = rightMost["medianValue"] + dx * slope;
    }

    //============================================

    groups = this.svg.selectAll(".dataGroup_" + id).data(currentData);
    groups.enter().append("g").attr("class", "dataGroup_" + id);
    groups.exit().remove();

    points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    //points = groups.selectAll(".subPoint").data(["lowDevValue", "medianValue", "highDevValue"]);
    points.enter().append("circle").attr("class", "subPoint").on(this.mouseEvents[id]);
    points
        .attr("cx", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"])}.bind(this))
        .attr("cy", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f][d]) }.bind(this))
        .attr(this.pointAttributes[id]);
    points.exit().remove();

    //============================================
 
    lines = groups.selectAll(".subline").data([""]);
    lines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke-dasharray":"5,5", "stroke":this.pointAttributes[id].stroke});
    lines
        .attr("x1", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"]); }.bind(this))
        .attr("x2", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"]); }.bind(this))
        .attr("y1", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f]["lowDevValue"]); }.bind(this))
        .attr("y2", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f]["highDevValue"]); }.bind(this));
    lines.exit().remove();

    //=============================================
    line = d3.svg.line()
        .x(function (d) { return this.jMargin.left + this.dXScale(d["xValue"]) }.bind(this))
        .y(function (d) { return this.jMargin.top + this.dYScale(d["medianValue"]) }.bind(this));

    if (dataNextLeft.xValue != undefined) { currentData.unshift(dataNextLeft); }
    if (dataNextRight.xValue != undefined) { currentData.push(dataNextRight); }

    paths = this.svg.selectAll(".dataPath_" + id).data([""]);

    if (currentData.length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "dataPath_" + id)
            .attr("stroke", this.pointAttributes[id].stroke)
            .attr("stroke-width", 1);

        paths.attr("d", function () { return line(currentData) });
        paths.exit().remove();
    }else{
        paths.remove();
    }
}

LineMagnitude.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d["xValue"] });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yMin = d3.min(this.data[id], function(d) { return d["minValue"] });
        yMax = d3.max(this.data[id], function(d) { return d["maxValue"] });
        //yMin = d3.min(this.data[id], function(d) { return d["lowDevValue"] });
        //yMax = d3.max(this.data[id], function(d) { return d["highDevValue"] });

        yRange = d3.extent(this.data[id], function(d) { return d.value});
        this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
        this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
        }.bind(this));

    this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}

LineMagnitude.prototype.updateDomains = function(xDomain, yDomain){
    
    var xValLeft, xValRight;
    range = xDomain[1] - xDomain[0];
    if(this.pad && range > 0){
        pad = 0.1 * range;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[1] + pad;
    }
    else if (this.pad){
        pad = xDomain[1] > 10 ? 10 : xValMax;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[0] + pad;
    }else{
        xValLeft = xDomain[0];
        xValRight = xDomain[1];
    }
    

	this.dXScale.range([0, this.width]).domain([xValLeft, xValRight]);
	this.dYScale.range([this.height, 0]).domain([0, this.yMax]);

	this.dYAxis.scale(this.dYScale);
	this.svg.select(".y.axis").call(this.dYAxis);

	this.dXAxis.scale(this.dXScale);
    this.svg.select(".x.axis").call(this.dXAxis);
}

