// Layout components
var jMarginFocus, jMarginContext, sFocusWidth, sContextWidth, sFocusHeight, sContextHeight;
// Scaling 
var dXScaleFocus, dYScaleFocus, dXScaleContext, dYScaleContext;
// Axes to be used for the graph
var xAxisFocus, yAxisFocus, xAxisContext, yAxisContext;
// Areas where things will be graphed
var svg, context, focus, brush;
// Our data, this will need to change
var designPointList, factorList,  runList, metricList, metricData, metricStatistics, aDisplayList, levelList, parameterList;
// Inportant details about the data
var xValMin, xValMax, yValMin, yValMax;
// The name of the independent variable / dependent variables (Array)
var aDisplayList, aDisplayColor, aColors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var exp_id;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
    jMarginFocus = {top: 40, right: 40, bottom: 20, left: 40};
	jMarginContext = { top: 20, right: jMarginFocus.right, bottom: 20, left: 40 };
	sFocusWidth = 960 - jMarginFocus.left - jMarginFocus.right;
	sFocusHeight = 500 - jMarginFocus.top - jMarginFocus.bottom;
	sContextWidth = sFocusWidth;
	sContextHeight = 150 - jMarginContext.top - jMarginContext.bottom;

	// Configure time scales
	dXScaleFocus = d3.scale.linear().range([0, sFocusWidth]).domain([0,0]).clamp(true);
	dYScaleFocus = d3.scale.linear().range([sFocusHeight, 0]).domain([0,0]).clamp(true);
	dXScaleContext = d3.scale.linear().range([0, sContextWidth]).domain([0, 0]);
	dYScaleContext = d3.scale.linear().range([sContextHeight, 0]).domain([0, 0]);

	// Configure axis
	xAxisFocus = d3.svg.axis().scale(dXScaleFocus).orient("bottom");
	yAxisFocus = d3.svg.axis().scale(dYScaleFocus).orient("left");
	xAxisContext = d3.svg.axis().scale(dXScaleContext).orient("bottom");
	yAxisContext = d3.svg.axis().scale(dYScaleContext).orient("left");

    // Add focus plot
	focus = d3.select("#chart1")
        .append("svg")
        .attr("width", sFocusWidth + jMarginFocus.left + jMarginFocus.right)
        .attr("height", sFocusHeight + jMarginFocus.top + jMarginFocus.bottom + 30);

    // Add context plot
    context = d3.select("#chart2")
        .append("svg")
        .attr("width", sContextWidth + jMarginContext.left + jMarginContext.right)
        .attr("height", sContextHeight + jMarginContext.top + jMarginContext.bottom + 30)

	// Text label for x axis
	focus.append("text")
        .attr("class", "x label")
		.attr("x", sFocusWidth / 2)
		.attr("y", sFocusHeight + jMarginFocus.top + jMarginFocus.bottom)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	focus.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((sFocusHeight) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	focus.append("g")
		.attr("class", "x axis focus")
		.attr("transform", "translate(" + jMarginFocus.left + "," + (sFocusHeight + jMarginFocus.top) + ")")
		.call(xAxisFocus);

	context.append("g")
        .attr("class", "x axis context")
        .attr("transform", "translate(" + jMarginContext.left + "," + (sContextHeight + jMarginContext.top) + ")")
        .call(xAxisContext);

	// Add y axis
    focus.append("g")
		.attr("class", "y axis focus")
		.attr("transform", "translate(" + jMarginFocus.left + "," + jMarginFocus.top + ")")
		.call(yAxisFocus);

    context.append("g")
		.attr("class", "y axis context")
		.attr("transform", "translate(" + jMarginContext.left + "," + jMarginContext.top + ")")
		.call(yAxisContext);

    brush = d3.svg.brush() 
        .x(dXScaleContext)
        .on("brush", brushed);

    context.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + jMarginContext.left + "," + jMarginContext.top + ")")
        .call(brush)
        .selectAll("rect")
            .attr("y", -6)
            .attr("height", sContextHeight + 7);

    // Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	aDisplayColor = {};
    aDisplayList = {};
	metricData = {};
	metricStatistics = {};
    aColors = ["blue", "green", "orange", "purple", "black", "red"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
	parameterList = {};
    numberFormat = d3.format(".2f");
}

// Initializes the selection for the graph
function initGraphSelect(){
    // Initialize the factor list
    d3.selectAll("#select")
        .append("div")
        .append("select")
        .attr("class", "key_line")
        .attr("id", "selectFactor")
        .on("change", factorSelectChange);

    d3.select("#selectFactor")
        .selectAll("option")
        .data(Object.keys(factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d } )
        .text(function(d) { return factorList[d].name });

    // Initialize metric selects
    var select_items = d3.select("#select").selectAll(".select_line").data(Object.keys(metricList)).enter()
        .append("div")
        .attr("class", "select_line")
        .attr("metric_id", function(d) { return metricList[d].id })
        .attr("id", function(d) { return "select_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_square")
        .attr("id", function(d) { return "select_square_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_label")
        .attr("id", function(d) { return "select_label_" + metricList[d].id })
        .text(function(d) { return metricList[d].name});

    d3.selectAll(".select_line")
        .on("click", selectElementChange);

    // Initialize the cross hairs option
    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

// Reads json data from the given url
// When it's loaded the given callBack function
// is called with the resulting data
function readData(url, callBack){
    d3.json(url, function(error, json){
        if (error) return console.warn(error);

        callBack(json);
    });
}

// Reads metric information
function readInitialData(){

    readData("/safe/exp/" + exp_id + "/designpoints/", function(data){
        designPointList = [];
        runList = [];
        data.forEach(function(designPoint){ designPointList[designPoint.id] = designPoint});
        data.forEach(function(designPoint){ readData("/safe/designpoint/" + designPoint.id + "/simruns/", function(data){
                runList.concat(data);
                designPoint["runs"] = data;
            });
            });

        readData("/safe/exp/" + exp_id + "/factors/", function(data){
            factorList = {};
            levelList = {};
            data.forEach(function(factor){ factorList[factor.id] = factor });
            readData("/safe/exp/" + exp_id + "/levels/", function(data){
                Object.keys(factorList).forEach(function(factor_id){
                    levelList[factor_id] = data[factor_id].levels.sort();
                  });
                metricList = {};
                readData("/safe/exp/" + exp_id + "/metrics/", function(data){
                    data.forEach(function(element){
                        metricList[element.id] = element;
                      });
                    initGraphSelect();
                    updateDataFeatures();
                    displayData();
                  });
              });
            
        });
    });
}


// displays the currently held data to the graph
function displayData(){
	Object.keys(aDisplayList).forEach(function (factor_id) {
            Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
                updateContextDataPoints(factor_id, metric_id);
            });
	});
}
// updates the positions of the center points described by the data groupings
function updateContextDataPoints(factor_id, metric_id){
    currentData = [];

	Object.keys(metricStatistics[factor_id][metric_id]).forEach(function(level){
        if(aDisplayList[factor_id][metric_id] == true)
            currentData.push(metricStatistics[factor_id][metric_id][level] );
	});

    contextgroups = context.selectAll(".contextDataGroup_" + factor_id + "_" + metric_id).data(currentData);
    contextgroups.enter().append("g").attr("class", "contextDataGroup_" + factor_id + "_" + metric_id);
    contextgroups.exit().remove();
    //console.log("Groups:", contextgroups);

    contextpoints = contextgroups.selectAll(".subpoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    contextpoints.enter()
        .append("circle")
        .attr("class", "contextDataPoint_" + factor_id + "_" + metric_id)
        .attr("stroke", aDisplayColor[factor_id][metric_id] )
        .attr("stroke-width", 1.5)
        .attr("fill", "white")
        .attr("fill-opacity", 0)
        .attr("r", contextDataPointRadius)
        .on("mouseover", mouseOverDataPoint)
        .on("mouseout", mouseOutDataPoint);

    contextpoints
        .attr("cx", function (d, e, f) { return jMarginContext.left + dXScaleContext(currentData[f]["xValue"])})
        .attr("cy", function (d, e, f) { return jMarginContext.top + dYScaleContext(currentData[f][d]) })
        .attr("info", function (d, e, f) { return numberFormat(currentData[f]["xValue"]) + "<br/>" + numberFormat(currentData[f][d]) } );

    contextpoints.exit()
        .remove();

    contextlines = contextgroups.selectAll(".subline").data([""]);
    contextlines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke-dasharray":"5,5", "stroke":aDisplayColor[factor_id][metric_id] });
    contextlines
        .attr("x1", function (d, e, f) { return jMarginContext.left + dXScaleContext(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return jMarginContext.left + dXScaleContext(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return jMarginContext.top + dYScaleContext(currentData[f]["lowDevValue"]); })
        .attr("y2", function (d, e, f) { return jMarginContext.top + dYScaleContext(currentData[f]["highDevValue"]); });
    contextlines.exit().remove();

//=============================================
    line = d3.svg.line()
        .x(function (d) { return jMarginContext.left + dXScaleContext(d["xValue"]) })
        .y(function (d) { return jMarginContext.top + dYScaleContext(d["medianValue"]) });


    paths = context.selectAll(".contextDataPath_" + factor_id + "_" + metric_id).data([""]);

    if (currentData.length > 0) {
        paths.enter()
            .append("path")
            .attr("class", "contextDataPath_" + factor_id + "_" + metric_id)
            .attr("stroke", aDisplayColor[factor_id][metric_id])
            .attr("stroke-width", 1);

        paths.attr("d", function () { return line(currentData) });
        paths.exit().remove();

    }else{
        paths.remove();
    }
}

function updateFocusDataPoints(factor_id, metric_id){
    currentData = [];
    var groupNextLeft = {}, groupNextRight = {};
    var dataNextLeft = {}, dataNextRight = {};
    console.log("metricStatistics:", metricStatistics[factor_id][metric_id]);
	Object.keys(metricStatistics[factor_id][metric_id]).forEach(function(level){
        if(aDisplayList[factor_id][metric_id] != true){ return }
       
        current = metricStatistics[factor_id][metric_id][level];
        console.log("Level:", level, ", Domain:", dXScaleFocus.domain());
        if (level < dXScaleFocus.domain()[0]){
            if(groupNextLeft["xValue"] != undefined && groupNextLeft["xValue"] > current["xValue"]){ return }
            else{ groupNextLeft["xValue"] = current["xValue"]; groupNextLeft["medianValue"] = current["medianValue"] }

        }else if (level > dXScaleFocus.domain()[1]){
            console.log("compare:", groupNextRight, current);
            if(groupNextRight["xValue"] != undefined && groupNextRight["xValue"] < current["xValue"]){ return }
            else{ groupNextRight["xValue"] = current["xValue"]; groupNextRight["medianValue"] = current["medianValue"] }

        }else{
            currentData.push(metricStatistics[factor_id][metric_id][level] );
        }
	});

    currentData.sort(function(a, b){ return a["xValue"] - b["xValue"] });
    var leftMost = currentData.length > 0 ? currentData[0] : groupNextRight;
    var rightMost = currentData.length > 0 ? currentData[currentData.length - 1] : groupNextLeft;
    console.log("Left:", leftMost, ", Right:", rightMost, ", GroupL:", groupNextLeft, ", GroupR:", groupNextRight);

    if(groupNextLeft["xValue"] != undefined){
        var slope = (groupNextLeft["medianValue"] - leftMost["medianValue"]) / (groupNextLeft["xValue"] - leftMost["xValue"]);
        var dx = dXScaleFocus.domain()[0] - leftMost["xValue"];
        console.log("Group:", groupNextLeft, ", LeftMost:", leftMost, ", Slope:", slope, ", dx:", dx, ", dataNextLeft:", groupNextLeft, ", leftMost:", leftMost);
        dataNextLeft["xValue"] = dXScaleFocus.domain()[0];
        dataNextLeft["medianValue"] = leftMost["medianValue"] + dx * slope;
    }

    if(groupNextRight["xValue"] != undefined){
        var slope = (groupNextRight["medianValue"] - rightMost["medianValue"]) / (groupNextRight["xValue"] - rightMost["xValue"]);
        var dx = dXScaleFocus.domain()[1] - rightMost["xValue"];
        console.log("Group:", groupNextRight, ", RightMost:", rightMost, ", Slope:", slope, ", dx:", dx, ", dataNextLeft:", groupNextLeft, ", leftMost:", leftMost);
        dataNextRight["xValue"] = dXScaleFocus.domain()[1];
        dataNextRight["medianValue"] = rightMost["medianValue"] + dx * slope;
    }

    focusgroups = focus.selectAll(".focusDataGroup_" + factor_id + "_" + metric_id).data(currentData);
    focusgroups.enter().append("g").attr("class", "focusDataGroup_" + factor_id + "_" + metric_id);
    focusgroups.exit().remove();
    //console.log("Groups:", focusgroups);

    focuspoints = focusgroups.selectAll(".subpoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    focuspoints.enter()
        .append("circle")
        .attr("class", "subpoint")
        .attr("stroke", aDisplayColor[factor_id][metric_id] )
        .attr("stroke-width", 1.5)
        .attr("fill", "white")
        .attr("fill-opacity", 0)
        .attr("r", focusDataPointRadius)
        .on("mouseover", mouseOverDataPoint)
        .on("mouseout", mouseOutDataPoint);

    focuspoints
        .attr("cx", function (d, e, f) { return jMarginFocus.left + dXScaleFocus(currentData[f]["xValue"])})
        .attr("cy", function (d, e, f) { return jMarginFocus.top + dYScaleFocus(currentData[f][d]) })
        .attr("info", function (d, e, f) { return numberFormat(currentData[f]["xValue"]) + "<br/>" + numberFormat(currentData[f][d]) } );

    focuspoints.exit()
        .remove();

    focuslines = focusgroups.selectAll(".subline").data([""]);
    focuslines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke-dasharray":"5,5", "stroke":aDisplayColor[factor_id][metric_id] });
    focuslines
        .attr("x1", function (d, e, f) { return jMarginFocus.left + dXScaleFocus(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return jMarginFocus.left + dXScaleFocus(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return jMarginFocus.top + dYScaleFocus(currentData[f]["lowDevValue"]); })
        .attr("y2", function (d, e, f) { return jMarginFocus.top + dYScaleFocus(currentData[f]["highDevValue"]); });
    focuslines.exit().remove();

//=============================================
    line = d3.svg.line()
        .x(function (d) { return jMarginFocus.left + dXScaleFocus(d["xValue"]) })
        .y(function (d) { return jMarginFocus.top + dYScaleFocus(d["medianValue"]) });

    if (dataNextLeft.xValue != undefined) { currentData.unshift(dataNextLeft); }
    if (dataNextRight.xValue != undefined) { currentData.push(dataNextRight); }
    console.log(currentData);

    paths = focus.selectAll(".focusDataPath_" + factor_id + "_" + metric_id).data([""]);

    if (currentData.length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "focusDataPath_" + factor_id + "_" + metric_id)
            .attr("stroke", aDisplayColor[factor_id][metric_id])
            .attr("stroke-width", 1);

        paths.attr("d", function () { return line(currentData) });
        paths.exit().remove();

    }else{
        paths.remove();
    }

}

function addDisplayMetric(factor_id, metric_id){
    var eFactorSelect = document.getElementById("selectFactor")
    var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
    if(factor_id == sSelectedFactor){ d3.selectAll("#select_square_" + metric_id).style("background", aDisplayColor[factor_id][metric_id]); }

    aDisplayList[factor_id][metric_id] = true;
        
    if(metricStatistics[factor_id] == undefined){ metricStatistics[factor_id] = {} }
    if(metricStatistics[factor_id][metric_id] == undefined){
        var simruns = [];
        //console.log(levelList, factor_id);
        levelList[factor_id].forEach(function(level, levelIndex){
            console.log(level);
            readData("/safe/dp-query/?factor=" + factorList[factor_id].name + "&exp_id=" + exp_id + "&value=" + level, function(designPoints){
                console.log(level, designPoints);
                designPoints.forEach(function(designPoint, index){
                    readData("/safe/designpoint/" + designPoint.designpoint_id+ "/simruns/", function(runs){
                            simruns.push.apply(simruns, runs);
                            if(index >= designPoints.length - 1){
                                d3.json("/safe/metric/" + metric_id + "/results/").post(JSON.stringify(simruns), function(error, simruns){
                                    if(error){ console.warn(error) }

                                    calculateStatistics(factor_id, metric_id, level, simruns);
                                    // If statistics for all levels have been computed
                                    if(Object.keys(metricStatistics[factor_id][metric_id]).length >= levelList[factor_id].length){
                                        updateDataFeatures();
                                        updateScaling();
                                        displayData();
                                        updateKeyItems();
                                    }
                                  });
                            }
                        });
                    });
                });
        });
    }else{
        updateDataFeatures();
        updateScaling();
        displayData();
		updateKeyItems();
    }
}

// removes the specified key from the list of dependent variables (no longer displayed)
function removeDisplayMetric(factor_id, metric_id){
    var eFactorSelect = document.getElementById("selectFactor")
    var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
    if(factor_id == sSelectedFactor){ d3.selectAll("#select_square_" + metric_id).style("background", "white"); }

	d3.selectAll(".focusDataGroup_" + factor_id + "_" + metric_id).remove();
	d3.selectAll(".contextDataGroup_" + factor_id + "_" + metric_id).remove();
    d3.selectAll(".focusDataPath_" + factor_id + "_" + metric_id).remove();
    d3.selectAll(".contextDataPath_" + factor_id + "_" + metric_id).remove();
	
    delete aDisplayList[factor_id][metric_id];
    updateDataFeatures();
    updateScaling();
    displayData();
	updateKeyItems();
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);
    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}



// adjusts the scales base on current data values, dependent, and independent variables
function updateScaling(){
    var xValLeft, xValRight;
    range = xValMax - xValMin;
    if(range > 0){
        pad = 0.1 * range;
        xValLeft = xValMin - pad;
        xValRight = xValMax + pad;
    }
    else{
        pad = xValMax > 10 ? 10 : xValMax;
        xValLeft = xValMin - pad;
        xValRight = xValMin + pad;
    }

	dXScaleContext.range([0, sContextWidth]).domain([xValLeft,xValRight]);
	dYScaleContext.range([sContextHeight, 0]).domain([0,yValMax]);

	yAxisContext.scale(dYScaleContext);
	d3.select(".y.axis.context").transition().call(yAxisContext);

	xAxisContext.scale(dXScaleContext);
    d3.select(".x.axis.context").transition().call(xAxisContext);

    brushed();
}

// updates important information (such as min/max values)
function updateDataFeatures(){
    xValMin = -1;
    xValMax = -1;
    yValMin = -1;
    yValMax = -1;
    Object.keys(aDisplayList).forEach(function (factor_id) {
        Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
            var currentDisplayList = aDisplayList[factor_id][metric_id];
            console.log(aDisplayList, metricStatistics, factor_id, metric_id);
            if (currentDisplayList != undefined && currentDisplayList == true) {
                xValRange = d3.extent(levelList[factor_id]);
                xValMin = (xValMin < 0) ? xValRange[0] : d3.min([xValMin, xValRange[0]]);
                xValMax = (xValMax < 0) ? xValRange[1] : d3.max([xValMax, xValRange[1]]);
				Object.keys(metricStatistics[factor_id][metric_id]).forEach(function(level){
					var current = metricStatistics[factor_id][metric_id][level];
                    console.log(metricStatistics, factor_id, metricStatistics[factor_id], metric_id, metricStatistics[factor_id][metric_id], level, current);

                	yValMin = (yValMin) < 0 ? current["minValue"] : d3.min([yValMin, current["minValue"]]);
                	yValMax = (yValMax) < 0 ? current["maxValue"] : d3.max([yValMax, current["maxValue"]]);
					});            
			}
        });
    });
    if (d3.sum(aDisplayList, function(d){ return d.length } == 0))
        xValMin = xValMax = yValMin = yValMax = 0;
}

function brushed() {
    var extent = brush.extent();
    dXScaleFocus.domain(brush.empty() ? dXScaleContext.domain() : [ extent[0], extent [1] ]);
    dYScaleFocus.domain(dYScaleContext.domain());

    focus.select(".x.axis.focus").call(xAxisFocus);
    focus.select(".y.axis.focus").call(yAxisFocus);

    Object.keys(aDisplayList).forEach(function(factor_id){
          Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
                updateFocusDataPoints(factor_id, metric_id);
        });
    });
}

function updateCrossHairs(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xleft = dXScaleFocus.range()[0] + jMarginFocus.left; 
    var xright = dXScaleFocus.range()[1] + jMarginFocus.left;
    var ytop = dYScaleFocus.range()[0] + jMarginFocus.top;
    var ybottom = dYScaleFocus.range()[1] + jMarginFocus.top;

    if (x < xleft) { x = xleft }
    if (x > xright) { x = xright }
    if (y < ybottom) { y = ybottom }
    if (y > ytop) { y = ytop }

    horizontal = focus.selectAll(".crosshairhorizontal").data([""]);
    vertical = focus.selectAll(".crosshairvertical").data([""]);

    horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.enter().append("line").attr("class", "crosshairhorizontal").style("stroke", "black").style("pointer-events", "none")
        .attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.enter().append("line").attr("class", "crosshairvertical").style("stroke", "black").style("pointer-events", "none")
        .attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

function enableCrossHairs(){
    if(this.checked){
        focus.on("mousemove", updateCrossHairs);
    }else{
        focus.selectAll(".crosshairhorizontal").remove();
        focus.selectAll(".crosshairvertical").remove();
        focus.on("mousemove", undefined);
    }
}

function factorSelectChange(){
    //console.log("Factor changed...");
	eFactorSelect = document.getElementById("selectFactor");
	sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
	Object.keys(metricList).forEach(function(metric_id){
		d3.select("#select_square_" + metric_id).style("background", "white");
	});

	Object.keys(aDisplayList).forEach(function(factor_id){
		Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
			if(sSelectedFactor == factor_id){ 
				aDisplayList[factor_id][metric_id] = true;
				d3.select("#select_square_" + metric_id).style("background", aDisplayColor[factor_id][metric_id]);
			}
			else{ aDisplayList[factor_id][metric_id] = false;}
			});
		});

    updateDataFeatures();
    updateScaling();
    displayData();
	updateKeyItems();
}

function selectElementChange(){
     //console.log("Selected Element: ", this.getAttribute("metric_id"));
    
     var eFactorSelect = document.getElementById("selectFactor");
	 var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
     var sSelectedMetric = this.getAttribute("metric_id");

     if(aDisplayColor[sSelectedFactor] == undefined){ aDisplayColor[sSelectedFactor] = {} }
     if(aDisplayList[sSelectedFactor] == undefined) { aDisplayList[sSelectedFactor] = {} }

     if(aDisplayColor[sSelectedFactor][sSelectedMetric] == undefined) { aDisplayColor[sSelectedFactor][sSelectedMetric] = aColors.shift() }

     if(aDisplayList[sSelectedFactor][sSelectedMetric] == undefined){
         addDisplayMetric(sSelectedFactor, sSelectedMetric);
     }else if(aDisplayList[sSelectedFactor][sSelectedMetric] == true){
         removeDisplayMetric(sSelectedFactor, sSelectedMetric);
     }else if(aDisplayList[sSelectedFactor][sSelectedMetric] == false){
         aDisplayList[sSelectedFactor][sSelectedMetric] = true;
     }
}

function updateKeyItems(){
	var values = [];
	Object.keys(aDisplayList).forEach(function(factor_id){
		Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
			if(aDisplayList[factor_id][metric_id] == true)
				values.push({"factor_id":factor_id, "metric_id": metric_id});
		});		
	});

	values.sort(function(a, b){ return a.factor_id - b.factor_id });
	d3.select("#key").selectAll(".key_line").remove();

	var displayed = d3.select("#key").selectAll(".key_line").data(values)
			.enter()
			.append("div")
			.attr("class", "key_line")

    displayed.append("div")
        .attr("factor_id", function(d) {  return d.factor_id })
		.attr("metric_id", function(d) { return d.metric_id })
        .style("float", "left")
        .text("X")
        .on("click", displayRemove);

	displayed.append("div")
		.attr("class", "key_square")
		.style("background", function(d){ return aDisplayColor[d.factor_id][d.metric_id] });

	displayed.append("div")
		.attr("class", "key_label")
		.text(function(d) { return "Run: " + d.factor_id + ", Metric: " + d.metric_id });
}

function displayRemove(){
    //console.log("Removing ", this, "...");
    removeDisplayMetric(this.getAttribute("factor_id"), this.getAttribute("metric_id"));
}

function calculateStatistics(factor_id, metric_id, level, data){
	//console.log("Calculating Statistics...");
    console.log("Calculatng for:", factor_id, metric_id, level, "...");
    data.sort(function(a, b) { return a.value - b.value});
    data.forEach(function(d, i) { data[i] = d.value });
    
    if(metricStatistics[factor_id] == undefined){ metricStatistics[factor_id] = {} }
    if(metricStatistics[factor_id][metric_id] == undefined){ metricStatistics[factor_id][metric_id] = {} }
    if(metricStatistics[factor_id][metric_id][level] == undefined){ metricStatistics[factor_id][metric_id][level] = {} }
    metricStatistics[factor_id][metric_id][level]["xValue"] = level;
    metricStatistics[factor_id][metric_id][level]["minValue"] = d3.quantile(data, 0);
    metricStatistics[factor_id][metric_id][level]["lowDevValue"] = d3.quantile(data, 0.25);
    metricStatistics[factor_id][metric_id][level]["medianValue"] = d3.quantile(data, 0.50);
    metricStatistics[factor_id][metric_id][level]["highDevValue"] = d3.quantile(data, 0.75);
    metricStatistics[factor_id][metric_id][level]["maxValue"] = d3.quantile(data, 1);
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
readInitialData();
