/**
 * Functions for loading simulation runs from the data base
 * in various ways
 */

safe.loader.simruns = {};
safe.loader.simruns.byExperiment = safe_loader_simruns_byExperiment;
safe.loader.simruns.byDesignpoint = safe_loader_simruns_byDesignpoint;

/**
 * Retrieves all simulation runs associated with the specified experiment
 * Data of the form [{"dp_id":<designpoint_id>, "id", <simrun_id>}, ...]
 *
 * @param experimentId The experiment with which the simulation runs are associated
 * @param callBack A function to be called after the data has been read. "callBack(data)"
 */
function safe_loader_simruns_byExperiment(experimentId, callBack){
    d3.json("/safe/exp/" + experimentId + "/simruns/", function(error, data){
        if(error){ console.warn(error) }
        
        callBack(data);
    });
}

/**
 * Retrieves all simulation runs associated with the specified design point 
 * Data of the form [{"simrun_id":<simrun_id>}, ...]
 *
 * @param designpointId The design point with which the simulation runs are associated
 * @param callBack A function to be called after the data has been read. "callBack(data)"
 */
function safe_loader_simruns_byDesignpoint(designpointId, callBack){
    d3.json("/safe/designpoint/" + designpointId + "/simruns/", function(error, data){
        if(error){ console.warn(error) }
        
        callBack(data);
    });
}
