// The experiment currently being observed
var exp_id, colors;
var focusDataPointRadius = 2;
var contextDataPointRadius = 1.5;
var numberFormat = d3.format(".2f");

// add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

//----------------------------------------------
// Apply Factor Selection Object
// ---------------------------------------------

selectionFactorApplyNumber = 0;
SelectionFactorApply = function(exp_id, applyCallBack, parentDomId){
	this.exp_id = exp_id;
	this.callBack = applyCallBack;
	this.dParent = d3.select("#" + parentDomId);
	this.eParent = document.getElementById(parentDomId);
	this.id = "selectionFactorApply_" + selectionFactorApplyNumber;
	selectionFactorApplyNumber += 1; 

	this.dSelf = this.dParent.selectAll("#" + this.id).data([""]).enter().append("div").attr("id", this.id);
	console.log(this.dSelf);
	this.selectFactor = new SelectionFactor(this.exp_id, undefined, this.id);
	this.selectFactor.readInitialData(this.initComponents.bind(this));
}

SelectionFactorApply.prototype.initComponents = function(){
	this.selectFactor.initComponents(undefined, {width:"200px"});
	this.dSelf
		.append("button")
		.text("Add Factor Graph")
		.on("click", this.addFactorGraph.bind(this));
}

SelectionFactorApply.prototype.addFactorGraph = function(){
	this.callBack(this.getSelectedFactor());
}

SelectionFactorApply.prototype.getSelectedFactor = function(){
	return this.selectFactor.getSelectedFactor();
}

//----------------------------------------------
// Plot Area Selection Object
// ---------------------------------------------

selectionPlotAreaNumber = 0;
SelectionPlotArea = function(exp_id, factor, selectionCallBack, parentDomId){
	this.exp_id = exp_id;
	this.factor = factor;
	this.selectionCallBack = selectionCallBack;
	this.id = "selectionPlotArea_" + selectionPlotAreaNumber;
	selectionPlotAreaNumber += 1;

	this.dSelf = d3.select("#" + parentDomId).append("div")
        .attr("id", this.id)
        .attr("class", "plot_area_selection");
	this.eSelf = document.getElementById(this.id);
	
	this.selectLevelMin = new SelectionLevel(this.exp_id, this.levelSelectChange.bind(this), this.id);
	this.selectLevelMax = new SelectionLevel(this.exp_id, this.levelSelectChange.bind(this), this.id);
	this.selectDesignPoint = new SelectionDesignPoint(this.exp_id, this.designPointSelectChange.bind(this), this.id);
	this.selectRun = new SelectionRun(this.exp_id, this.runSelectChange.bind(this), this.id);
	this.selectMetric = new SelectionMetric(this.exp_id, this.selectElementChange.bind(this), this.id);
	
	this.selectLevelMin.readInitialData(function(){
		this.selectLevelMax.readInitialData(function(){
			this.selectMetric.readInitialData(this.initComponents.bind(this))
		}.bind(this));
	}.bind(this));
}

SelectionPlotArea.prototype.initComponents = function(){
	this.selectLevelMin.initComponents(undefined, {width:"50%"});
	this.selectLevelMax.initComponents(undefined, {width:"50%"});

    this.selectDesignPoint.initComponents();
    this.selectRun.initComponents();
    this.selectMetric.initComponents();
      
	this.selectLevelMin.updateLevels(this.factor.id);
	this.selectLevelMax.updateLevels(this.factor.id);

    this.selectLevelMax.eSelect.selectedIndex = this.selectLevelMax.eSelect.options.length - 1;

    this.updateDesignPoints();
}

SelectionPlotArea.prototype.levelSelectChange = function(){
	console.log("levelSelectChange");
	var min = this.selectLevelMin.getSelectedLevel();
	var max = this.selectLevelMax.getSelectedLevel();

	if(min > max){
        alert("The factor's beginning level cannot be greater than the factor's ending level");
		return;
    }

    var parameter = {"name":this.factor.name, "factor_id":this.factor.id, "min_value":min, "max_value":max };
	this.selectDesignPoint.addSelectionParameter(this.factor.id, parameter);
	this.updateDesignPoints();
}

SelectionPlotArea.prototype.updateDesignPoints= function(){
	this.selectDesignPoint.updateDesignPoints(this.designPointSelectChange.bind(this));	
}

SelectionPlotArea.prototype.designPointSelectChange = function(){
	this.selectRun.updateRuns(this.selectDesignPoint.getSelectedDesignPoint(), function(){
		this.selectMetric.setDistinctionId(this.selectRun.getSelectedRun());
		this.selectMetric.updateSelectedMetrics();
	}.bind(this));
}

SelectionPlotArea.prototype.runSelectChange = function(){
	this.selectMetric.setDistinctionId(this.selectRun.getSelectedRun());
	this.selectMetric.updateSelectedMetrics();
}

SelectionPlotArea.prototype.addDisplayed = function(run_id, metric_id, color){
    this.selectMetric.setMetricColor(run_id, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

SelectionPlotArea.prototype.removeDisplayed = function(run_id, metric_id){
    this.selectMetric.setMetricColor(run_id, metric_id, "white");
    this.selectMetric.updateSelectedMetrics();
}

SelectionPlotArea.prototype.selectElementChange = function(metric){
     this.selectionCallBack(this.selectRun.getSelectedRun(), metric);
}


//----------------------------------------------
// Plot Area Object
// ---------------------------------------------

metricPlotAreaNumber = 0;
MetricPlotArea = function(exp_id, factor, parentDomId){
	this.exp_id = exp_id;
    this.factor = factor;
	this.dParent = d3.select("#" + parentDomId);
	this.eParent = document.getElementById(parentDomId);
    this.margin = {top: 10, right: 40, bottom: 20, left: 40};
	this.id = "plotArea_" + metricPlotAreaNumber;
	metricPlotAreaNumber += 1;

	this.colorList = {};		// list of colors by run_id and metric_id
	this.displayList = {};		// list of metrics being displayed by run_id and metric_id
	this.dataList = {};			// list of data that has been loaded by run_id and metric_id
	
    this.graphWidth = undefined;
    this.graphHeight = 150;
	this.plotList = {};
    this.initComponents();
}

MetricPlotArea.prototype.selectionChange = function(run_id, metric){
    metric_id = metric.id;
	if(this.colorList[run_id] == undefined){ this.colorList[run_id] = {} }
	if(this.displayList[run_id] == undefined) { this.displayList[run_id] = {} }

	if(this.colorList[run_id][metric_id] == undefined) { this.colorList[run_id][metric_id] = colors.shift() }

	if(this.displayList[run_id][metric_id] == undefined){
		this.addDisplayMetric(run_id, metric);
	}else{
		this.removeDisplayMetric(run_id, metric.id);
	}
}

MetricPlotArea.prototype.addDisplayMetric = function(run_id, metric){
    metric_id = metric.id;
    this.displayList[run_id][metric_id] = true;

    var id = run_id + "_" + metric_id;

	var update = function(){
        this.selection.addDisplayed(run_id, metric_id, this.colorList[run_id][metric_id]);
        this.plotList[id].displayAll();

        var displayString = "Run " + run_id + ", Metric: " + metric_id;
        var color = this.colorList[run_id][metric_id];
        var returnAttributes = {"run_id":run_id, "metric_id":metric_id };

        this.displayKey.addElement(id, displayString, color, returnAttributes)
        this.displayKey.updateElements();
    }.bind(this)


    if(this.dataList[run_id] == undefined){ this.dataList[run_id] = {} }
    if(this.dataList[run_id][metric_id] == undefined){
        this.plotList[id] = new ScatterMagnitude(this.margin, this.graphWidth, this.graphHeight, this.id + "_graph_area");
        d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/", function(error, data){
            if(error){ console.warn(error) }
        
            this.dataList[run_id][metric_id] = true;

            pointAttributes = {"stroke": this.colorList[run_id][metric_id], 
                               "stroke-width": 1.5, 
                               "fill": "white", 
                               "fill-opacity": 0, 
                               "r": focusDataPointRadius};
            
            pointMouseEvent = {"mouseover": mouseOverDataPoint,
                               "mouseout": mouseOutDataPoint };

            //context.addDataSet(id, data, pointAttributes); 

            pointAttributes["info"] = function(d) { return numberFormat(d.time) + "<br/>" + numberFormat(d.value) };
            this.plotList[id].addDataSet(id, data, pointAttributes, pointMouseEvent);
            this.plotList[id].setYAxisLabel(metric.name + "(" + metric.units + ")");
            this.plotList[id].setXAxisLabel("Time (seconds)");

			update();
        }.bind(this));
    }else{
        this.plotList[id].setVisible(id);
		update();
    }

}

MetricPlotArea.prototype.removeMetricFromDisplay = function(attributes){
    this.removeDisplayMetric(attributes.run_id, attributes.metric_id);
}

MetricPlotArea.prototype.removeDisplayMetric = function(run_id, metric_id){
    console.log("Removing:", run_id, metric_id);
    id = run_id + "_" + metric_id;

    this.displayKey.removeElement(id);
    this.selection.removeDisplayed(run_id, metric_id);
    this.plotList[id].svg.remove();

    delete this.dataList[run_id][metric_id];
    delete this.displayList[run_id][metric_id];
    delete this.plotList[id];
}

MetricPlotArea.prototype.initComponents = function(){
	this.dSelf = this.dParent.selectAll("#" + this.id).data([""]).enter()
		.append("div")
        .attr("id", this.id)
        .attr("class", "plot_area");
	this.eSelf = document.getElementById(this.id);

    this.dSelf
        .append("div")
        .attr("class", "plot_area_label")
        .text(this.factor.name);

    this.dSelf
        .append("div")
        .attr("class", "plot_selection_label")
        .text("Select");

    this.dSelf
      .append("div")
      .attr("height", this.graphHeight + "px")
      .attr("class", "graph_area")
      .attr("id", this.id + "_graph_area");
    this.graphWidth = document.getElementById(this.id + "_graph_area").offsetWidth;
    
	this.selection = new SelectionPlotArea(this.exp_id, this.factor, this.selectionChange.bind(this), this.id);
    this.displayKey = new DisplayElements(this.removeMetricFromDisplay.bind(this) , this.id);
}

//======================================================

function initVisualComponents(){
    colors = ["blue", "green", "orange", "purple", "black", "red"];
	new SelectionFactorApply(exp_id, addFactorPlotArea, "select_factor");
}

function addFactorPlotArea(factor){
	console.log("Factor Changed: ", factor);
	new MetricPlotArea(exp_id, factor, "place_graphs");
    d3.select("#place_graphs").append("div").attr("class", "graph_sep");
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);

    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

function brushed() {
    var extent = brush.extent();
    xDomain = brush.empty() ? context.dXScale.domain() : [ extent[0][0], extent [1][0] ];
    yDomain = brush.empty() ? context.dYScale.domain() : [ extent[0][1], extent [1][1] ];

   focus.updateDomains(xDomain, yDomain); 
   focus.displayAll();
}

function enableCrossHairs(){
    if(this.checked){
        focus.enableCrossHairs();
    }else{
        focus.disableCrossHairs()
    }
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
