var brush, selection, displayParameters;
// Our data, this will need to change
var metricList, metricData, aDisplayList;
// The name of the independent variable / dependent variables (Array)
var aColorList, aColors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var exp_id;



// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
	jMarginFocus = {top: 10, right: 40, bottom: 40, left: 40};
	jMarginContext = { top: 10, right: jMarginFocus.right, bottom: 20, left: 40 };

    focus = new ScatterMagnitude(jMarginFocus, 800, 500, undefined, "chart1");
    context = new ScatterMagnitude(jMarginContext, 500, 500, undefined, "chart2");

    focus.setXAxisLabel("Time (Seconds)");
    focus.setYAxisLabel("Queue Size (Packets)");

    context.setYAxisLabel("");
    brush = d3.svg.brush() 
        .x(context.dXScale)
        .y(context.dYScale)
        .on("brush", brushed);

    context.svg.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + jMarginContext.left + "," + jMarginContext.top + ")")
        .call(brush);

    // Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	aDisplayColor = {};
    aDisplayList = {};
	metricData = {};
    aColors = ["blue", "green", "orange", "purple", "black", "red"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
    numberFormat = d3.format(".2f");
}

// Initializes the selection for the graph
function initDataComponents(){
    var removeFromDisplay = function(attributes){
        removeDisplayMetric(attributes.run_id, attributes.metric_id);
    }

    var removeFromParameters = function(attributes){
        selection.removeSelectionFilter(attributes.factor_id);
        displayParameters.removeElement(attributes.factor_id);
    }

    var addToParameters = function(parameter){
        var key = parameter.factor_id;
        var elementString = "Factor: " + parameter.name + ", Range: " + parameter.min_value + " - " + parameter.max_value;
        var returnAttributes = {"factor_id": parameter.factor_id };
        displayParameters.addElement(key, elementString, returnAttributes);
    }

    selection = new SelectionFull(exp_id, selectionChange, undefined, "select");
    displayKey = new DisplayElements(removeFromDisplay, "key");
    displayParameters = new SelectionParameters( removeFromParameters, "parameters");

    selection.init(exp_id, selectionChange, addToParameters, "select");

    // Initialize the cross hairs option
    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

function selectionChange(run_id, metric){
    metric_id = metric.id; 
    if(aDisplayColor[run_id] == undefined){ aDisplayColor[run_id] = {} }
    if(aDisplayList[run_id] == undefined) { aDisplayList[run_id] = {} }

    if(aDisplayColor[run_id][metric_id] == undefined) { aDisplayColor[run_id][metric_id] = aColors.shift() }

    if(aDisplayList[run_id][metric_id] == undefined){
        addDisplayMetric(run_id, metric);
    }else{
        removeDisplayMetric(run_id, metric_id);
    }
}

function addDisplayMetric(run_id, metric){
    var metric_id = metric.id;
    aDisplayList[run_id][metric_id] = true;
    var id = run_id + "_" + metric_id;

    var update = function(){
        selection.addDisplayed(run_id, metric_id, aDisplayColor[run_id][metric_id]);
        focus.displayAll();
        context.displayAll();
        var displayString = "Run " + run_id + ", Metric: " + metric.name;
        var color = aDisplayColor[run_id][metric_id];
        var returnAttributes = {"run_id":run_id, "metric_id":metric_id };
        displayKey.addElement(id, displayString, color, returnAttributes)
        displayKey.updateElements();
    }
        
    if(metricData[run_id] == undefined){ metricData[run_id] = {} }
    if(metricData[run_id][metric_id] == undefined){
        d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/", function(error, data){
            if(error){ console.warn(error) }

            var pointAttributes = 
                {"stroke": aDisplayColor[run_id][metric_id], 
                 "stroke-width": 1.5, 
                 "fill": "white", 
                 "fill-opacity": 0, 
                 "r": focusDataPointRadius,
                 "info": function(d) { return numberFormat(d.time) + "<br/>" + numberFormat(d.value) }};

            var pointMouseEvent = 
                {"mouseover": mouseOverDataPoint,
                 "mouseout": mouseOutDataPoint };

            metricData[run_id][metric_id] = true;
            context.addDataSet(id, data, pointAttributes); 
            focus.addDataSet(id, data, pointAttributes, pointMouseEvent);
            update();
         });
    }else{
        focus.setVisible(id);
        context.setVisible(id);
        update();
    }
}


// removes the specified key fro the list of dependent variables (no longer displayed)
function removeDisplayMetric(run_id, metric_id){
    delete aDisplayList[run_id][metric_id];
    selection.removeDisplayed(run_id, metric_id);

    id = run_id + "_" + metric_id;
    displayKey.removeElement(id);
	displayKey.updateElements();

    focus.setInvisible(id); 
    context.setInvisible(id);
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);

    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

function brushed() {
    var extent = brush.extent();
    xDomain = brush.empty() ? context.dXScale.domain() : [ extent[0][0], extent [1][0] ];
    yDomain = brush.empty() ? context.dYScale.domain() : [ extent[0][1], extent [1][1] ];

   focus.updateDomains(xDomain, yDomain); 
   focus.displayAll();
}

function enableCrossHairs(){
    if(this.checked){
        focus.enableCrossHairs();
    }else{
        focus.disableCrossHairs()
    }
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
initDataComponents();
