// Layout components
var marginFocus, focusWidth, contextWidth, focusHeight, contextHeight;
// Scaling 
var xScaleFocus, yScaleFocus, xScaleContext, yScaleContext;
// Axes to be used for the graph
var xAxisFocus, yAxisFocus, xAxisContext, yAxisContext;
// Areas where things will be graphed
var svg, context, focus, brush;
// Our data, this will need to change
var metricList, metricData, metricTypes;
// Inportant details about the data
var xValMin, xValMax, yValMin, yValMax;
// The name of the independent variable / dependent variables (Array)
var xKey, yKey, yKeyColors, colors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var srun_id = 1;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
	marginFocus = {top: 10, right: 40, bottom: 20, left: 40};
	marginContext = { top: 10, right: marginFocus.right, bottom: 20, left: 40 };
	focusWidth = 960 - marginFocus.left - marginFocus.right;
	focusHeight = 500 - marginFocus.top - marginFocus.bottom;
	contextWidth = 500 - marginContext.left - marginContext.right;
	contextHeight = focusHeight;

	// Configure time scales
	xScaleFocus = d3.scale.linear().range([0, focusWidth]).domain([0,0]).clamp(true);
	yScaleFocus = d3.scale.linear().range([focusHeight, 0]).domain([0,0]).clamp(true);
	xScaleContext = d3.scale.linear().range([0, contextWidth]).domain([0, 0]);
	yScaleContext = d3.scale.linear().range([contextHeight, 0]).domain([0, 0]);

	// Configure axis
	xAxisFocus = d3.svg.axis().scale(xScaleFocus).orient("bottom");
	yAxisFocus = d3.svg.axis().scale(yScaleFocus).orient("left");
	xAxisContext = d3.svg.axis().scale(xScaleContext).orient("bottom");
	yAxisContext = d3.svg.axis().scale(yScaleContext).orient("left");

    // Add focus plot
	focus = d3.select("#chart1")
        .append("svg")
        .attr("width", focusWidth + marginFocus.left + marginFocus.right)
        .attr("height", focusHeight + marginFocus.top + marginFocus.bottom + 30);

    // Add context plot
    context = d3.select("#chart2")
        .append("svg")
        .attr("width", contextWidth + marginContext.left + marginContext.right)
        .attr("height", contextHeight + marginContext.top + marginContext.bottom + 30)

	// Text label for x axis
	focus.append("text")
        .attr("class", "x label")
		.attr("x", focusWidth / 2)
		.attr("y", focusHeight + marginFocus.top + marginFocus.bottom)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	focus.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((focusHeight) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	focus.append("g")
		.attr("class", "x axis focus")
		.attr("transform", "translate(" + marginFocus.left + "," + (focusHeight + marginFocus.top) + ")")
		.call(xAxisFocus);

	context.append("g")
        .attr("class", "x axis context")
        .attr("transform", "translate(" + marginContext.left + "," + (contextHeight + marginContext.top) + ")")
        .call(xAxisContext);

	// Add y axis
    focus.append("g")
		.attr("class", "y axis focus")
		.attr("transform", "translate(" + marginFocus.left + "," + marginFocus.top + ")")
		.call(yAxisFocus);

    context.append("g")
		.attr("class", "y axis context")
		.attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
		.call(yAxisContext);

    brush = d3.svg.brush() 
        .x(xScaleContext)
        .y(yScaleContext)
        .on("brush", brushed);

    context.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
        .call(brush);

    // Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	yKey = [];
	xKey = null;
	yKeyColors = [];
	metricData = [];
    colors = ["blue", "green", "orange"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
    numberFormat = d3.format(".2f");
}

// Initializes the key for the graph
function initGraphKey(){
	metricTypeList = d3.set([]);
	Object.keys(metricList).forEach(function (key) {
	    d = metricList[key];
	    if (!metricTypeList.has(d.metric_typeA)) {
	        metricTypeList.add(d.metric_typeA);
	    }
	    if (!metricTypeList.has(d.metric_typeB)) {
	        metricTypeList.add(d.metric_typeB);
	    }
	});

    // Add a selector for the x-axis (independent variable)
	var select = d3.select("#key")
		.append("select")
		.attr("id", "xAxisSelect")
		.on("change", xAxisSelectChange);

	// Populate the selector
	select.selectAll("option")
		.data(metricTypeList.values())
		.enter()
			.append("option")
			.text(function(d) { return d; });

    // Initialize independent variable
	xAxisSelectChange.call(document.getElementById("xAxisSelect"));

	// Add a new key line for each metric, binding its metric_id to id
	var key_items = d3.select("#key") 
		.selectAll("div") 
		.data(Object.keys(metricList)) 
		.enter()
			.append("div") 
			.attr("class","key_line") 
			.attr("metric_id", function(d){ return metricList[d].metric_id })
			.attr("id",function(d){return "key_" + metricList[d].metric_id});

	// Add a new key square for each metric
	key_items.append("div") 
		.attr("id", function(d){ return "key_square_" + metricList[d].metric_id}) 
		.attr("class", "key_square");

	// Add a label for each metric
	key_items.append("div")
		.attr("class","key_label")
        .attr("id", function(d){ return "key_label_" + metricList[d].metric_id }) 
		.text(function(d){return metricList[d].metric_name});

	// Listen for clicks to turn on/off the time series
	d3.selectAll(".key_line")
		.on("click", keyElementSelectChange);

    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

// Reads metric information (currently from file)
function readInitialData(){
    d3.json("../data/simrun=" + srun_id + ".txt", function (error, json) {
        if (error) return console.warn(error);

        metricList = [];
        metricTypes = [];
        json.forEach(function (metric) {
            metricList[metric.metric_id] = metric;
            if (!(metric.typeA in metricTypes)) { metricTypes.push(metric.typeA); }
            if (!(metric.typeB in metricTypes)) { metricTypes.push(metric.typeB); }
        });

        initGraphKey();
        updateDataFeatures();
        updateFromIndependentVariable();
        displayData();
    });
}

// Reads metric information (currently from file)
function readMetric(metric_id){
    d3.json("../data/simrun=" + srun_id + "&metric=" + metric_id + ".txt", function (error, json) {
        if (error) {
            removeDependentVariable(metric_id);
            return console.warn(error);
        }
        metricData[metric_id] = json;

        updateDataFeatures();
        adjustScaling();
        displayData();
    });
}

// Displays the currently held data to the graph
function displayData(){
	// Can't draw until data is specified
	if(xKey == null || yKey.length == 0){	
		return;
	}

	yKey.forEach(function (key) {
        updateContextDataPoints(key);
	});
}

// Updates the positions of the center points described by the data groupings
function updateContextDataPoints(metric_id){
    contextPoints = context.selectAll(".contextDataPoint_" + metric_id).data(metricData[metric_id]);

    var currentYKey;
    if(metricList[metric_id].metric_typeA == xKey){ currentYKey = metricList[metric_id].metric_typeB; }
    else{ currentYKey = metricList[metric_id].metric_typeA; }


    contextPoints.enter()
        .append("circle")
        .attr("class", "contextDataPoint_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("fill", "none")
        .attr("r", contextDataPointRadius);

    contextPoints.transition()
        .attr("cx", function (d) { return marginContext.left + xScaleContext(d[xKey]) })
        .attr("cy", function (d) { return marginContext.top + yScaleContext(d[currentYKey]) });

	contextPoints.exit()
        .remove();	
}

function updateFocusDataPoints(metric_id){
    currentData = [];

    var currentYKey;
    if(metricList[metric_id].metric_typeA == xKey){ currentYKey = metricList[metric_id].metric_typeB; }
    else{ currentYKey = metricList[metric_id].metric_typeA; }

    metricData[metric_id].forEach(function (d) {
        if (xScaleFocus.domain()[0] <= d[xKey] && d[xKey] <= xScaleFocus.domain()[1])
            if (yScaleFocus.domain()[0] <= d[currentYKey] && d[currentYKey] <= yScaleFocus.domain()[1])
                currentData.push(d);
    })

    focusPoints = focus.selectAll(".focusDataPoint_" + metric_id).data(currentData);

    focusPoints
        .attr("cx", function (d) { return marginFocus.left + xScaleFocus(d[xKey]) })
        .attr("cy", function (d) { return marginFocus.top + yScaleFocus(d[currentYKey]) })
        .attr("info", function (d) { return d[xKey] + "<br/>" + d[currentYKey] } );

    focusPoints.enter()
        .append("circle")
        .attr("class", "focusDataPoint_" + metric_id)
        .attr("stroke", yKeyColors[metric_id] )
        .attr("stroke-width", 1.5)
        .attr("fill", "white")
        .attr("fill-opacity", 0)
        .attr("r", focusDataPointRadius)
        .attr("cx", function (d) { return marginFocus.left + xScaleFocus(d[xKey]) })
        .attr("cy", function (d) { return marginFocus.top + yScaleFocus(d[currentYKey]) })
        .attr("info", function (d) { return d[xKey] + "<br/>" + d[currentYKey] } )
        .on("mouseover", mouseOverDataPoint)
        .on("mouseout", mouseOutDataPoint);

    focusPoints.exit()
        .remove();
}

// Called when a new dependent variable is selected. Updates required data
function xAxisSelectChange(){
    setIndependentVariable(this.options[this.selectedIndex].value);
    updateDataFeatures();
    updateFromIndependentVariable();
    adjustScaling();
    displayData();
}

// Called when a key element is selected. Updates required data
function keyElementSelectChange(){
	metric_id = this.getAttribute("metric_id");
	index = yKey.indexOf(metric_id);
	if(index < 0){
	    if (addDependentVariable(metric_id)) {
	        var d = document.getElementById("key_square_" + metric_id)
                .style.background = yKeyColors[metric_id];
	    }
	}else{
		removeDependentVariable(metric_id);
		document.getElementById("key_square_" + metric_id)
            .style.background = "white";
	}
}

// Called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);      
    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// Called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

// Sets the current independent variable
function setIndependentVariable(name){
	xKey = name;
	d3.select(".x.label")
        .text(xKey);
}

// Adds the given key to the list to be graphed
function addDependentVariable(name){
    if (!(metricList[name].metric_typeA == xKey) && !(metricList[name].metric_typeB == xKey))
        return false;
	yKey.push(name);
    if(yKeyColors[name] == undefined){
        readMetric(name);
        yKeyColors[name] = colors.shift();
    }else{
        updateDataFeatures();
	    adjustScaling();
	    displayData();
    }
    return true;
}

function updateFromIndependentVariable(){
    yKey.forEach(function (metric_id) {
        if (metricList[metric_id].metric_typeA != xKey && metricList[metric_id].metric_typeB != xKey) {
            removeDependentVariable(metric_id);
            document.getElementById("key_square_" + metric_id)
                .style.background = "white";
        } 
    });
        Object.keys(metricList).forEach(function (key) {
            metric = metricList[key];
            if (metric.metric_typeA != xKey && metric.metric_typeB != xKey) {
                d3.select("#key_label_" + metric.metric_id)
                    .style("color", "rgb(200, 200, 200)");
            } else {
                d3.select("#key_label_" + metric.metric_id)
                    .style("color", "black");
            }
        });
}

// Removes the specified key fro the list of dependent variables (no longer displayed)
function removeDependentVariable(key){
	index = yKey.indexOf(key);
	yKey.splice(index, 1);
	context.selectAll(".contextDataPoint_" + key).remove();
	focus.selectAll(".focusDataPoint_" + key).remove();
	updateDataFeatures();
	adjustScaling();
	displayData();
}

// Adjusts the scales base on current data values, dependent, and independent variables
function adjustScaling(){
	xScaleContext.range([0, contextWidth]).domain([0,xValMax]);
	yScaleContext.range([contextHeight, 0]).domain([0,yValMax]);

	yAxisContext.scale(yScaleContext);
	d3.select(".y.axis.context").transition().call(yAxisContext);

	xAxisContext.scale(xScaleContext);
    d3.select(".x.axis.context").transition().call(xAxisContext);

    brushed();
}

// Updates important information (such as min/max values)
function updateDataFeatures(){
    xValMin = -1;
    xValMax = -1;
    yValMin = -1;
    yValMax = -1;
    yKey.forEach(function (metric_id) {
        var currentYKey;
        if (xKey == metricList[metric_id].metric_typeA) { currentYKey = metricList[metric_id].metric_typeB; }
        if (xKey == metricList[metric_id].metric_typeB) { currentYKey = metricList[metric_id].metric_typeA; }
        if (currentYKey != undefined) {
            xValRange = d3.extent(metricData[metric_id], function (d) { return d[xKey]; });
            xValMin = (xValMin < 0) ? xValRange[0] : d3.min([xValMin, xValRange[0]]);
            xValMax = (xValMax < 0) ? xValRange[1] : d3.max([xValMax, xValRange[1]]);

            yValRange = d3.extent(metricData[metric_id], function (d) { return d[currentYKey]; });
            yValMin = (yValMin) < 0 ? yValRange[0] : d3.min([yValMin, yValRange[0]]);
            yValMax = (yValMax) < 0 ? yValRange[1] : d3.max([yValMax, yValRange[1]]);
        }
    });
    if (yKey.length == 0)
        xValMin = xValMax = yValMin = yValMax = 0;
}

function brushed() {
    var extent = brush.extent();
    xScaleFocus.domain(brush.empty() ? xScaleContext.domain() : [ extent[0][0], extent [1][0] ]);
    yScaleFocus.domain(brush.empty() ? yScaleContext.domain() : [ extent[0][1], extent [1][1] ]);

    focus.select(".x.axis.focus").call(xAxisFocus);
    focus.select(".y.axis.focus").call(yAxisFocus);

    yKey.forEach(function (metric_id) {
        updateFocusDataPoints(metric_id);
    });
}

function updateCrossHairs(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xLeft = xScaleFocus.range()[0] + marginFocus.left; 
    var xRight = xScaleFocus.range()[1] + marginFocus.left;
    var yTop = yScaleFocus.range()[0] + marginFocus.top;
    var yBottom = yScaleFocus.range()[1] + marginFocus.top;

    if (x < xLeft) { x = xLeft }
    if (x > xRight) { x = xRight }
    if (y < yBottom) { y = yBottom }
    if (y > yTop) { y = yTop }

    horizontal = focus.selectAll(".crossHairHorizontal").data([""]);
    vertical = focus.selectAll(".crossHairVertical").data([""]);

    horizontal.attr("x1", xLeft).attr("x2", xRight).attr("y1", y).attr("y2", y);
    vertical.attr("y1", yBottom).attr("y2", yTop).attr("x1", x).attr("x2", x);

    horizontal.enter().append("line").attr("class", "crossHairHorizontal").style("stroke", "black").style("pointer-events", "none")
        .attr("x1", xLeft).attr("x2", xRight).attr("y1", y).attr("y2", y);
    vertical.enter().append("line").attr("class", "crossHairVertical").style("stroke", "black").style("pointer-events", "none")
        .attr("y1", yBottom).attr("y2", yTop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

function enableCrossHairs(){
    if(this.checked){
        focus.on("mousemove", updateCrossHairs);
    }else{
        focus.selectAll(".crossHairHorizontal").remove();
        focus.selectAll(".crossHairVertical").remove();
        focus.on("mousemove", undefined);
    }
}

initVisualComponents();
readInitialData();
