var brush, selection;
var jMarginFocus, sFocusWidth, sFocusHeight, sFocusDomTag;
// Our data, this will need to change
var designPointList, factorList,  runList, metricList, metricData, aDisplayList, levelList, parameterList;
// The name of the independent variable / dependent variables (Array)
var aColorList, aColors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var exp_id;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
	jMarginFocus = {top: 10, right: 40, bottom: 20, left: 40};
	jMarginContext = { top: 10, right: jMarginFocus.right, bottom: 20, left: 40 };
    
    sFocusHeight = 150;
    sFocusWidth = 960;
    sFocusDomTag = "chart1";

    context = new ScatterMagnitude(jMarginContext, 960, 50, "chart2");

    brush = d3.svg.brush() 
        .x(context.dXScale)
        .on("brush", brushed);

    context.svg.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + jMarginContext.left + "," + jMarginContext.top + ")")
        .call(brush)
        .selectAll("rect")
            .attr("y", -6)
            .attr("height", context.height + 7);


    // add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	aDisplayColor = {};
    aDisplayList = {};
	metricData = {};
    aColors = ["blue", "green", "orange", "purple", "black", "red"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
	parameterList = {};
    numberFormat = d3.format(".2f");
}

// Initializes the selection for the graph
function initGraphSelect(){
    selection = new SelectionFull(exp_id, selectionChange, updateSelectionItems, "select");

    // Initialize the cross hairs option
    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

function selectionChange(run_id, metric_id){
     if(aDisplayColor[run_id] == undefined){ aDisplayColor[run_id] = {} }
     if(aDisplayList[run_id] == undefined) { aDisplayList[run_id] = {} }

     if(aDisplayColor[run_id][metric_id] == undefined) { aDisplayColor[run_id][metric_id] = aColors.shift() }

     if(aDisplayList[run_id][metric_id] == undefined){
         addDisplayMetric(run_id, metric_id);
     }else{
         removeDisplayMetric(run_id, metric_id);
     }
}

function addDisplayMetric(run_id, metric_id){
    console.log("Adding:", run_id, metric_id);
    aDisplayList[run_id][metric_id] = true;

    var id = run_id + "_" + metric_id;

    if(metricData[run_id] == undefined){ metricData[run_id] = {} }
    if(metricData[run_id][metric_id] == undefined){
        focus[id] = new ScatterMagnitude(jMarginFocus, 960, 150, "chart1");
        d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/", function(error, data){
            if(error){ console.warn(error) }
        
            metricData[run_id][metric_id] = true;

            pointAttributes = {"stroke": aDisplayColor[run_id][metric_id], 
                               "stroke-width": 1.5, 
                               "fill": "white", 
                               "fill-opacity": 0, 
                               "r": focusDataPointRadius};
            
            pointMouseEvent = {"mouseover": mouseOverDataPoint,
                               "mouseout": mouseOutDataPoint };

            context.addDataSet(id, data, pointAttributes); 
            //context.displayAll();

            pointAttributes["info"] = function(d) { return numberFormat(d.time) + "<br/>" + numberFormat(d.value) };
            focus[id].addDataSet(id, data, pointAttributes, pointMouseEvent);
			focus[id].displayAll();

            brushed();
            selection.addDisplayed(run_id, metric_id, aDisplayColor[run_id][metric_id]);
            updateKeyItems();
        });
    }else{
        console.log(run_id, metric_id);
        focus[id].setVisible(id);
        //context.setVisible(id);
        selection.addDisplayed(run_id, metric_id, aDisplayColor[run_id][metric_id]);
		updateKeyItems();
    }
}

// removes the specified key fro the list of dependent variables (no longer displayed)
function removeDisplayMetric(run_id, metric_id){
    delete aDisplayList[run_id][metric_id];
    delete metricData[run_id][metric_id];
	
    selection.removeDisplayed(run_id, metric_id);
    //focus[run_id + "_" + metric_id].setInvisible(run_id + "_" + metric_id); 
    focus[run_id + "_" + metric_id].svg.remove();
    delete focus[run_id + "_" + metric_id];
    context.setInvisible(run_id + "_" + metric_id);

	updateKeyItems();
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);

    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

function brushed() {
    var extent = brush.extent();
    xDomain = brush.empty() ? context.dXScale.domain() : [ extent[0], extent [1] ];

    Object.keys(focus).forEach(function(id){
        focus[id].updateDomains(xDomain); 
        focus[id].displayAll();
    });
}

function enableCrossHairs(){
    if(this.checked){
        Object.keys(focus).forEach(function(id){
            focus[id].enableCrossHairs();
        });
    }else{
        Object.keys(focus).forEach(function(id){
            focus[id].disableCrossHairs()
        });
    }
}



function updateKeyItems(){
	var values = [];
	Object.keys(aDisplayList).forEach(function(run_id){
		Object.keys(aDisplayList[run_id]).forEach(function(metric_id){
			values.push({"run_id":run_id, "metric_id": metric_id});
		});		
	});

	values.sort(function(a, b){ return a.run_id - b.run_id });
	d3.select("#key").selectAll(".key_line").remove();

	var displayed = d3.select("#key").selectAll(".key_line").data(values)
			.enter()
			.append("div")
			.attr("class", "key_line")

    displayed.append("div")
        .attr("run_id", function(d) { return d.run_id })
		.attr("metric_id", function(d) { return d.metric_id })
        .style("float", "left")
        .text("X")
        .on("click", displayRemove);

	displayed.append("div")
		.attr("class", "key_square")
		.style("background", function(d){ return aDisplayColor[d.run_id][d.metric_id] });

	displayed.append("div")
		.attr("class", "key_label")
		.text(function(d) { return "Run: " + d.run_id + ", Metric: " + d.metric_id });
}

function displayRemove(){
    console.log("Removing ", this, "...");
    removeDisplayMetric(this.getAttribute("run_id"), this.getAttribute("metric_id"));
}

function updateSelectionItems(){
	console.log("Updating Selection...");
    d3.select("#parameters").selectAll(".select_line").remove();

    var values = selection.getFilterParameters().sort(function(a) { return a.name});
    var parameters = d3.select("#parameters").selectAll(".select_line").data(values)
        .enter()
        .append("div")
        .attr("class", "select_line")

     parameters
        .append("div")
        .attr("factor_id", function(d) { return d.factor_id })
        .style("float", "left")
        .text("X : ")
        .on("click", function(d){ return selection.removeSelectionFilter(d.factor_id); }); 

    parameters
        .append("div")
        .attr("class", "select_label")
        .text(function(d) { return "Factor: " + d.name + ",   Range: " + d.min_value + " - " + d.max_value });
    console.log(values);
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
initGraphSelect();
