// The experiment currently being observed
var exp_id, normalize;
var color = safe.colors.QUALITATIVE_0[0];
var focusDataPointRadius = 2;
//var contextDataPointRadius = 1.5;
var numberFormat = d3.format(".2f");
var windowResizeFunctions = [];

var selectFactor;

plotArea = function(exp_id, parentElement){
	this.exp_id = exp_id;
	this.parent = safe.tools.d3Object(parentElement);
    this.margin = {top: 10, right: 40, bottom: 50, left: 60};
    this.dataList = {};

    this.graphWidth = undefined;
    this.focusHeight = 500;

    this.selectedGraph = undefined;

    this.initComponents();
}

plotArea.prototype.initComponents = function(){
    this.dSelf = this.parent.append("div").attr("class", "plot_area");
    windowResizeFunctions.push(this.resize.bind(this));

    this.main = this.dSelf.append("div").style({"width":"80%", "display":"inline-block"});
    var title = this.main.append("div").attr("class", "plot_area_label").text("Histogram");
    this.graphArea = this.main.append("div").attr("class", "graph_area");
    this.graphWidth = this.graphArea[0][0].offsetWidth;

    this.select = this.dSelf.append("div").style({"width":"20%", "display":"inline-block", "vertical-align":"top"});
    this.select.append("div").attr("class", "plot_selection_label").text("Select");
	this.selection = new safe.selector.full(this.exp_id, this.selectionChange.bind(this), this.addSelectionParameter.bind(this), this.select);

    this.select.append("div").style("height","0.5cm");
    this.select.append("div").attr("class", "plot_selection_label").text("Options");

    this.options = new safe.selector.histogram_options(title, this.select);
    this.options.disableGraphMovement();

    this.displayed = this.main.append("div").attr("class", "plot_displayed");
    this.displayed.append("div").attr("class", "plot_displayed_label").text("Displayed Metrics");
    this.displayKey = new safe.infobox.colored(this.removeMetricFromDisplay.bind(this) , this.displayed);
    
    this.filtered = this.main.append("div").attr("class", "plot_filtered");
    this.filtered.append("div").attr("class", "plot_filtered_label").text("Design Point Filter");
    this.selectParameters = new safe.infobox.default(this.removeSelectionParameter.bind(this), this.filtered);

    this.focus = new safe.graph.histogram(this.margin, this.graphWidth, this.focusHeight, undefined, this.graphArea);

    this.options.setSelectedGraph(this.focus);
    this.options.disableGraphMovement();


}

// levelDict -> a dictionary of (level_name, level) pairs
plotArea.prototype.selectionChange = function(run_id, metric){
    var metric_id = metric.id;

    this.setDisplayMetric(run_id, metric);
}

plotArea.prototype.setDisplayMetric = function(run_id, metric){
    if(this.currentRun != undefined){
        this.removeDisplayMetric(this.currentRun, this.currentMetric);
    }

    metric_id = metric.id;

    this.currentRun = run_id;
    this.currentMetric = metric_id;

    var id = run_id + "_" + metric_id;

	var update = function(){
        this.options.setSelectedGraph(this.focus);
        this.options.binsDropdownChange();
        this.selection.addDisplayed(run_id, metric_id, color);

        var displayString = "Run " + run_id + ", Metric: " + metric.name;
        var returnAttributes = {"run_id":run_id, "metric_id":metric_id };

        this.displayKey.addElement(id, displayString, color, returnAttributes)
        this.displayKey.updateElements();
    }.bind(this)

    var addPlot = function(data){
        pointAttributes = {"stroke": color, 
                           "fill": color };
        
        this.focus.setData(data, pointAttributes);
        this.focus.setXAxisLabel(metric.name + "(" + metric.units + ")");
    }.bind(this)

    var parameters = this.formatResultsParameters(this.selection.getResultsParameters()); 

    d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/" + parameters, function(error, data){
        if(error){ console.warn(error) }
        var data = data.map(function(d) { return d.value });

        addPlot(data);
        update();
    }.bind(this));


}

plotArea.prototype.removeMetricFromDisplay = function(attributes){
    this.removeDisplayMetric(attributes.run_id, attributes.metric_id);
}

plotArea.prototype.removeDisplayMetric = function(run_id, metric_id){
    var id = run_id + "_" + metric_id;

    this.displayKey.removeElement(id);
    this.selection.removeDisplayed(run_id, metric_id);
    this.focus.remove(id);
}

plotArea.prototype.removeSelectionParameter = function(parameter){
    console.log("Removing Selection: ", parameter);
    this.selectParameters.removeElement(parameter.factor_id);
    this.selection.removeDesignFilter(parameter.factor_id);
}

plotArea.prototype.addSelectionParameter = function(parameter){
    console.log("Adding Selection: ", parameter);
    var id = parameter.factor_id;
    var displayString = "Factor: " + parameter.name + " , Range: "+ parameter.min_value + " - " + parameter.max_value;
    var returnAttributes = parameter;
    this.selectParameters.addElement(id, displayString, returnAttributes);
}

plotArea.prototype.onGraphSelect = function(graph, src){
    if(this.selectedGraph)
        this.selectedGraph.svg.style("border", "none");
    graph.svg.style({"border": "solid 1px red"});

    this.selectedGraph = graph;

    var id = graph.getVisibleIds()[0];
    var dataSet = graph.getData();
    var pointAttributes = graph.getPointAttributes(id);
    

    this.options.setSelectedGraph(graph);
    if(src == undefined){ src = d3.event.target }
    cxt.parentNode.insertBefore(cxt, src.nextSibling);
}

plotArea.prototype.resize = function(){
    var graphArea = this.main[0][0];
    if(graphArea == undefined){ return }

    this.graphWidth = graphArea.offsetWidth;

    this.focus.setWidth(this.graphWidth);
    this.focus.display();
}

plotArea.prototype.formatResultsParameters = function(parameters){
    var parameterString = "?";
    Object.keys(parameters).forEach(function(key, index){
          if(index > 0){ parameterString += "&" }
          parameterString += key;
          parameterString += "=" + parameters[key];
    });
    return parameterString;
}

//======================================================

function onWindowResize(){
    windowResizeFunctions.forEach(function(resizeFunction){
        resizeFunction();
    });
}

function initVisualComponents(){
    d3.select(window).on("resize", onWindowResize);
	new plotArea(exp_id, "place_graphs")}

function addFactorPlotArea(factor){
	//console.log("Factor Changed: ", factor);
	new plotArea(exp_id, factor, "place_graphs");
    d3.select("#place_graphs").append("div").attr("class", "graph_sep");
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
    normalize = parameters["normalize"] == undefined ? false: parameters["normalize"] == "true";
}

readParameters();
initVisualComponents();
