// The experiment currently being observed
var exp_id;
var colors = safe.colors.QUALITATIVE_0;
var focusDataPointRadius = 2;
var contextDataPointRadius = 1.5;
var numberFormat = d3.format(".2f");
var windowResizeFunctions = [];

var selectFactor;
    
//----------------------------------------------
// Plot Area Object
// ---------------------------------------------

MetricPlotArea = function(exp_id, parentElement){
	this.exp_id = exp_id;
	this.parent = safe.tools.d3Object(parentElement);
    this.margin = {top: 10, right: 40, bottom: 50, left: 60};

	this.colorList = {};		// list of colors by run_id and metric_id
	this.displayList = {};		// list of metrics being displayed by run_id and metric_id
	this.dataList = {};			// list of data that has been loaded by run_id and metric_id
	
    this.graphWidth = undefined;
    this.graphHeight = 180;
	this.plotList = {};

    this.selectedGraph = undefined;
    //this.contextGraph = undefined;

    this.initComponents();
}

MetricPlotArea.prototype.initComponents = function(){
    this.dSelf = this.parent.append("div").attr("class", "plot_area");
    windowResizeFunctions.push(this.resize.bind(this));

    this.main = this.dSelf.append("div").style({"width":"80%", "display":"inline-block"});
    var title = this.main.append("div").attr("class", "plot_area_label").text("Histogram");
    this.graphArea = this.main.append("div").attr("class", "graph_area");
    this.graphWidth = this.graphArea[0][0].offsetWidth;

    this.select = this.dSelf.append("div").style({"width":"20%", "display":"inline-block", "vertical-align":"top"});
    this.select.append("div").attr("class", "plot_selection_label").text("Select");
	this.selection = new safe.selector.full(this.exp_id, this.selectionChange.bind(this), this.addSelectionParameter.bind(this), this.select);

    this.select.append("div").style("height","0.5cm");
    this.select.append("div").attr("class", "plot_selection_label").text("Options");
    this.options = new safe.selector.histogram_options(title, this.select);

    this.displayed = this.main.append("div").attr("class", "plot_displayed");
    this.displayed.append("div").attr("class", "plot_displayed_label").text("Displayed Metrics");
    this.displayKey = new safe.infobox.colored(this.removeMetricFromKey.bind(this) , this.displayed);

    this.reserved = this.main.append("div").attr("class", "plot_reserved");
    this.reserved.append("div").attr("class", "plot_reserved_label").text("Reserved Colors");
    this.reservedColors = new safe.infobox.colored(this.removeReservedColor.bind(this), this.reserved);

    this.filtered = this.main.append("div").attr("class", "plot_filtered");
    this.filtered.append("div").attr("class", "plot_filtered_label").text("Design Point Filter");
    this.selectParameters = new safe.infobox.default(this.removeSelectionParameter.bind(this), this.filtered);

    //this.context = new safe.graph.histogram(this.margin, this.graphWidth, this.graphHeight, undefined, this.graphArea);
    //this.context.yAxis.ticks(5);
    //this.context.svg.style({"border": "solid 3px red"}); 
    //this.options.setContextGraph(this.context);
    onWindowResize();
}

// levelDict -> a dictionary of (level_name, level) pairs
MetricPlotArea.prototype.selectionChange = function(run_id, metric){
    var metric_id = metric.id;

    if(this.colorList[run_id] == undefined){ this.colorList[run_id] = {} }
	if(this.displayList[run_id] == undefined) { this.displayList[run_id] = {} }

	if(this.colorList[run_id][metric_id] == undefined) { 
        if(colors.length == 0){
            alert("All available colors have been used. Please remove one or more colors from the reserved colors list for further use.");
            return;
        }
        this.colorList[run_id][metric_id] = colors.shift() 
    }else{
        this.reservedColors.removeElement(run_id + "_" + metric_id);
    }	
    
    if(this.displayList[run_id][metric_id] == undefined){
		this.addDisplayMetric(run_id, metric);
	}else{
		this.removeDisplayMetric(run_id, metric);
	}
}

MetricPlotArea.prototype.addDisplayMetric = function(run_id, metric){
    metric_id = metric.id;
    this.displayList[run_id][metric_id] = true;

    var id = run_id + "_" + metric_id;

	var update = function(){
        this.plotList[id].display(id);
        if(this.selectedGraph == undefined){ this.onGraphSelect(this.plotList[id], this.plotList[id].svg[0][0]) }

        this.selection.addDisplayed(run_id, metric_id, this.colorList[run_id][metric_id]);

        var displayString = "Run " + run_id + ", Metric: " + metric.name;
        var color = this.colorList[run_id][metric_id];
        var returnAttributes = {"run_id":run_id, "metric":metric };

        this.displayKey.addElement(id, displayString, color, returnAttributes)
        this.displayKey.updateElements();
    }.bind(this)

    var addPlot = function(data){
        this.dataList[run_id][metric_id] = true;

        pointAttributes = {"stroke": this.colorList[run_id][metric_id], 
                           "fill": this.colorList[run_id][metric_id]};
        
        this.plotList[id].setData(data, pointAttributes);
        this.plotList[id].setYAxisLabel(metric.name + "(" + metric.units + ")");
        this.plotList[id].setXAxisLabel("Time (seconds)");

        this.options.addGraph(id, this.plotList[id]);
    }.bind(this)



    if(this.dataList[run_id] == undefined){ this.dataList[run_id] = {} }
    if(this.dataList[run_id][metric_id] == undefined){
        this.plotList[id] = new safe.graph.histogram(this.margin, this.graphWidth, this.graphHeight, this.onGraphSelect.bind(this), this.graphArea);
        var parameters = this.formatResultsParameters(this.selection.getResultsParameters()); 

        d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/" + parameters, function(error, data){
            if(error){ console.warn(error) } 
            var data = data.map(function(d) { return d.value });

            addPlot(data);
            this.updateDomains();
			update();
        }.bind(this));
    }else{
        this.plotList[id].setVisible(id);
		update();
    }


}

MetricPlotArea.prototype.removeMetricFromKey = function(attributes){
    this.removeDisplayMetric(attributes.run_id, attributes.metric);
}

MetricPlotArea.prototype.removeDisplayMetric = function(run_id, metric){
    var metric_id = metric.id;
    var id = run_id + "_" + metric_id;
    var displayString = "Run " + run_id + ", Metric: " + metric.name;
    var returnAttributes = {"run_id":run_id, "metric":metric};

    var reselect = false;
    if(this.plotList[id] == this.selectedGraph){ reselect = true; }

    this.displayKey.removeElement(id);
    this.reservedColors.addElement(id, displayString, this.colorList[run_id][metric.id], returnAttributes);
    this.selection.removeDisplayed(run_id, metric_id);
    this.plotList[id].svg.remove();

    delete this.dataList[run_id][metric_id];
    delete this.displayList[run_id][metric_id];
    delete this.plotList[id];

    if(reselect){
        delete this.selectedGraph;
        if(Object.keys(this.plotList).length > 0){
            this.selectedGraph = this.plotList[Object.keys(this.plotList)[0]];
            this.onGraphSelect(this.selectedGraph, this.selectedGraph.svg[0][0]);
        }
    }

    this.updateDomains();
}

MetricPlotArea.prototype.removeSelectionParameter = function(parameter){
    this.selectParameters.removeElement(parameter.factor_id);
    this.selection.removeDesignFilter(parameter.factor_id);
}

MetricPlotArea.prototype.addSelectionParameter = function(parameter){
    var id = parameter.factor_id;
    var displayString = "Factor: " + parameter.name + " , Range: "+ parameter.min_value + " - " + parameter.max_value;
    var returnAttributes = parameter;
    this.selectParameters.addElement(id, displayString, returnAttributes);
}

MetricPlotArea.prototype.removeReservedColor = function(attributes){
    var run_id = attributes.run_id;
    var metric_id = attributes.metric.id;

    colors.push(this.colorList[run_id][metric_id]);
    delete this.colorList[run_id][metric_id];
    this.reservedColors.removeElement(run_id + "_" + metric_id);
}

MetricPlotArea.prototype.onGraphSelect = function(graph, src){
    if(this.selectedGraph)
        this.selectedGraph.svg.style("border", "none");
    graph.svg.style({"border": "solid 1px red"});

    this.selectedGraph = graph;
    this.options.setSelectedGraph(graph);
}

MetricPlotArea.prototype.updateDomains = function() {
    var ranges = [];
    d3.values(this.plotList).forEach(function(d){
        ranges.push(d.getExtent());
    });
    var min = d3.min(ranges, function(d){ return d[0] });
    var max = d3.max(ranges, function(d){ return d[1] });

    d3.values(this.plotList).forEach(function(d){
        d.setXDomain([min, max]);
        d.update();
        d.display();
    });
}

MetricPlotArea.prototype.resize = function(){
    var graphArea = this.graphArea[0][0];
    if(graphArea == undefined){ return }

    this.graphWidth = graphArea.offsetWidth;

    Object.keys(this.plotList).forEach(function(key){
        this.plotList[key].setWidth(this.graphWidth);
        this.plotList[key].displayAll();
    }.bind(this));
}

MetricPlotArea.prototype.formatResultsParameters = function(parameters){
    var parameterString = "?";
    Object.keys(parameters).forEach(function(key, index){
          if(index > 0){ parameterString += "&" }
          parameterString += key;
          parameterString += "=" + parameters[key];
    });
    return parameterString;
}

//======================================================

function onWindowResize(){
    windowResizeFunctions.forEach(function(resizeFunction){
        resizeFunction();
    });
}

function initVisualComponents(){
    d3.select(window).on("resize", onWindowResize);
	new MetricPlotArea(exp_id, "place_graphs")}

function addFactorPlotArea(factor){
	//console.log("Factor Changed: ", factor);
	new MetricPlotArea(exp_id, factor, "place_graphs");
    d3.select("#place_graphs").append("div").attr("class", "graph_sep");
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
