//------------------------------
// Initialize components
// -----------------------------
//
// Initializes components necesary for the proper function of
// components of the safe library

// The object to be referenced for the safe library
var safe = {};

// The names of the various modules with the safe library
safe.selector = {};
safe.graph = {};
safe.tools = {};
safe.infobox = {};
//------------------------------
// Initialize colors 
// -----------------------------
//
// Initializes all color pallets available through SAFE

safe.colors = {};

const safe_colors_DIVERGING_0 = [ "#543005", "#8c510a", "#bf812d", "#dfc27d", "#f6e8c3", "#f5f5f5", "#c7eae5", "#80cdc1", "#35978f", "#01665e", "#003c30"];
const safe_colors_DIVERGING_1 = [ "#8e0152", "#c51b7d", "#de77ae", "#f1b6da", "#fde0ef", "#f7f7f7", "#e6f5d0", "#b8e186", "#7fbc41", "#4d9221", "#276419"];
const safe_colors_DIVERGING_2 = [ "#40004b", "#762a83", "#9970ab", "#c2a5cf", "#e7d4e8", "#f7f7f7", "#d9f0d3", "#a6dba0", "#5aae61", "#1b7837", "#00441b"];
const safe_colors_DIVERGING_3 = [ "#7f3b08", "#b35806", "#e08214", "#fdb863", "#fee0b6", "#f7f7f7", "#d8daeb", "#b2abd2", "#8073ac", "#542788", "#2d004b"];
const safe_colors_DIVERGING_4 = [ "#67001f", "#b2182b", "#d6604d", "#f4a582", "#fddbc7", "#f7f7f7", "#d1e5f0", "#92c5de", "#4393c3", "#2166ac", "#053061"];
const safe_colors_DIVERGING_5 = [ "#67001f", "#b2182b", "#d6604d", "#f4a582", "#fddbc7", "#ffffff", "#e0e0e0", "#bababa", "#878787", "#4d4d4d", "#1a1a1a"];
const safe_colors_DIVERGING_6 = [ "#a50026", "#d73027", "#f46d43", "#fdae61", "#fee090", "#ffffbf", "#e0f3f8", "#abd9e9", "#74add1", "#4575b4", "#313695"];

const safe_colors_QUALITATIVE_0 = [ "#377eb8", "#4daf4a", "#984ea3", "#ff7f00", "#a65628", "#f781bf", "#999999", "#e41a1c"];


safe.colors.DIVERGING_0 = safe_colors_DIVERGING_0;
safe.colors.DIVERGING_1 = safe_colors_DIVERGING_1;
safe.colors.DIVERGING_2 = safe_colors_DIVERGING_2;
safe.colors.DIVERGING_3 = safe_colors_DIVERGING_3;
safe.colors.DIVERGING_4 = safe_colors_DIVERGING_4;
safe.colors.DIVERGING_5 = safe_colors_DIVERGING_5;
safe.colors.DIVERGING_6 = safe_colors_DIVERGING_6;

safe.colors.QUALITATIVE_0 = safe_colors_QUALITATIVE_0;
//------------------------------
// Initialize constants 
// -----------------------------
//
// Initializes the constants used throughout safe 

safe.constants = {}
safe.constants.X_AXIS = 0;
safe.constants.Y_AXIS = 1;
/**--------------------------------------
 * The base graph object
 * -------------------------------------
 *
 * This is the base graph object. It is expected that all
 * graphs will make use of this object's initialization and
 * ptototype for their functionality.
 */

safe.graph.base = safe_graph_base;
safe.graph.clipPathNumber = 0;

/**
 * Sets up the layout of the graph and adds it to the specified element.
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */

function safe_graph_base(margins, width, height, selectedCallBack, parentElement){
    this.margin = margins;
    this.height = height - this.margin.top - this.margin.bottom;
    this.width = width - this.margin.left - this.margin.right;
    this.selectedCallBack = selectedCallBack;
    this.parentElement = safe.tools.d3Object(parentElement);

    this.xScale = d3.scale.linear().domain([0, 0]).range([0, this.width]);
    this.yScale = d3.scale.linear().domain([0, 0]).range([this.height, 0]);

    this.xAxis = d3.svg.axis().scale(this.xScale).orient("bottom");
    this.yAxis = d3.svg.axis().scale(this.yScale).orient("left");

    this.svg = this.parentElement
        .append("svg")
        .attr("width", this.width + this.margin.left + this.margin.right)
        .attr("height", this.height + this.margin.top + this.margin.bottom)
        .on("click", this.selected.bind(this));

	// Add x axis
	this.svg.append("g")
		.attr("class", "x axis")
		.attr("transform", "translate(" + this.margin.left + "," + (this.height + this.margin.top) + ")")
        .on("click", this.selected.bind(this))
		.call(this.xAxis);

    // Add y axis
    this.svg.append("g")
		.attr("class", "y axis")
		.attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")")
		.call(this.yAxis);

	// Text label for x axis
	this.xAxisLabel = this.svg
        .append("text")
        .attr("class", "xLabel")
		.attr("x", this.width / 2)
		.attr("y", this.height + this.margin.top)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	this.yAxisLabel = this.svg
        .append("text")
		.attr("class", "yLabel")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((this.height) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

    // Set the cliping area
    this.clipId = "clip_" + safe.graph.clipPathNumber;
    safe.graph.clipPathNumber += 1;

    this.clip = this.svg.append("defs")
        .append("svg:clipPath")
        .attr("id", this.clipId)
        .append("svg:rect")
        .attr("y", this.margin.top)
        .attr("x", this.margin.left)
        .attr("width", this.width)
        .attr("height", this.height);

    return this;
}

/** Called when the user clicks on this graph */
safe_graph_base.prototype.selected = function(){
    if(this.selectedCallBack){ this.selectedCallBack(this) }
}

/** 
 * Sets the label for the x axis of the graph
 * @param label The desired label as a string
 */
safe_graph_base.prototype.setXAxisLabel = function(label){
    this.xAxisString = label;
    this.svg.selectAll(".xLabel").text(label);
}

/** 
 * Sets the label for the y axis of the graph
 * @param label The desired label as a string
 */
safe_graph_base.prototype.setYAxisLabel = function(label){
    this.yAxisString = label;
    this.svg.selectAll(".yLabel").text(label);
}

/** 
 * Returns the current x axis label
 * @return The current x axis label as a string
 */
safe_graph_base.prototype.getXAxisLabel = function(){
    return this.xAxisString;
}

/** 
 * Returns the current y axis label
 * @return The current y axis label as a string
 */
safe_graph_base.prototype.getYAxisLabel = function(){
    return this.yAxisString;
}

/** 
 * Returns the pixel width of the graph including margins
 * @return the pixel width of the graph including margins
 */
safe_graph_base.prototype.getWidth = function(){
    return this.width + this.margin.left + this.margin.right;
}

/** 
 * Returns the pixel height of the graph including margins
 * @return the pixel height of the graph including margins
 */
safe_graph_base.prototype.getHeight = function(){
    return this.height + this.margin.top + this.margin.bottom;
}


/** 
 * Sets the height of the graph (including margins)
 * @param height The pixel height for the graph
 */
safe_graph_base.prototype.setHeight = function(height){
    if(height === undefined){ return }

    this.svg.attr("height", height);
    this.height = height - this.margin.top - this.margin.bottom;
    this.yAxisLabel.attr("x", 0-((this.height) / 2));
	this.xAxisLabel.attr({"x": this.width / 2, "y": this.height + this.margin.top } );
    this.yScale.range([this.height, 0]);
    this.svg.select(".x.axis").attr("transform", "translate(" + this.margin.left + "," + (this.height + this.margin.top) + ")");
    this.svg.select(".y.axis").call(this.yAxis);
    this.clip.attr({"y": this.margin.top, "height": this.height});
}

/** 
 * Sets the width of the graph (including margins)
 * @param width The pixel width for the graph
 */
safe_graph_base.prototype.setWidth = function(width){
    if(width === undefined){ return }
    this.svg.attr("width", width);
    this.width = width - this.margin.left - this.margin.right;
    this.xScale.range([0, this.width]);
	this.xAxisLabel.attr({"x": this.width / 2, "y": this.height + this.margin.top } );
    this.svg.select(".x.axis").call(this.xAxis);
    this.clip.attr({"x": this.margin.left, "width": this.width});
}

/** 
 * Sets the size of the graph (including margins)
 * @param width The pixel width for the graph
 * @param height The pixel height for the graph
 */
safe_graph_base.prototype.setSize = function(width, height){
    this.setWidth(width);
    this.setHeight(height);
}

/**
 * Sets the domain to be used for the x Axis
 * @param domain The domain for the x-axis in the form [xMin, xMax]
 */
safe_graph_base.prototype.setXDomain = function(domain){
    if(domain != undefined && domain.length == 2) {
        if(isNaN(domain[0]))
            domain[0] = 0;
        if(isNaN(domain[1]))
            domain[1] = 0;
        this.xScale.domain(domain) 
    }
    this.svg.select(".x.axis").call(this.xAxis);
}

/**
 * Sets the domain to be used for the y Axis
 * @param domain The domain for the y-axis in the form [yMin, yMax]
 */
safe_graph_base.prototype.setYDomain = function(domain){
    if(domain != undefined && domain.length == 2) {
        if(isNaN(domain[0]))
            domain[0] = 0;
        if(isNaN(domain[1]))
            domain[1] = 0;
        this.yScale.domain(domain) 
    }
    this.svg.select(".y.axis").call(this.yAxis)
}

/** 
 * Sets the domains to be used for this graph
 * Only data points withing these domains will be displayed
 * @param xDomain The domain for the x axis in the form [xMin, xMax]
 * @param yDomain The domain for the y axis in the form [yMin, yMax]
 */
safe_graph_base.prototype.setDomains = function(xDomain, yDomain){
    this.setXDomain(xDomain);
    this.setYDomain(yDomain);
}

/**
 * Returns the current x domain of this graph
 * @return The current x domain of this graph
 */
safe_graph_base.prototype.getXDomain = function(){
    return this.xScale.domain();
}

/**
 * Returns the current y domain of this graph
 * @return The current y domain of this graph
 */
safe_graph_base.prototype.getYDomain = function(){
    return this.yScale.domain();
}

/**-------------------------------------
 * The base cartesian graph object
 * -------------------------------------
 *
 * This is the base carteisna graph object. It is expected that all
 * graphs will make use of this object's initialization and
 * ptototype for their functionality.
 */

safe.graph.cartesian = safe_graph_cartesian;
safe.graph.clipPathNumber = 0;

safe_graph_cartesian.prototype = Object.create(safe.graph.base.prototype);
safe_graph_cartesian.constructor = safe_graph_cartesian;

/**
 * Sets up the layout of the graph and adds it to the specified element.
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */

function safe_graph_cartesian(margins, width, height, selectedCallBack, parentElement){
    safe.graph.base.apply(this, [margins, width, height, selectedCallBack, parentElement]);
    this.xMin = this.yMin = this.xMax = this.yMax = 0;
    this.data = {};
    this.xAccessor = {};
    this.yAccessor = {};
    this.toDisplay = {};
    this.pointAttributes = {};

    this.tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    return this;
}

/** Redisplays all data sets for this graph */
safe_graph_cartesian.prototype.displayAll = function(){
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] == true) { this.display(id) }
    }.bind(this));
}

/** 
 * Adds the specified data set to this graph
 * @param id A string that distinguishes this data set from all others on this graph
 * @param data The data to be added to this graph
 * @param xAccessor A function which will return the x value of a point when called with the point as a parameter
 * @param yAccessor A function which will return the y value of a point when called with the point as a parameter
 * @param pointAttributes A dictionary of html attributes to add to each displayed point
 * @param mouseEvents A dictionary of mouse events to occur when mouse is over a data point
 */
safe_graph_cartesian.prototype.addDataSet = function(id, data, xAccessor, yAccessor, pointAttributes){
    this.data[id] = data;
    this.xAccessor[id] = xAccessor;
    this.yAccessor[id] = yAccessor;
    this.pointAttributes[id] = pointAttributes;
    this.toDisplay[id] = true;
    this.updateDataFeatures();
    this.displayAll();
}

/** 
 * Sets the data for the specified data set 
 * @param id A string that distinguishes this data set from all others on this graph
 * @param data The data to be added to this graph
 */
safe_graph_cartesian.prototype.addData = function(id, data){
    this.data[id] = data;
}

/**
 * Sets the x value accessor function for the specified data set
 * @param id The string id for the desired data set specified in "addDataSet"
 * @param xAccessor A function that will return the x value of a point when called with the point as a parameter
 */
safe_graph_cartesian.prototype.setXAccessor = function(id, xAccessor){
    this.xAccessor[id] = xAccessor;
}

/**
 * Sets the y value accessor function for the specified data set
 * @param id The string id for the desired data set specified in "addDataSet"
 * @param yAccessor A function that will return the y value of a point when called with the point as a parameter
 */
safe_graph_cartesian.prototype.setYAccessor = function(id, yAccessor){
    this.yAccessor[id] = yAccessor;
}

/** 
 * Returns the data associated with the given id
 * @param id The string id for the desired data set specified in "addDataSet"
 * @return The data associated with the given id
 */
safe_graph_cartesian.prototype.getDataSet = function(id){
    return this.data[id];
}

/** 
 * Returns all data associated with this graph 
 * @return The data associated with this graph
 */
safe_graph_cartesian.prototype.getData = function(){
    results = {};
    Object.keys(this.data).forEach(function(key){
        if(this.toDisplay[key] == true)
            results[key] = this.data[key];
    }.bind(this));
    return results;
}

/**
 * Determines whether the data associated with the given id exists
 * @param id The string id for the desired data set specified in "addDataSet"
 * @return true if the data set exists, false otherwise
 */
safe_graph_cartesian.prototype.hasDataSet = function(id){
    return this.data[id] != undefined;
}

/** 
 * Returns the dictionary of attributes associated with the specified id
 * @param id The string id for the desired data set specified in "addDataSet"
 * @return The dictionary of attributes associated with the specified id
 */
safe_graph_cartesian.prototype.getPointAttributes = function(id){
    return this.pointAttributes[id];
}

/** 
 * Sets the specified data set as invisible without removing its data from memory
 * @paran id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_cartesian.prototype.setInvisible = function(id){
    if(this.toDisplay[id] == false){ return }
    this.toDisplay[id] = false;
    this.updateDataFeatures();
    this.display(id);
    this.displayAll();
}

/** 
 * Sets the specified data set as visible
 * @paran id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_cartesian.prototype.setVisible = function(id){
    if(this.toDisplay[id] == true){ return }
    this.toDisplay[id] = true;
    this.updateDataFeatures();
    this.display(id);
    this.displayAll();
}

/** 
 * Returns a list of all ids currently visible on this graph
 * @return A list of all ids currently visible on this graph
 */
safe_graph_cartesian.prototype.getVisibleIds = function(){
    values = [];
    Object.keys(this.toDisplay).forEach(function(id){
        if(this.toDisplay[id] == true)
            values.push(id);
    }.bind(this));
    return values;
}

/** 
 * Removes all information about the specified data set from the graph.
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_cartesian.prototype.remove = function(id){
    this.setInvisible(id);
    delete this.data[id];
    delete this.pointAttributes[id];
}

/** Removes all information for all data sets associated with this graph */
safe_graph_cartesian.prototype.removeAll = function(){
    Object.keys(this.data).forEach(function(id){
        this.setInvisible(id);
        this.remove(id);
    }.bind(this));
}

/** Updates cross hairs on this graph if they are enabled */
safe_graph_cartesian.prototype.updateCrossHairs = function(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xleft = this.xScale.range()[0] + this.margin.left; 
    var xright = this.xScale.range()[1] + this.margin.left;
    var ytop = this.yScale.range()[0] + this.margin.top;
    var ybottom = this.yScale.range()[1] + this.margin.top;

    if (x < xleft) { x = xleft }
    if (x > xright) { x = xright }
    if (y < ybottom) { y = ybottom }
    if (y > ytop) { y = ytop }

    horizontal = this.svg.selectAll(".crosshairhorizontal").data([""]);
    vertical = this.svg.selectAll(".crosshairvertical").data([""]);
 
    horizontal.enter().append("line").attr("class", "crosshairhorizontal").style({"stroke": "black", "pointer-events": "none"})
    vertical.enter().append("line").attr("class", "crosshairvertical").style({"stroke": "black", "pointer-events": "none"})
   
    horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

/** Enables cross hairs for this graph */
safe_graph_cartesian.prototype.enableCrossHairs = function(){
    this.svg.on("mousemove", this.updateCrossHairs.bind(this));
    this.svg.on("mouseout", function(){
        this.svg.selectAll(".crosshairhorizontal").remove();
        this.svg.selectAll(".crosshairvertical").remove();
        }.bind(this));
}

/** Disables cross hairs for this graph */
safe_graph_cartesian.prototype.disableCrossHairs = function(){
    this.svg.selectAll(".crosshairhorizontal").remove();
    this.svg.selectAll(".crosshairvertical").remove();
    this.svg.on("mousemove", undefined);
}

/** Called when mouse is over a data point */
safe_graph_cartesian.prototype.mouseOverPoint = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0.9);

    this.tooltip.html(d3.event.target.getAttribute("info"))
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
}

/** Called when mouse is no longer over a data point */
safe_graph_cartesian.prototype.mouseOutPoint = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0);
}

/** 
 * Sets the height of the graph (including margins)
 * @param height The pixel height for the graph
 */
safe_graph_cartesian.prototype.setHeight = function(height){
    safe.graph.base.prototype.setHeight.apply(this, [height]);
    this.updateBrush();
}

/** 
 * Sets the width of the graph (including margins)
 * @param width The pixel width for the graph
 */
safe_graph_cartesian.prototype.setWidth = function(width){
    safe.graph.base.prototype.setWidth.apply(this, [width]);
    this.updateBrush();
}

/**
 * Creates a brush to be used on this graph.
 * @param brushCallBack A function to be called when the brush is resized
 *        called as "brushCallBack(brush)"
 * @param constrain An axis constant from safe.constants indicating an axis
 *        to be constrained
 */
safe_graph_cartesian.prototype.addBrush = function(brushCallBack, constrain){
    this.brushConstrain = constrain;
    this.brushCallBack = brushCallBack;
    this.brush = d3.svg.brush().on("brush", this.brushed.bind(this));

    this.svg.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")")
        .call(this.brush);

    this.updateBrush();
}

/**
 * Calls the function associated with this graph's brush
 * if a brush has been created though the addBrush function
 * The function is called as brushCallBack(this.brush)
 */
safe_graph_cartesian.prototype.brushed = function(){
    if(!this.brush){ return }

    this.brushCallBack(this.brush);
}

/**
 * Updates the brush based on the current size of the graph.
 */
safe_graph_cartesian.prototype.updateBrush = function(){
    if(! this.brush){ return }

    if(this.brushConstrain == safe.constants.Y_AXIS){
        this.brush.x(this.xScale);
        this.svg.selectAll(".brush").call(this.brush)
            .selectAll("rect")
            .attr({"height": this.height + 7, "y": -6 });

    }else if(this.brushConstrain == safe.constants.X_AXIS){
        this.brush.y(this.yScale);
        this.svg.selectAll(".brush").call(this.brush)
            .selectAll("rect")
            .attr({"width": this.width, "x": 1 })

    }else{
        this.brush.x(this.xScale).y(this.yScale);
        this.svg.selectAll(".brush").call(this.brush);
    }
}

/** Updates information about maximum and minimum value in the data sets */
safe_graph_cartesian.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }

        xRange = d3.extent(this.data[id], function(d) { return this.xAccessor[id](d) }.bind(this));
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yRange = d3.extent(this.data[id], function(d) { return this.yAccessor[id](d) }.bind(this));
        this.yMin = (this.yMin == undefined) ? yRange[0] : d3.min([this.yMin, yRange[0]]);
        this.yMax = (this.yMax == undefined) ? yRange[1] : d3.max([this.yMax, yRange[1]]);
        }.bind(this));

    this.setDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}
/**----------------------------------
 * A graph for histograms
 *----------------------------------
 */

safe.graph.histogram = safe_graph_histogram;

safe_graph_histogram.prototype = Object.create(safe.graph.base.prototype);
safe_graph_histogram.constructor = safe_graph_histogram;

/**
 * Creates a new histogram object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_histogram(margins, width, height, onSelectedCallBack, parentElement){
    safe.graph.base.apply(this, [margins, width, height, onSelectedCallBack, parentElement]);

    this.frequency = false;
    this.infoFormat = d3.format(".4f");
    this.tickFormat = d3.format(".2f");
    this.bins = 5;
    this.tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    return this;
}

/** Redisplays all data sets for this graph */
safe_graph_histogram.prototype.displayAll = function(){
    this.display();
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_histogram.prototype.display = function(){
    if(this.histogram == undefined){ return }

    var bars = this.svg.selectAll(".bar").data(this.histogram);
    bars.enter().append("g").attr("class", "bar");
    bars.exit().remove();

    bars.selectAll("rect").remove();
    bars.append("rect");

    var rects = bars.selectAll("rect");
    rects
        .attr("x", function(d) { return this.margin.left + this.xScale(d.x  + 0.05 * d.dx) }.bind(this))
        .attr("y", function(d) { return this.margin.top + this.yScale(d.y)}.bind(this))
        .attr("width", function(d) { return this.xScale(this.xScale.domain()[0] + d.dx) * 0.9 }.bind(this))
        .attr("height", function(d) { return this.height - this.yScale(d.y) }.bind(this))
        .attr("info", function(d) { return this.infoFormat(d.y) + "</br>" + 
                                    this.infoFormat(d.x) + " - " + this.infoFormat(parseFloat(d.x) + parseFloat(d.dx)) }.bind(this))
        .style(this.style)
        .on("mouseover", this.mouseOverBox.bind(this))
        .on("mouseout", this.mouseOutBox.bind(this));

    bars.selectAll("text").remove();
    bars.append("text");

    var text = bars.selectAll("text");
    text
        .attr("x", function(d) { return this.margin.left + this.xScale(d.x) }.bind(this))
        .attr("y", function(d) { return this.margin.top + this.yScale(d.y)}.bind(this))
        .attr("dy", "20px")
        .attr("dx", function(d) { return this.xScale(this.xScale.domain()[0] + (d.dx/2)); }.bind(this))
        .attr("fill", "#fff")
        .attr("text-anchor", "middle")
        .attr("pointer-events", "none")
        .text(function(d) { return this.tickFormat(d.y) }.bind(this));

}

/**
 * Sets the data to be dispayed on this graph
 * @param data A list of values to be used in the creation of the graph
 * @param style A dictionary of css attributes that will be applied
 *        to each of the rectangles (bars) of the graph
 */
safe_graph_histogram.prototype.setData = function(data, style){
    if(data == undefined){ return }

    this.data = data;
    this.style = style;
    this.extent = d3.extent(data)
    this.setXDomain(this.extent);
    this.update();
}

/** Updates the content of this graph based on current instance values */
safe_graph_histogram.prototype.update = function(){
    this.histogram = d3.layout.histogram().frequency(this.frequency).bins(this.bins).range(this.xScale.domain())(this.data);
    this.setYDomain([0, d3.max(this.histogram, function(d) { return d.y })]);

    var tempScale = d3.scale.linear().domain([0, this.bins]).range(this.xScale.domain());
    var tickArray = d3.range(this.bins + 1).map(tempScale);
    this.xAxis.tickValues(tickArray).tickFormat(this.tickFormat);
    this.svg.select(".x.axis").call(this.xAxis);
}

/** Returns the data currently contained in the graph */
safe_graph_histogram.prototype.getData = function(){
    return this.data;
}

/** Returns the range of values currently associated with this graph */
safe_graph_histogram.prototype.getExtent = function(){
    return this.extent;
}

/** Removes the currently displayed data from the graph */
safe_graph_histogram.prototype.remove = function(){
    this.svg.selectAll(".bar").remove();
}

/** 
 * Sets the number of bins to the nearest number of bins by which the 
 * bins will be of equal size
 *
 * Note: The number of bins used may not be exactly what is specified
 *
 * @param bins The number of bins desired
 */
safe_graph_histogram.prototype.setBinNumber = function(bins){
    this.bins = bins;
    this.update();
}

/**
 * Returns the actual number of bins currently being used
 * @return The number of bins currently being used
 */
safe_graph_histogram.prototype.getBinNumber = function(){
    return this.bins;
}

/**
 * Sets the approximate width of the bins. The width that will
 * be used is the closest width that will provide equal width
 * to all bins
 *
 * @param width The desired width of the bins
 */
safe_graph_histogram.prototype.setBinWidth = function(width){
    this.bins = Math.round(d3.max(this.data) / width);
    if(this.bins <= 0){ this.bins = 1 }
    this.update();
}

/**
 * Returns the actual width of the bins currently being used
 * @return The width of the bins currently bing used
 */
safe_graph_histogram.prototype.getBinWidth = function(){
    if(!(this.histogram && this.histogram.length > 0)){ return 0 }
    return this.histogram[0].dx;
}

/**
 * Sets whether the values are calculated by as frequency or percentage
 * @param frequency A boolean which is true if the desired measurement
 *        is frequency
 */
safe_graph_histogram.prototype.setFrequency = function(frequency){
    this.frequency = frequency;
    this.update();
}

/**
 * Returns the boolean representing whether frequency or percentage is being used
 * @return true if frequency is being used, false if percentage is being used
 */
safe_graph_histogram.prototype.getFrequency = function(){
    return this.frequency;
}

/** Called when mouse is over a histogram box */
safe_graph_histogram.prototype.mouseOverBox = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0.9);

    this.tooltip.html(d3.event.target.getAttribute("info"))
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
}

/** Called when mouse is no longer over a histogram box */
safe_graph_histogram.prototype.mouseOutBox = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0);
}
/**-------------------------------------
 * A Colored Infobox Object
 * -------------------------------------
 *
 * The prototype of an object which holds lines of information which
 * can be removed by the user and with which a color is associated.
 * An example is a graph key which is the primary use for this object
 * within safe.
 */

safe.infobox.colored = safe_infobox_colored;

/**
 * The constructor of the information object
 *
 * @param removeCallBack The function to be called when an element
 *        is selected for removal. Called as "removeCallBack(key)"
 *        where key is the key given thorugh "addElement"
 * @param parentElement The element to which this infobox should be
 *        added. It can be and element id, dom element, or d3 object.
 */
function safe_infobox_colored(removeCallBack, parentElement){
    this.removeCallBack = removeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);

    this.dSelf = this.dParent
        .append("div")
        .attr("class", "container well col-xs-12")
        .style("width", "100%");

    this.elements = {};
    this.colors = {};
    this.returnAttributes = {};

}

/**
 * Adds an element to the infobox
 * @param key A unique identifier for this element
 * @param elementString The informational string to be displayed
 * @param elementColor The color to be associated with this element
 * @param returnAttributes A value (or object) to be returned along 
 *        with an element's key when the element is removed.
 */
safe_infobox_colored.prototype.addElement = function(key, elementString, elementColor, returnAttributes){
    this.elements[key] = elementString;
    this.colors[key] = elementColor;
    this.returnAttributes[key] = returnAttributes;
    this.updateElements();
}

/**
 * Removes an element from the infobox
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_colored.prototype.removeElement = function(key){
    delete this.elements[key];
    delete this.colors[key];
    delete this.returnAttributes[key];
    this.updateElements();
}

/**
 * Called when an element is selected for removal by the user
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_colored.prototype.removeSelected = function(key){
    this.removeCallBack(this.returnAttributes[key]);
}

/**
 * Updates the elements being displayed in the infobox
 * to those currently associated with it.
 */
safe_infobox_colored.prototype.updateElements = function(){
	this.dSelf.selectAll("div").remove();
	var displayed = this.dSelf.selectAll("div").data(Object.keys(this.elements))
			.enter()
			.append("div")
			.attr("class", "col-xs-12")

    displayed.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle col-xs-1")
        .style("cursor", "pointer")
        .on("click", function(d){ this.removeSelected(d) }.bind(this));

	displayed.append("div")
		.attr("class", "glyphicon glyphicon-check col-xs-1")
		.style("color", function(d){ return this.colors[d] }.bind(this));

	displayed.append("div")
		.attr("class", "key_label")
		.text(function(d) { return this.elements[d] }.bind(this));

    displayed.sort(function(a, b){ return this.elements[a].localeCompare(this.elements[b]) }.bind(this));
}
/**-------------------------------------
 * A Default Infobox Object
 * -------------------------------------
 *
 * The prototype of an object which holds lines of information which
 * can be removed by the user.
 */

safe.infobox.default = safe_infobox_default;

/**
 * The constructor of the information object
 *
 * @param removeCallBack The function to be called when an element
 *        is selected for removal. Called as "removeCallBack(key)"
 *        where key is the key given thorugh "addElement"
 * @param parentElement The element to which this infobox should be
 *        added. It can be and element id, dom element, or d3 object.
 */
function safe_infobox_default(removeCallBack, parentElement){
    this.removeCallBack = removeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    
    this.dSelf = this.dParent
        .append("div")
        .attr("class", "container well col-xs-12")
        .style("width", "100%");

    this.elements = {};
    this.returnAttributes = {};
}

/**
 * Adds an element to the infobox
 * @param key A unique identifier for this element
 * @param elementString The informational string to be displayed
 * @param returnAttributes A value (or object) to be returned along 
 *        with an element's key when the element is removed.
 */
safe_infobox_default.prototype.addElement = function(key, elementString, returnAttributes){
    this.elements[key] = elementString;
    this.returnAttributes[key] = returnAttributes;
    this.updateElements();
}

/**
 * Removes an element from the infobox
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_default.prototype.removeElement = function(key){
    console.log("Deleting: ", key);
    delete this.elements[key];
    delete this.returnAttributes[key];
    this.updateElements();
}

/**
 * Called when an element is selected for removal by the user
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_default.prototype.removeSelected = function(key){
    this.removeCallBack(this.returnAttributes[key]);
}

/**
 * Updates the elements being displayed in the infobox
 * to those currently associated with it.
 */
safe_infobox_default.prototype.updateElements = function(){
    console.log(this.elements);
    this.dSelf.selectAll("div").remove();

    var parameters = this.dSelf.selectAll("div").data(Object.keys(this.elements))
        .enter()
        .append("div")
        .attr("class", "col-xs-12")

    parameters.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle col-xs-1")
        .style("cursor", "pointer")
        .on("click", function(d){ this.removeSelected(d) }.bind(this));

    parameters
        .append("div")
        .attr("class", "select_label")
        .text(function(d) { return this.elements[d] }.bind(this));
}
/**-------------------------------------
 * An x-axis selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of axis 
 * in addition to the default selection options
 */

safe.selector.axis = safe_selector_axis;

/**
 * Creates a new selection object
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_axis(changeCallBack, parentElement){
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.xAxis = safe.constants.X_AXIS;
    this.yAxis = safe.constants.Y_AXIS;
    this.data = [{"name":"X Axis", "value":this.xAxis},
                 {"name":"Y Axis", "value":this.yAxis}];
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_axis.prototype.initComponents = function(){
    this.dSelf = this.dParent
        .append("div");

    this.dSelect = this.dSelf
        .append("select")
        .attr("class", "form-control")
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];

    this.dSelect.selectAll("option").data(this.data)
        .enter()
        .append("option")
        .attr("axis", function(d){ return d.value })
        .text(function(d){ return d.name });
}

/**
 * Called when there is a change in the currently selected axis.
 */
safe_selector_axis.prototype.selectChange = function(){
    this.callBack(this.getSelectedAxis());
}

/**
 * Returns the design point that is currently selected
 * @return The currently selected axis 
 */
safe_selector_axis.prototype.getSelectedAxis = function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("axis");
}
/**-------------------------------------
 * A designpoint selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of design points
 * which are stored in the database.
 */

safe.selector.designpoint = safe_selector_designpoint;

/**
 * Creates a new selection object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_designpoint(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);

    this.designPointList = {};
    this.designPointFactors = undefined;
    this.parameters = {};
}

/**
 * Reads data from the database about available design points
 * @param callBack A function to be called when the read is complete
 */
safe_selector_designpoint.prototype.readInitialData = function(callBack){
    this.designPointFactors = {};
    d3.json("/safe/exp/" + this.exp_id + "/designpoints/", function(error, designPoints){
        designPoints.forEach(function(designPoint){
            this.designPointFactors[designPoint.id] = designPoint.factors;
        }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

/**
 * Reads data from the database about design points associated
 * with the selection parameters currently in place for the selector
 *
 * @param callBack A functio to be called when the read is complete
 */
safe_selector_designpoint.prototype.readData = function(callBack){
    var update = function(){ 
        var parameters = this.getParameters();

        d3.json("/safe/exp/" + this.exp_id + "/dp-range-query/").post(JSON.stringify(parameters), function(error, designPoints){
            this.designPointList = designPoints;
            if(callBack != undefined){ callBack() };
        }.bind(this));
    }.bind(this)
    
    if(this.designPointFactors == undefined ){
        this.readInitialData(update.bind(this));
    }else{
      update();
    }
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_designpoint.prototype.initComponents = function(){
    this.dSelf = this.dParent
        .append("div");

    this.dSelect = this.dSelf
        .append("select")
        .attr("class", "form-control")
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];

    this.dInfo = this.dSelf
        .append("div");
}

/**
 * Updates the currently displayed design points based on data in the database
 * @param callBack A function to be called after the update is complete
 */
safe_selector_designpoint.prototype.updateDesignPoints = function(callBack){
    var update = function(){
        designPoints = this.dSelect.selectAll("option").data(this.designPointList);
        designPoints.enter().append("option");
        designPoints.attr("designPoint", function(d) { return d })
            .text(function(d) { return "DesignPoint " + d });
        designPoints.exit().remove();

        this.updateDesignPointInfo();
        
        if(callBack != undefined){ callBack() }
    }.bind(this);
    this.readData(update.bind(this));
}

/**
 * Updates the display which holds information about the currently 
 * selected design point.
 */
safe_selector_designpoint.prototype.updateDesignPointInfo = function(){
    var designPoint = this.designPointFactors[this.getSelectedDesignPoint()];

    var lines = this.dInfo.selectAll("label").data(Object.keys(designPoint))
    lines.enter().append("label");
    lines.attr("class", "col-xs-12").text(function(d) { return d + " : " + designPoint[d] });
    lines.exit().remove();
    lines.sort();
}

/**
 * Called when there is a change in the currently selected design point.
 */
safe_selector_designpoint.prototype.selectChange = function(){
    this.updateDesignPointInfo();
    this.callBack(this.getSelectedDesignPoint());
}

/**
 * Returns the design point that is currently selected
 * @return The currently selected design point
 */
safe_selector_designpoint.prototype.getSelectedDesignPoint = function(){
    designpoint_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("designpoint");
    return designpoint_id;
}

/**
 * Adds a parameter for the reading of design point data
 * @param key A distinct string with which to associate the parameter
 * @param parameter A restriction of the form {"factor_name": <factor_name>, "min_value":<min_value>, "max_value":<max_value>}
 */
safe_selector_designpoint.prototype.addParameter = function(key, parameter){
    this.parameters[key] = parameter;
}

/**
 * Removes a parameter that was previously added
 * @param key The key associated with the parameter through "addParameter"
 */ 
safe_selector_designpoint.prototype.removeParameter = function(key){
    delete this.parameters[key];
}

/**
 * Returns a list of all parameters being used by this selector
 * @return A list of all parameters of the form
 *      [{"factor_name": <factor_name>, "min_value":<min_value>, "max_value":<max_value>}, ...]
 */
safe_selector_designpoint.prototype.getParameters = function(){
    parameters = [];
    Object.keys(this.parameters).forEach(function(parameterKey){
        parameters.push(this.parameters[parameterKey]);
    }.bind(this));
    return parameters;
}
/**-------------------------------------
 * A factor selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of factors
 * which are stored in the database.
 */
safe.selector.factor = safe_selector_factor;

/**
 * Creates a new factor selector object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBAck a function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_factor(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.factorList = {};
}

/**
 * Reads data from the database about available factors
 * @param callBack A function to be called when the read is complete
 */
safe_selector_factor.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/factors/", function(error, data){
        if(error){ console.warn(error) }

        data.forEach(function(factor){ this.factorList[factor.id] = factor }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_factor.prototype.initComponents = function(attributes, style){
    this.dSelect = this.dParent
        .append("select")
        .attr("class", "form-control")
        .style(style)
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];

    this.dSelect
        .selectAll("option")
        .data(Object.keys(this.factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d })
        .text(function(d) { return this.factorList[d].name }.bind(this));
}

/**
 * Called when there is a change in the currently selected factor.
 */
safe_selector_factor.prototype.selectChange = function(){
    if(this.callBack != undefined){ this.callBack(this.getSelectedFactor()) }
}

/**
 * Returns the factor that is currently selected
 * @return The currently selected factor
 */
safe_selector_factor.prototype.getSelectedFactor = function(){
    var factor_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("factor");
    return this.factorList[factor_id];
}
/**-------------------------------------
 * A file selector object 
 * -------------------------------------
 *
 * An object that allows the user to select a file from
 * their file system
 */

safe.selector.file = safe_selector_file;

/**
 * Creates a new file selector object
 *
 * @param parentElement The dom element, d3 object, or dom id
 *        to which this selector should be added
 */
function safe_selector_file(parentElement){
    this.parent = safe.tools.d3Object(parentElement);
    this.data = {};
}

/** Initializes the dom components of this selector */
safe_selector_file.prototype.initComponents = function(){
    this.dSelf = this.parent.append("div");

    var file = this.dSelf.append("input")
        .attr("type", "file")
        .attr("name", "file")
        .style("width", "100%");

    var progress = this.dSelf.append("div");

    var list = this.dSelf.append("output");

    var div = this.dSelf.append("div")
        .attr("class", "file_name_select");
     
    div.append("label")
        .attr("class", "half_line")
        .text("Distribution Name:");
    
    var fileName = div.append("input")
        .attr("type", "text")
        .attr("class", "bordered_half_line");
        
    this.dSelf.append("button")
        .attr("class", "upload_button")
        .text("Upload")
        .on("click", this.uploadFile.bind(this)); 

    this.fileName = fileName[0][0];

    this.file = file[0][0];
    this.list = list[0][0];
    this.progress = progress[0][0];

    this.file.addEventListener("change", this.fileSelectChange.bind(this), false);
}

/** called when there is a change in the selected file */
safe_selector_file.prototype.fileSelectChange = function(event){
    var files = this.file.files;
    var output = [];

    for(var i = 0, f; f = files[i]; i++){

        var reader = new FileReader();

        reader.onload = this.loadedListener.bind(this);
        reader.onprogress = this.progressListener.bind(this);

        reader.readAsText(f);

        output.push("<li><strong>", escape(f.name), 
                    "</strong> (",
                    f.type || "n/a",
                    ") - ",
                    f.size,
                    " bytes, last modified: ",
                    f.lastModifiedDate ? f.lastModifiedDate.toLocaleDateString() : "n/a",
                    "</li>");
    }
    this.list.innerHTML = "<ul>" + output.join("") + "</ul>";
}

/** Listens for progress in the file upload */
safe_selector_file.prototype.progressListener = function(event){
    if(event.lengthComputable){
        var percent = event.loaded / event.total;
        
        while(this.progress.hasChildNodes()){
            this.progress.removeChild(this.progress.firstChild);
        }

        this.progress.appendChild(document.createTextNode(Math.round(percent * 100) + "%"));
    }
}

/** Listens for the completion of a file load */
safe_selector_file.prototype.loadedListener = function(event){
    this.fileData = event.target.result;
    this.progress.style.display = "none";
}

/** Listens for errors in the upload of the file */
safe_selector_file.prototype.errorListener = function(event){
    alert("Upload failed");
}

/** Returns the file object that has been uploaded */
safe_selector_file.prototype.getFile = function(){
    return this.file.files[0];
}

/** Returns the data of the uploaded file */
safe_selector_file.prototype.getFileData = function(){
    return this.fileData;
}

/**
 * Uploads as a distribution the file stored in the file selector as a distribution
 */
safe_selector_file.prototype.uploadFile = function(){
    var file = this.getFile();
    var data = this.getFileData();

    var match = file.type.match(".*(csv|json)");
    if(match != null && match[1] == "csv" || match == null && file.name.match(".*\.csv")){
        data = d3.csv.parse(data);
    }else if(match != null && match[1] == "json" || match == null && file.name.match(".*\.json")){
        data = JSON.parse(data);
    }else{
        alert("Invalid file type. Must be .csv or .json");
    }

    console.log(this.fileName.value, data);
}
/**-------------------------------------
 * A level selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of factor levels
 * which are stored in the database.
 */
safe.selector.level = safe_selector_level;

/**
 * Creates a new selection object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_level(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.levelList = {};
}

/**
 * Reads data from the database about available factors and their levels 
 * @param callBack A function to be called when the read is complete
 */
safe_selector_level.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + exp_id + "/levels/", function(error, data){
        if(error){ console.warn(error) }

        Object.keys(data).forEach(function(factor_id){ 
            this.levelList[factor_id] = data[factor_id].levels.sort(); 
        }.bind(this));

        if(callBack != undefined){ callBack() }
    }.bind(this));
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_level.prototype.initComponents = function(attributes, styles){
    this.dSelect = this.dParent
        .append("select")
        .attr("class", "form-control")
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];
}

/**
 * Updates the currently displayed levels based on data in the database
 * @param factor_id The id of the factor whose levels should be displayed
 */
safe_selector_level.prototype.updateLevels = function(factor_id){
    levels = this.dSelect.selectAll("option").data(this.levelList[factor_id])
    levels.enter().append("option");
    levels.attr("level", function(d) { return d }).text(function(d) { return d });
    levels.exit().remove();
    this.eSelect.selectedIndex = 0;
}

/**
 * Sets the levels to be associated with a given factor
 * @param factor_id The id of the factor whose levels are being specified
 * @param levels A list of the levels to associate with the factor
 */
safe_selector_level.prototype.setLevels = function(factor_id, levels){
    this.levelList[factor_id] = levels;
}

/**
 * Called when the selected level is changed
 */
safe_selector_level.prototype.selectChange = function(){
    if(this.callBack){ this.callBack(this.getSelectedLevel()) }
}

/**
 * Returns the currently selected level
 * @return The currently selected level
 */
safe_selector_level.prototype.getSelectedLevel= function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("level");
}

/**
 * Returns the minimum selectable level for the specified factor
 * @param factor_id The id of the factor to be observed
 * @return The minimum selectable level of the specified factor
 */
safe_selector_level.prototype.getMinLevel = function(factor_id){
    return this.levelList[factor_id][0];
}

/**
 * Returns the maximum selectable level for the specified factor
 * @param factor_id The id of the factor to be observed
 * @return The maximum selectable level of the specified factor
 */
safe_selector_level.prototype.getMaxLevel = function(factor_id){
    return this.levelList[factor_id][this.levelList[factor_id].length - 1 ];
}
/**-------------------------------------
 * A metric selector object 
 * -------------------------------------
 *
 * An object prototype which allows the selection of metrics
 * which are stored in the database.
 */
safe.selector.metric = safe_selector_metric;

/**
 * Creates a new selection object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_metric(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.metricList = {};
    this.colorList = {};
    this.distinctionId = undefined;
}

/**
 * Reads data from the database about available metrics 
 * @param callBack A function to be called when the read is complete
 */
safe_selector_metric.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/metrics/", function(error, data){
        if(error) { console.warn(error) }

        data.forEach(function(metric){
            this.metricList[metric.id] = metric;
        }.bind(this));
        callBack();
    }.bind(this));
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_metric.prototype.initComponents = function(){
    this.dSelect = this.dParent
        .append("div")
        .attr("class", "container")
        .style("padding", "0px")
        .style("width", "100%");
    this.eSelect = this.dSelect[0][0];

    this.dSelect.append("div").attr("class", "clearfix");
    this.dSelect.append("div").style("height", "0.5cm");

    select_items = this.dSelect.selectAll(".div").data(Object.keys(this.metricList)).enter()
        .append("div")
        .attr("class", "col-xs-12")
        .style({"padding": "0px", "cursor": "pointer"});

    select_items.append("i")
        .attr("class", function(d) { return "glyphicon glyphicon-unchecked select_metric_square_" + d }.bind(this))
        .attr("metric_id", function(d) { return d }.bind(this))
        .on("click", this.selectChange.bind(this));

    select_items.append("label")
        .attr("class", "col-xs-11")
        .attr("metric_id", function(d) {  return d }.bind(this))
        .text(function(d) { return this.metricList[d].name}.bind(this));

    this.dSelect.append("div").attr("class", "clearfix");
    this.dSelect.append("div").style("height", "0.5cm");
}

/**
 * Sets the color to be associated with a given metric and distinctionId
 * @param distinctionId A unique string to be used to distinguish metrics of the same type
 *        that are from different runs/factors/designpoints/ etc.
 * @param metric_id The id of the metric to be assigned
 * @param color The color to be associated with the metric
 */
safe_selector_metric.prototype.setMetricColor = function(distinctionId, metric_id, color){
    if(this.colorList[distinctionId] == undefined) { this.colorList[distinctionId] = {} }
    this.colorList[distinctionId][metric_id] = color;
    this.updateSelectedMetrics();
}

/**
 * Updates the colors associated with the metrics based on the current
 * distinction id.
 */
safe_selector_metric.prototype.updateSelectedMetrics = function(){
    Object.keys(this.metricList).forEach(function(metric_id){
        if(this.colorList[this.distinctionId] != undefined && this.colorList[this.distinctionId][metric_id] != undefined){
            this.dParent.selectAll(".select_metric_square_" + metric_id)
                .classed("glyphicon-unchecked", false)
                .classed("glyphicon-check", true)
                .style("color", this.colorList[this.distinctionId][metric_id]);
        }else{
            this.dParent.selectAll(".select_metric_square_" + metric_id)
                .classed("glyphicon-unchecked", true)
                .classed("glyphicon-check", false)
                .style("color", "black");
        }
    }.bind(this));
}

/**
 * Sets the distinction id to be used currently
 * @param distinctionId The unique string which identifies metrics
 *        which are to be displayed currently.
 */
safe_selector_metric.prototype.setDistinctionId = function(distinctionId){
    this.distinctionId = distinctionId;
    this.updateSelectedMetrics();
}

/**
 * Called when a metric is selected
 */
safe_selector_metric.prototype.selectChange = function(){
    var metric_id = d3.event.target.getAttribute("metric_id");
    this.callBack(this.metricList[metric_id]);
}

/**
 * Returns the metric object associated with the given metric_id
 * @param metric_id The id of the metric to be returned
 * @return The metric object associated with the given metric_id
 */
safe_selector_metric.prototype.getMetric = function(metric_id){
    return this.metricList[metric_id];
}

/**
 * Clears all colors from the metric selector
 * @param distinctionId The unique string which identifies the set
 *      of metric to clear
 */
safe_selector_metric.prototype.clear = function(distinctionId){
    delete this.colorList[distinctionId];
    this.updateSelectedMetrics();
}
/**-------------------------------------
 * A graph options object
 * -------------------------------------
 *
 * An object prototype which allows the selection of various
 * setting for a currently selected graph
 */
safe.selector.options = safe_selector_options;

/**
 * Creates a new selection object
 * @param exp_id The experiment id with which this selector is associated
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_options(titleElement, parentElement){
    this.titleElement = safe.tools.d3Object(titleElement);
    this.dParent = safe.tools.d3Object(parentElement);

    this.dSelf = this.dParent.append("div")
        .attr("class", "container well")
        .style("width", "95%");
    this.eSelf = this.dSelf[0][0];

    this.initComponents();
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_options.prototype.initComponents = function(){
    var hider = new safe.tools.hider(this.dSelf, this.dParent);
    hider.initComponents();

    this.dSelf.append("label")
        .attr("class", "col-xs-12") 
        .text("Chart Title:");

    var chartTitle = this.dSelf.append("input")
        .attr("class", "form-control")
        .attr("value", this.titleElement[0][0].innerText)
        .on("change", this.titleChange.bind(this));

    var xAxis = this.dSelf.append("div");
    xAxis.append("label").attr("class", "col-xs-12").text("X Axis Label:");
    var xAxisLabel = xAxis.append("input").attr("class", "form-control").on("change", this.labelChange.bind(this));

    var yAxis = this.dSelf.append("div");
    yAxis.append("label").attr("class", "col-xs-12").text("Y Axis Label:");
    var yAxisLabel = yAxis.append("input").attr("class", "form-control").on("change", this.labelChange.bind(this));

    this.dSelf.append("label").attr("class", "col-xs-12").text("Selected Graph Height:");
    var selectedHeight = this.dSelf.append("input")
        .attr("class", "form-control")
        .attr("type", "number")
        .attr("min", 0)
        .on("change", this.selectedHeightChange.bind(this));

    this.dSelf.append("label").attr("class", "col-xs-12").text("Context Graph Height:");
    var contextHeight = this.dSelf.append("input")
        .attr("class", "form-control")
        .attr("type", "number")
        .attr("min", 0)
        .on("change", this.contextHeightChange.bind(this));

    this.dSelf.append("label").attr("class", "col-xs-12").text("Move Selected Graph:");
    var moveUp = this.dSelf.append("button").attr("class", "btn btn-default col-xs-6").text("UP").on("click", this.moveSelectedUp.bind(this));
    var moveDown = this.dSelf.append("button").attr("class", "btn btn-default col-xs-6").text("DOWN").on("click", this.moveSelectedDown.bind(this));

    this.dSelf.append("label").attr("class", "col-xs-12").text("Download Data:");
    var downloadData = this.dSelf.append("button")
        .attr("class", "btn btn-default col-xs-12")
        .text("Download JSON Data")
        .on("click", this.downloadData.bind(this));

    var downloadSvg = this.dSelf.append("button")
        .attr("class", "btn btn-default col-xs-12")
        .text("Download Graph SVG")
        .on("click", function(){ if(this.selectedGraph){ safe.tools.downloadElement(this.selectedGraph.svg[0][0], "graph.svg")} }.bind(this));

    this.chartTitle = chartTitle[0][0];
    this.xAxisLabel = xAxisLabel[0][0];
    this.yAxisLabel = yAxisLabel[0][0];
    this.selectedGraphHeight = selectedHeight[0][0];
    this.contextGraphHeight = contextHeight[0][0];
    this.moveUp = moveUp[0][0];
    this.moveDown = moveDown[0][0];
    this.downloadData = downloadData[0][0];
    this.downloadSvg = downloadSvg[0][0];

    this.contextGraphHeight.disabled = true;
    this.selectedGraphHeight.disabled = true;
}

/**
 * Sets an object to be treated as the curretnly selected graph
 * @param graph The graph to be treated as the currently selected graph
 */
safe_selector_options.prototype.setSelectedGraph = function(graph){
    this.selectedGraph = graph;
    this.selectedGraphHeight.value = graph.getHeight();
    this.setLabels();
    this.selectedGraphHeight.disabled = false;
}

/**
 * Sets an object to be treated as the context graph
 * @param graph The graph to be treated as the context graph
 */
safe_selector_options.prototype.setContextGraph = function(graph){
    this.contextGraph = graph;
    this.contextGraphHeight.value = graph.getHeight();
    this.contextGraphHeight.disabled = false;
}

/**
 * Sets the title of the grpah to the currently set title
 */
safe_selector_options.prototype.titleChange = function(){
    if(!this.titleElement){ return }
    this.titleElement.text(this.chartTitle.value);
}


/**
 * Called when a label text box has been changed
 */
safe_selector_options.prototype.labelChange = function(){
    if(this.selectedGraph){
        this.selectedGraph.setXAxisLabel(this.xAxisLabel.value);
        this.selectedGraph.setYAxisLabel(this.yAxisLabel.value);
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.xAxisLabel.value);
        this.contextGraph.setYAxisLabel(this.yAxisLabel.value);
    }
}

/**
 * Sets the label text boxes based on the currently selected graph
 */
safe_selector_options.prototype.setLabels = function(){
    if(this.selectedGraph){
        this.xAxisLabel.value = this.selectedGraph.getXAxisLabel();
        this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.selectedGraph.getXAxisLabel());
        this.contextGraph.setYAxisLabel(this.selectedGraph.getYAxisLabel());
    }
}

/**
 * Switches the positions of the currently selected graph and its
 * previous sibling in the dom tree. Also moves the context to its
 * correct position.
 */
safe_selector_options.prototype.moveSelectedUp = function(){
    if(!this.selectedGraph){ return }

    var selected = this.selectedGraph.svg[0][0];
    selected.parentNode.insertBefore(selected, selected.previousSibling);

    if(this.contextGraph){
        var context = this.contextGraph.svg[0][0];
        context.parentNode.insertBefore(context, context.previousSibling);
    }
}

/**
 * Switches the positions of the currently selected graph and its
 * next sibling in the dom tree. Also moves the context to its
 * correct position.
 */
safe_selector_options.prototype.moveSelectedDown = function(){
    if(!(this.selectedGraph)){ return }

    var selected = this.selectedGraph.svg[0][0];

    if(this.contextGraph){
        var context = this.contextGraph.svg[0][0];
        if(context.nextSibling){ 
            context.parentNode.insertBefore(context, context.nextSibling.nextSibling);
            selected.parentNode.insertBefore(selected, selected.nextSibling.nextSibling);
        }else{
            context.parentNode.insertBefore(context, context.parentNode.firstChild);
            selected.parentNode.insertBefore(selected, selected.parentNode.firstChild);
        }
   }else{
        if(selected.nextSibling){
            selected.parentNode.insertBefore(selected, selected.nextSibling.nextSibling);
        }else{
            selected.parentNode.insertBefore(selected, selected.parentNode.firstChild);
        }
   }
}

/**
 * Called when the selected graph height selector is changed and reasigns the
 * height of the graph accordingly.
 */
safe_selector_options.prototype.selectedHeightChange = function(){
    if(this.selectedGraph){
        this.selectedGraph.setHeight(parseInt(this.selectedGraphHeight.value));
        this.selectedGraph.displayAll();
    }
}

/**
 * Called when the context graph height selector is changed and reasigns the
 * height of the graph accordingly.
 */
safe_selector_options.prototype.contextHeightChange = function(){
    if(this.contextGraph){
        this.contextGraph.setHeight(parseInt(this.contextGraphHeight.value));
        this.contextGraph.displayAll();

        this.contextGraph.svg.selectAll(".brush").selectAll("rect")
            .attr("y", -6)
            .attr("height", parseInt(this.contextGraphHeight.value - this.contextGraph.margin.bottom - this.contextGraph.margin.top) + 7);
    }
}

/**
 * Makes that the user can no longer move graphs up and down
 */
safe_selector_options.prototype.disableGraphMovement = function(){
    this.moveUp.disabled = true;
    this.moveDown.disabled = true;
}

/**
 * Ensures that the user can move graphs up and down
 */
safe_selector_options.prototype.enableGraphMovement = function(){
    this.moveUp.disabled = false;
    this.moveDown.disabled = false;
}

safe_selector_options.prototype.downloadData = function(){
    var data = "text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(this.selectedGraph.getData()));
    //var link = d3.select("#container").append("a")
    //    .attr("href", "data:" + data)
    //    .attr("download", "data.json");
    //var element = link[0][0];
    var element = document.createElement("a");
    element.setAttribute("href", "data:" + data);
    element.setAttribute("download", "data.json");
    element.click();
    //$('<a id="dataDownload" href="data:' + data + '" download="data.json"></a>').appendTo('#container');
    //$("#dataDownload")[0].click();
}
/**-------------------------------------
 * A results selector object 
 * -------------------------------------
 *
 * An object prototype which allows the selection of filtering options
 * for results that are returned from the database
 */
safe.selector.results = safe_selector_results;

/**
 * Creates a new selection object
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added
 */
function safe_selector_results(parentElement){
    this.exp_id = exp_id;
    this.dParent = safe.tools.d3Object(parentElement);
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_results.prototype.initComponents = function(){

    this.dSelect = this.dParent.append("div");
    this.eSelect = this.dSelect[0][0];

    var data = [{name: "Filter Step:"},
                {name: "Max Results:"},
                {name: "Moving Average:"},
                {name: "Outlier Filter:"}];

    var lines = this.dSelect.selectAll(".key_line").data(data);

    lines = lines.enter().append("div")
        .attr("class", "col-xs-12")
        .style("padding", "0px");

    lines
        .append("label")
        .attr("class", "col-xs-6")
        .text(function(d) { return d.name });

    var inputs = lines
        .append("div")
        .attr("class", "col-xs-6")
            .append("input")
            .attr("type", "number")
            .attr("min", 0)
            .attr("value", 0)
            .attr("class", "form-control");

    this.clusterSize = inputs[0][0];
    this.maxResults = inputs[0][1];
    this.denoiseWindowSize = inputs[0][2];
    this.outlierFilter = inputs[0][3];
}

/**
 * Returns the currently selected cluster size option
 * @return The currently selected cluster size option
 */
safe_selector_results.prototype.getClusterSize = function(){
    if (this.clusterSize.value == ""){ this.clusterSize.value = 0 }
    return this.clusterSize.value;
}

/**
 * Returns the currently selected maximum results setting
 * @return The curretnly selected maximum results setting
 */
safe_selector_results.prototype.getMaxResults = function(){
    if (this.maxResults.value == ""){ this.maxResults.value = 0 }
    return this.maxResults.value;
}

/**
 * Returns the currently selected denoise window setting
 * @return The curretnly selected denoise window setting
 */
safe_selector_results.prototype.getDenoiseWindowSize = function(){
    if (this.denoiseWindowSize.value == ""){ this.denoiseWindowSize.value = 0 }
    return this.denoiseWindowSize.value;
}

/**
 * Returns the currently selected outlier filter setting
 * @return The curretnly selected outlier filter setting
 */
safe_selector_results.prototype.getFilterOutliers = function(){
    if (this.outlierFilter.value == "" || parseFloat(this.outlierFilter.value) < 0){ this.outlierFilter.value = 0 }
    return this.outlierFilter.value;
}

/**
 * Returns an object of all results parameters
 * @return An object of all results parameters in the form
 *      {<parameter_name>:<parameter_value>, ...}
 */
safe_selector_results.prototype.getResultsParameters = function(){
    return {"filter_step": this.getClusterSize(),
            "max_results":this.getMaxResults(),
            "denoise_window":this.getDenoiseWindowSize(),
            "filter_outliers":this.getFilterOutliers()};
}
/**-------------------------------------
 * A run selector object 
 * -------------------------------------
 *
 * An object prototype which allows the selection of runs 
 * which are stored in the database.
 */
safe.selector.run = safe_selector_run;

/**
 * Creates a new selection object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_run(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.runsByDesignPoint = {};
}

/**
 * Reads data from the database about available runs 
 * @param desinpointId The design point for which the runs are to be read
 * @param callBack A function to be called when the read is complete
 */
safe_selector_run.prototype.readData = function(designpointId, callBack){
    d3.json("/safe/designpoint/" + designpointId + "/simruns/", function(error, data){
        if(error) { console.warn(error) }

        this.runsByDesignPoint[designpointId] = data;
        callBack();
    }.bind(this));
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_run.prototype.initComponents = function(attributes){
    this.dSelect = this.dParent
        .append("select")
        .attr(attributes)
        .attr("class", "form-control")
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];
}

/**
 * Updates the currently displayed runs based on data in the database
 * @param desinpointId The design point for which the runs are to be displayed 
 * @param callBack A function to be called after the update is complete
 */
safe_selector_run.prototype.updateRuns = function(designpointId, callBack){
    var update = function(){
        runs = this.dSelect.selectAll("option").data(this.runsByDesignPoint[designpointId]);
        runs.enter().append("option");
        runs.attr("run", function(d) { return d.simrun_id }).text(function(d) { return "Run " + d.simrun_id });
        runs.exit().remove();
        callBack();
    }.bind(this);
    
    if(this.runsByDesignPoint[designpointId] == undefined){
        this.readData(designpointId, update);
    }else{
        update();
    }
}

/**
 * Called when there is a change in the currently selected run
 */
safe_selector_run.prototype.selectChange = function(){
    this.callBack(this.getSelectedRun());
}

/**
 * Returns the currently selected run
 * @return The curretnly selected run
 */
safe_selector_run.prototype.getSelectedRun = function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("run");
}
/**-------------------------------------
 * Various SAFE tools 
 * -------------------------------------
 *
 * A collection of helpful functions for the SAFE
 * visualization tools
 */

safe.tools.confidence = safe_tools_confidence;
safe.tools.confidenceError = safe_tools_confidenceError;
safe.tools.css2json = safe_tools_css2json;
safe.tools.d3Object = safe_tools_d3Object;
safe.tools.embedCSS = safe_tools_embedCSS;
safe.tools.downloadElement = safe_tools_downloadElement;
safe.tools.getCSS = safe_tools_getCSS;
safe.tools.qqplot = safe_tools_qqplot;
safe.tools.qqnorm = safe_tools_qqnorm;
safe.tools.quartiles = safe_tools_quartiles;
safe.tools.stdev = safe_tools_stdev;
safe.tools.variance = safe_tools_variance;


/**
 * Selects a d3 object based on the given element
 * @param element A dom ID, d3 object, or dom element to be found
 * @return A d3 object for the specified element
 */
function safe_tools_d3Object(element){
    if(element.tagName){
        return d3.select(element);
    }else if(typeof(element) == "string" || element instanceof String){
        return d3.select("#" + element);
    }else{
        return element;
    }
}

/**
 * Returns the variance of a set of numbers
 * @param data A list of objects or numbers
 * @param accessor A function that, when called on on of the objects,
 *          will return a numerical value to be used in the computation.
 *          If this value is undefined, it is assumed that the objects
 *          themselves are numerical.
 */
function safe_tools_variance(data, accessor){
    if(!accessor){ accessor = function(d){ return d } }

    var mean = d3.mean(data, accessor);
    return d3.sum(data, function(d) { return Math.pow(accessor(d) - mean, 2 ) / (data.length - 1) });
}

/**
 * Returns the standard deviation of a set of numbers
 * @param data A list of objects or numbers
 * @param accessor A function that, when called on on of the objects,
 *          will return a numerical value to be used in the computation.
 *          If this value is undefined, it is assumed that the objects
 *          themselves are numerical.
 */
function safe_tools_stdev(data, accessor){
    return Math.sqrt(safe.tools.variance(data, accessor));
}

/**
 * Returns the confidence error
 * @param zScore The zScore for the data set
 * @param dev The deviation for the set of numbers
 * @param n The number of elements in the set
 */
function safe_tools_confidenceError(zScore, dev, n){
    return zScore * dev / Math.sqrt(n);
}

/**
 * Calculates the confidence intervals for the given data.
 * @paran data The data for which the intervals are to be calculated
 * @param zScore The zScore representing the desired confidence
 * @param xAccessor A function used on each element to extract the x value
 * @param yAccessor A function used on each element to extract the y value
 * @return An array of elements by x-value of the form [{"x":__, "lowY":__, "midY":__, "highY":__}, ...]
 */
function safe_tools_confidence(data, zScore, accessor){
    if(!accessor){ accessor = function(d){ return d } }
    
    returnData = [];
    var mean = d3.mean(data, accessor);
    var stdev = safe.tools.stdev(data, accessor);
    var error = safe.tools.confidenceError(zScore, stdev, data.length);

    return [ mean - error, mean, mean + error ];
}

/**
 * Calculates the quartiles for the given data.
 * @paran data The data for which the intervals are to be calculated
 * @param xAccessor A function used on each element to extract the x value
 * @param yAccessor A function used on each element to extract the y value
 * @return An array of elements by x-value of the form [{"x":__, "minY":__, "lowY":__, "midY":__, "highY":__, "maxY":__}, ...]
 */
function safe_tools_quartiles(data, accessor){
    if(!accessor){ accessor = function(d){ return d } }

    currentData = data.map(function(d){ return accessor(d) })
    currentData.sort(function(a, b){ return a - b });
    returnData = [];

    returnData.push(d3.quantile(currentData, 0));
    returnData.push(d3.quantile(currentData, 0.25));
    returnData.push(d3.quantile(currentData, 0.5));
    returnData.push(d3.quantile(currentData, 0.75));
    returnData.push(d3.quantile(currentData, 1));

    return returnData;
}

/**
 * Computes the points of a qqplot for a given set of data
 * with a normal distribution
 * @param givenData A list of objects or numbers
 * @param accessor A function that, when called on on of the objects,
 *          will return a numerical value to be used in the computation.
 *          If this value is undefined, it is assumed that the objects
 *          themselves are numerical.
 * @return A list of values in the form [{"x":__, "y":__, "norm":__}, ...]
 */
function safe_tools_qqnorm(givenData, accessor){
    if(!accessor){ accessor = function(d){ return d } }

    var results = [];
    var hypothetical = [];
    var data = [];

    givenData.forEach(function(d){ data.push(accessor(d))});
    data.sort(function(a, b){ return a - b });
    
    var mean = d3.mean(data);
    var n = data.length;
    var stdev = safe.tools.stdev(data);
    //var myString = "";
    //data.forEach(function(d){ myString += d + "\n" });
    //console.log(myString);

    data.forEach(function(value, index){
        var prob = (index + 1) / (n + 1);
        var normQuantile = normsInv(prob, mean, stdev);
        var devs = (normQuantile - mean)/stdev;
        results.push({"x":devs, "y":value, "norm":normQuantile});
    });
    return results;
}

/**
 * Computes the points of a qqplot for a one data set against the other 
 *
 * @param dataSet1 A list of objects or numbers
 * @param dataSet2 A list of objects or numbers
 * @param accessor A function that, when called on on of the objects,
 *          will return a numerical value to be used in the computation.
 *          If this value is undefined, it is assumed that the objects
 *          themselves are numerical.
 * @return A list of values in the form [{"x":__, "y":__}, ...]
 */
function safe_tools_qqplot(dataSet1, dataSet2, accessor){
    if(!accessor){ accessor = function(d){ return d } }

    var results = [];
    var data1 = [];
    var data2 = [];

    dataSet1.forEach(function(d){ data1.push(accessor(d))});
    dataSet2.forEach(function(d){ data2.push(accessor(d))});

    data1.sort(function(a, b){ return a - b });
    data2.sort(function(a, b){ return a - b });

    var n = data1.length;
    data1.forEach(function(value, index){
        var prob = (index + 1) / (n + 1);
        var quan1 = d3.quantile(data1, prob);
        var quan2 = d3.quantile(data2, prob);
        results.push({"x":quan1, "y":quan2});
    });

    return results;
}

/**
 * Takes the given element and includes all associated css styles
 * into the html code. The result is then given for download.
 * @param element The dom element to be downloaded (or jQuery selector for the element)
 * @param fileName The name to be given to the downloaded file
 */
function safe_tools_downloadElement(element, fileName){
    var serializer = new XMLSerializer;
    var element = element.cloneNode(true);
    safe.tools.embedCSS($(element));
    $(element).css("border", "1px solid black");

    var data = "text/svg;charset=utf-8," + serializer.serializeToString(element);
    var link = document.createElement("a");
    link.setAttribute("href", "data:" + data);
    link.setAttribute("download", fileName);
    link.click();
}

/**
 * Takes dom element and embeds all associated css styles into
 * the html code. This allows an element to be downloaded without
 * the loss of style components.
 * @param element The element to embed or its jquery selector
 */
function safe_tools_embedCSS(element){
    var style = safe.tools.getCSS($(element));
    $(element).css(style);
    var children = element.children("*");
    if(children.length > 0){
        children.each(function(i, d){ safe.tools.embedCSS($(d)) });
    }
}

/**
 * Determines all css styles associated with a given element
 * @param a A jquery object for which the css is to be found
 * @return A javascript object containing the styles of the object
 */
function safe_tools_getCSS(a){
    var sheets = document.styleSheets, o = {};
    for (var i = 0; i < sheets.length; i++){
        if(sheets[i].href.substring(sheets[i].href.length - 13) == "bootstrap.css"){
            continue;
        }
        var rules = sheets[i].rules || sheets[i].cssRules;
        for(var r = 0; r < rules.length; r++){
            if( a.is(rules[r].selectorText)){
                o = $.extend(o, 
                             safe.tools.css2json(rules[r].style),
                             safe.tools.css2json(a.attr("style")));
            }
        }
    }
    return o;
}

/**
 * Converts a css style to a javascript/json object
 * @param style The style to be converted
 * @return The associated javascript object
 */
function safe_tools_css2json(style){
    var s = {};
    if(!style) return s;
    if(style instanceof CSSStyleDeclaration){
        for(var i in style){
            if((style[i]).toLowerCase){
                s[(style[i]).toLowerCase()] = (style[style[i]]);
            }
        }
    } else if (typeof style == "string"){
        style = style.split("; ");
        for(var i in style){
            var x = style[i].split(": ");
            s[x[0].toLowerCase()] = (x[1]);
        }
    }
    return s;
}

/// Original C++ implementation found at http://www.wilmott.com/messageview.cfm?catid=10&threadid=38771
/// C# implementation found at http://weblogs.asp.net/esanchez/archive/2010/07/29/a-quick-and-dirty-implementation-of-excel-norminv-function-in-c.aspx
/*
 *     Compute the quantile function for the normal distribution.
 *
 *     For small to moderate probabilities, algorithm referenced
 *     below is used to obtain an initial approximation which is
 *     polished with a final Newton step.
 *
 *     For very large arguments, an algorithm of Wichura is used.
 *
 *  REFERENCE
 *
 *     Beasley, J. D. and S. G. Springer (1977).
 *     Algorithm AS 111: The percentage points of the normal distribution,
 *     Applied Statistics, 26, 118-121.
 *
 *      Wichura, M.J. (1988).
 *      Algorithm AS 241: The Percentage Points of the Normal Distribution.
 *      Applied Statistics, 37, 477-484.
 */
 
function normsInv(p, mu, sigma)
{
    if (p < 0 || p > 1)
    {
        throw "The probality p must be bigger than 0 and smaller than 1";
    }
    if (sigma < 0)
    {
        throw "The standard deviation sigma must be positive";
    }
 
    if (p == 0)
    {
        return -Infinity;
    }
    if (p == 1)
    {
        return Infinity;
    }
    if (sigma == 0)
    {
        return mu;
    }
 
    var q, r, val;
 
    q = p - 0.5;
 
    /*-- use AS 241 --- */
    /* double ppnd16_(double *p, long *ifault)*/
    /*      ALGORITHM AS241  APPL. STATIST. (1988) VOL. 37, NO. 3
 
            Produces the normal deviate Z corresponding to a given lower
            tail area of P; Z is accurate to about 1 part in 10**16.
    */
    if (Math.abs(q) <= .425)
    {/* 0.075 <= p <= 0.925 */
        r = .180625 - q * q;
        val =
               q * (((((((r * 2509.0809287301226727 +
                          33430.575583588128105) * r + 67265.770927008700853) * r +
                        45921.953931549871457) * r + 13731.693765509461125) * r +
                      1971.5909503065514427) * r + 133.14166789178437745) * r +
                    3.387132872796366608)
               / (((((((r * 5226.495278852854561 +
                        28729.085735721942674) * r + 39307.89580009271061) * r +
                      21213.794301586595867) * r + 5394.1960214247511077) * r +
                    687.1870074920579083) * r + 42.313330701600911252) * r + 1);
    }
    else
    { /* closer than 0.075 from {0,1} boundary */
 
        /* r = min(p, 1-p) < 0.075 */
        if (q > 0)
            r = 1 - p;
        else
            r = p;
 
        r = Math.sqrt(-Math.log(r));
        /* r = sqrt(-log(r))  <==>  min(p, 1-p) = exp( - r^2 ) */
 
        if (r <= 5)
        { /* <==> min(p,1-p) >= exp(-25) ~= 1.3888e-11 */
            r += -1.6;
            val = (((((((r * 7.7454501427834140764e-4 +
                       .0227238449892691845833) * r + .24178072517745061177) *
                     r + 1.27045825245236838258) * r +
                    3.64784832476320460504) * r + 5.7694972214606914055) *
                  r + 4.6303378461565452959) * r +
                 1.42343711074968357734)
                / (((((((r *
                         1.05075007164441684324e-9 + 5.475938084995344946e-4) *
                        r + .0151986665636164571966) * r +
                       .14810397642748007459) * r + .68976733498510000455) *
                     r + 1.6763848301838038494) * r +
                    2.05319162663775882187) * r + 1);
        }
        else
        { /* very close to  0 or 1 */
            r += -5;
            val = (((((((r * 2.01033439929228813265e-7 +
                       2.71155556874348757815e-5) * r +
                      .0012426609473880784386) * r + .026532189526576123093) *
                    r + .29656057182850489123) * r +
                   1.7848265399172913358) * r + 5.4637849111641143699) *
                 r + 6.6579046435011037772)
                / (((((((r *
                         2.04426310338993978564e-15 + 1.4215117583164458887e-7) *
                        r + 1.8463183175100546818e-5) * r +
                       7.868691311456132591e-4) * r + .0148753612908506148525)
                     * r + .13692988092273580531) * r +
                    .59983220655588793769) * r + 1);
        }
 
        if (q < 0.0)
        {
            val = -val;
        }
    }
 
    return mu + sigma * val;
}
 
 
/**----------------------------------
 * A graph for histograms
 *----------------------------------
 */

safe.graph.barchart = safe_graph_barchart;

safe_graph_barchart.prototype = Object.create(safe.graph.base.prototype);
safe_graph_barchart.constructor = safe_graph_barchart;

/**
 * Creates a new bar chart object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_barchart(margins, width, height, onSelectedCallBack, parentElement){
    safe.graph.base.apply(this, [margins, width, height, onSelectedCallBack, parentElement]);

    // Set x axis as discrete (not continuous as by default)
    this.xScale = d3.scale.ordinal().rangeRoundBands([0, this.width], 0.05);
    this.yAxis.scale(this.xScale);

    this.infoFormat = d3.format(".4f");
    this.tickFormat = d3.format(".2f");
    this.tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    return this;
}

/** Displays all data asssociated with this graph */
safe_graph_barchart.prototype.displayAll = function(){
    this.display();
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_barchart.prototype.display = function(){
    if(this.data == undefined){ return }

    var bars = this.svg.selectAll(".bar").data(this.data);
    bars.enter().append("g").attr("class", "bar");
    bars.exit().remove();

    bars.selectAll("rect").remove();
    bars.append("rect");

    var rects = bars.selectAll("rect");
    rects
        .attr("x", function(d) { return this.margin.left + this.xScale(d.x  + 0.05 * d.dx) }.bind(this))
        .attr("y", function(d) { return this.margin.top + this.yScale(d.y)}.bind(this))
        .attr("width", function(d) { return this.xScale(this.xScale.domain()[0] + d.dx) * 0.9 }.bind(this))
        .attr("height", function(d) { return this.height - this.yScale(d.y) }.bind(this))
        .attr("info", function(d) { return this.infoFormat(d.y) + "</br>" + 
                                    this.infoFormat(d.x) + " - " + this.infoFormat(parseFloat(d.x) + parseFloat(d.dx)) }.bind(this))
        .style(this.style)
        .on("mouseover", this.mouseOverBox.bind(this))
        .on("mouseout", this.mouseOutBox.bind(this));

    bars.selectAll("text").remove();
    bars.append("text");

    var text = bars.selectAll("text");
    text
        .attr("x", function(d) { return this.margin.left + this.xScale(d.x) }.bind(this))
        .attr("y", function(d) { return this.margin.top + this.yScale(d.y)}.bind(this))
        .attr("dy", "20px")
        .attr("dx", function(d) { return this.xScale(this.xScale.domain()[0] + (d.dx/2)); }.bind(this))
        .attr("fill", "#fff")
        .attr("text-anchor", "middle")
        .attr("pointer-events", "none")
        .text(function(d) { return this.tickFormat(d.y) }.bind(this));

}

/** 
 * Adds the specified data set to this graph
 * @param data The data to be added to this graph
 * @param style A set of css attributes that will be applied to each rectangle (bar) on the chart
 * @param xAccessor A function which will return the x value of a point when called with the point as a parameter
 * @param yAccessor A function which will return the y value of a point when called with the point as a parameter
 */
safe_graph_barchart.prototype.setData = function(data, style, xAccessor, yAccessor){
    if(data == undefined){ return }

    this.data = data;
    this.xAccessor = xAccessor;
    this.yAccessor = yAccessor;
    this.style = style;

    this.display();
}

safe_graph_barchart.prototype.update = function(){
    this.display();
}

safe_graph_barchart.prototype.getData = function(){
    return this.data;
}

safe_graph_barchart.prototype.getExtent = function(){
    return this.extent;
}

safe_graph_barchart.prototype.remove = function(){
    this.svg.selectAll(".bar").remove();
}

safe_graph_barchart.prototype.setBinNumber = function(bins){
    this.bins = bins;
    this.update();
}

safe_graph_barchart.prototype.getBinNumber = function(){
    return this.bins;
}

safe_graph_barchart.prototype.setBinWidth = function(width){
    this.bins = Math.round(d3.max(this.data) / width);
    if(this.bins <= 0){ this.bins = 1 }
    this.update();
}

safe_graph_barchart.prototype.getBinWidth = function(){
    if(!(this.histogram && this.histogram.length > 0)){ return 0 }
    return this.histogram[0].dx;
}

safe_graph_barchart.prototype.setFrequency = function(frequency){
    this.frequency = frequency;
    this.update();
}

safe_graph_barchart.prototype.getFrequency = function(){
    return this.frequency;
}

/** Called when mouse is over a histogram box */
safe_graph_barchart.prototype.mouseOverBox = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0.9);

    this.tooltip.html(d3.event.srcElement.getAttribute("info"))
        .style("left", (d3.event.pageX) + "px")
        .style("top", (d3.event.pageY - 28) + "px");
}

/** Called when mouse is no longer over a histogram box */
safe_graph_barchart.prototype.mouseOutBox = function(){
    this.tooltip.transition()
        .duration(200)
        .style("opacity", 0);
}
/**----------------------------------
 * A graph for box and whiskers plots 
 *----------------------------------
 *
 * This object uses given data with a specified format to create
 * a graph with confidence intervals. The data given should be 
 * of the form [{x:__, maxY:__, highY:__, midY:__, lowY:__, minY:__}, ...] 
 */


safe.graph.boxplot = safe_graph_boxplot;

// This plot inherits from the cartesian plot
safe_graph_boxplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_boxplot.prototype.constructor = safe_graph_boxplot;

/**
 * Creates a new boxplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_boxplot(margins, width, height, onSelectedCallBack, parentDomId, pad){
    safe.graph.cartesian.apply(this, [margins, width, height, onSelectedCallBack, parentDomId]);

    this.pad = (pad == undefined) ? false : pad;

    this.line = d3.svg.line()
        .x(function (d) { return this.margin.left + this.xScale(d["x"]) }.bind(this))
        .y(function (d) { return this.margin.top + this.yScale(d["midY"]) }.bind(this));

    this.numberFormat = d3.format(".2f");
    this.boxWidth = 10.0;

    return this;
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_boxplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".dataGroup_" + id).remove();
        this.svg.selectAll(".dataLine_" + id).remove();
        this.svg.selectAll(".dataPath_" + id).remove();
        this.svg.selectAll(".crossLine_" + id).remove();
        this.svg.selectAll(".box_" + id).remove();
        return;
    }

    //============================================
    groups = this.svg.selectAll(".dataGroup_" + id).data(this.data[id]);
    groups.enter().append("g").attr("class", "dataGroup_" + id);
    groups.exit().remove();

    //points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    points = groups.selectAll(".subPoint").data(function(d){ 
        values =
               [ {"cx": d["x"], "cy": d["maxY"]},
                 {"cx": d["x"], "cy": d["minY"]}] ;
        return values; });

    points.enter().append("circle")
        .attr("class", "subPoint")
        .attr(this.pointAttributes[id])
        .attr("info", function(d) { return this.numberFormat(d.cx) + "<br/>" + this.numberFormat(d.cy) }.bind(this))
        .on("mouseover", this.mouseOverPoint.bind(this))
        .on("mouseout", this.mouseOutPoint.bind(this));
    points
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("cx", function (d) { return this.margin.left + this.xScale(d.cx) }.bind(this))
        .attr("cy", function (d) { return this.margin.top + this.yScale(d.cy) }.bind(this))
    points.exit().remove();

    //============================================
 
    lines = groups.selectAll(".dataLine_" + id).data(function(d){
        values = 
                [ {"x": d["x"], "y1": d["highY"], "y2": d["maxY"]},
                  {"x": d["x"], "y1": d["lowY"], "y2": d["minY"]}];
        return values; }.bind(this));

    lines.enter().append("line").attr("class", "dataLine_" + id).attr({"stroke-width":"1", "stroke":this.pointAttributes[id].stroke});
    lines
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("x1", function (d) { return this.margin.left + this.xScale(d["x"]); }.bind(this))
        .attr("x2", function (d) { return this.margin.left + this.xScale(d["x"]); }.bind(this))
        .attr("y1", function (d) { return this.margin.top + this.yScale(d["y1"]); }.bind(this))
        .attr("y2", function (d) { return this.margin.top + this.yScale(d["y2"]); }.bind(this));
    lines.exit().remove();

    crossLines = this.svg.selectAll(".crossLine_" + id).data(this.data[id]);
    crossLines.enter().append("line").attr("class", "crossLine_" + id).attr({"stroke-width":"1", "stroke":this.pointAttributes[id].stroke});
    crossLines
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("x1", function (d) { return this.margin.left + this.xScale(d["x"]) - this.boxWidth/2; }.bind(this))
        .attr("x2", function (d) { return this.margin.left + this.xScale(d["x"]) + this.boxWidth/2; }.bind(this))
        .attr("y1", function (d) { return this.margin.top + this.yScale(d["midY"]); }.bind(this))
        .attr("y2", function (d) { return this.margin.top + this.yScale(d["midY"]); }.bind(this));
    crossLines.exit().remove();

    //=============================================

    boxes = this.svg.selectAll(".box_" + id).data(this.data[id]);

    boxes.enter().append("rect").attr("class", "box_" + id).attr({"stroke-width":2, "stroke":this.pointAttributes[id].stroke});
    boxes
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("x", function(d) { return this.margin.left + this.xScale(d["x"]) - this.boxWidth/2 }.bind(this))
        .attr("y", function(d) { return this.margin.top + this.yScale(d["highY"]) }.bind(this))
        .attr("width", this.boxWidth)
        .attr("height", function(d) { return this.yScale(this.yScale.domain()[1] - (d["highY"] - d["lowY"])) }.bind(this))
        .attr("info", function(d) { return "" + 
                this.numberFormat(d["x"]) + "</br>" + 
                this.numberFormat(d["lowY"]) + " - " + 
                this.numberFormat(d["midY"]) + " - " + 
                this.numberFormat(d["highY"]) }.bind(this))
        .style("fill", "white")
        .style("fill-opacity", 0)
        .on("mouseover", this.mouseOverPoint.bind(this))
        .on("mouseout", this.mouseOutPoint.bind(this));
    boxes.exit().remove();

    //=============================================

    paths = this.svg.selectAll(".dataPath_" + id).data([this.data[id]]);

    if (this.data[id].length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "dataPath_" + id)
            .attr("stroke", this.pointAttributes[id].stroke)
            .attr("stroke-width", 1);

        paths
            .attr("clip-path", "url(#" + this.clipId + ")" )
            .attr("d", function (d) { return this.line(d) }.bind(this));

        paths.exit().remove();
    }else{
        paths.remove();
    }
}

/** Updates information about maximum and minimum value in the data sets */
safe_graph_boxplot.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d["x"] });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yMin = d3.min(this.data[id], function(d) { return d["minY"] });
        yMax = d3.max(this.data[id], function(d) { return d["maxY"] });
        this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
        this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
    }.bind(this));

    this.setDomains([this.xMin, this.xMax], [0, this.yMax]);
}

/**
 * Sets the domain for the x-axis
 * @param domain The domain for the x-axis in the form [xMin, xMax]
 */
safe_graph_boxplot.prototype.setXDomain = function(domain){
    var valLeft, valRight;
    range = parseFloat(domain[1]) - parseFloat(domain[0]);
    if(this.pad && range > 0){
        pad = 0.1 * range;
        valLeft = parseFloat(domain[0]) - pad;
        valRight = parseFloat(domain[1]) + pad;
    }else if (this.pad){
        pad = parseFloat(domain[1]) > 10 ? 10 : this.xMax;
        valLeft = parseFloat(domain[0]) - pad;
        valRight = parseFloat(domain[0]) + pad;
    }else{
        valLeft = parseFloat(domain[0]);
        valRight = parseFloat(domain[1]);
    }

    safe.graph.cartesian.prototype.setXDomain.apply(this, [[valLeft, valRight]]);
}
/**----------------------------------
 * A graph for lineplots
 *----------------------------------
 *
 * This object uses given data with a specified format to create
 * a graph with confidence intervals. The data given should be 
 * of the form [{x:__, highY:__, midY:__, lowY:__}, ...] 
 */


safe.graph.lineplot = safe_graph_lineplot;

// This plot inherits from the cartesian plot
safe_graph_lineplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_lineplot.prototype.constructor = safe_graph_lineplot;

/**
 * Creates a new lineplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_lineplot(margins, width, height, onSelectedCallBack, parentDomId, pad){
    safe.graph.cartesian.apply(this, [margins, width, height, onSelectedCallBack, parentDomId]);

    this.pad = (pad == undefined) ? false : pad;

    this.line = d3.svg.line()
        .x(function (d) { return this.margin.left + this.xScale(d["x"]) }.bind(this))
        .y(function (d) { return this.margin.top + this.yScale(d["midY"]) }.bind(this));

    this.numberFormat = d3.format(".2f");

    return this;
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_lineplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".dataGroup_" + id).remove();
        this.svg.selectAll(".dataLine_" + id).remove();
        this.svg.selectAll(".dataPath_" + id).remove();
        return;
    }

    //============================================
    groups = this.svg.selectAll(".dataGroup_" + id).data(this.data[id]);
    groups.enter().append("g").attr("class", "dataGroup_" + id);
    groups.exit().remove();

    //points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    points = groups.selectAll(".subPoint").data(function(d){ 
        values =
               [ {"cx": d["x"], "cy": d["lowY"]},
                 {"cx": d["x"], "cy": d["midY"]},
                 {"cx": d["x"], "cy": d["highY"]}] ;
        return values; });

    points.enter().append("circle")
        .attr("class", "subPoint")
        .attr(this.pointAttributes[id])
        .attr("info", function(d) { return this.numberFormat(d.cx) + "<br/>" + this.numberFormat(d.cy) }.bind(this))
        .on("mouseover", this.mouseOverPoint.bind(this))
        .on("mouseout", this.mouseOutPoint.bind(this));
    points
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("cx", function (d) { return this.margin.left + this.xScale(d.cx) }.bind(this))
        .attr("cy", function (d) { return this.margin.top + this.yScale(d.cy) }.bind(this))
    points.exit().remove();

    //============================================
 
    lines = this.svg.selectAll(".dataLine_" + id).data(this.data[id]);
    lines.enter().append("line").attr("class", "dataLine_" + id).attr({"stroke-width":"1", "stroke":this.pointAttributes[id].stroke});
    lines
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("x1", function (d) { return this.margin.left + this.xScale(d["x"]); }.bind(this))
        .attr("x2", function (d) { return this.margin.left + this.xScale(d["x"]); }.bind(this))
        .attr("y1", function (d) { return this.margin.top + this.yScale(d["lowY"]); }.bind(this))
        .attr("y2", function (d) { return this.margin.top + this.yScale(d["highY"]); }.bind(this));
    lines.exit().remove();

    //=============================================

    paths = this.svg.selectAll(".dataPath_" + id).data([this.data[id]]);

    if (this.data[id].length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "dataPath_" + id)
            .attr("stroke", this.pointAttributes[id].stroke)
            .attr("stroke-width", 1);

        paths
            .attr("clip-path", "url(#" + this.clipId + ")" )
            .attr("d", function (d) { return this.line(d) }.bind(this));

        paths.exit().remove();
    }else{
        paths.remove();
    }
}

/** Updates information about maximum and minimum value in the data sets */
safe_graph_lineplot.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d["x"] });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yMin = d3.min(this.data[id], function(d) { return d["lowY"] });
        yMax = d3.max(this.data[id], function(d) { return d["highY"] });
        this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
        this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
    }.bind(this));

    this.setDomains([this.xMin, this.xMax], [0, this.yMax]);
}

/**
 * Sets the domain for the x-axis
 * @param domain The domain for the x-axis in the form [xMin, xMax]
 */
safe_graph_lineplot.prototype.setXDomain = function(domain){
    var valLeft, valRight;
    range = parseFloat(domain[1]) - parseFloat(domain[0]);
    if(this.pad && range > 0){
        pad = 0.1 * range;
        valLeft = parseFloat(domain[0]) - pad;
        valRight = parseFloat(domain[1]) + pad;
    }else if (this.pad){
        pad = parseFloat(domain[1]) > 10 ? 10 : this.xMax;
        valLeft = parseFloat(domain[0]) - pad;
        valRight = parseFloat(domain[0]) + pad;
    }else{
        valLeft = parseFloat(domain[0]);
        valRight = parseFloat(domain[1]);
    }

    safe.graph.cartesian.prototype.setXDomain.apply(this, [[valLeft, valRight]]);
}
/**--------------------------------------
 * A graph for time series
 * -------------------------------------
 *
 * This graph take data with {time:__, value:__} pairs and
 * graphs them as a time series
 */

safe.graph.timeplot = safe_graph_timeplot;

// This plot inherits from the cartesian plot
safe_graph_timeplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_timeplot.prototype.constructor = safe_graph_timeplot;

/**
 * Creates a new lineplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 * @param displayPoints If false or undefined, points are not displayed on lines. Otherwise, points are displayed
 *
 * @return The newly created object
 * */
function safe_graph_timeplot(margins, width, height, selectedCallBack, parentDomId, displayPoints){
    safe.graph.cartesian.apply(this, [margins, width, height, selectedCallBack, parentDomId]);

    this.displayPoints = displayPoints == undefined || displayPoints != false ? true : displayPoints;

    this.line = d3.svg.line()
        .x(function(d) { return this.margin.left + this.xScale(this.xAccessor[id](d)) }.bind(this) )
        .y(function(d) { return this.margin.top + this.yScale(this.yAccessor[id](d)) }.bind(this) );

    this.numberFormat = d3.format(".2f");
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_timeplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".circle_" + id).remove();
        this.svg.selectAll(".path_" + id).remove();
        return;
    }

    var paths = this.svg.selectAll(".path_" + id).data([this.data[id]]);

    paths.enter().append("path")

    paths
        .attr("d", function(d){ return this.line(d) }.bind(this))
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("class", "path_" + id)
        .attr({"stroke-width":"1", "stroke":this.pointAttributes[id].stroke });

    paths.exit().remove();

    if(this.displayPoints == true){
        var circles = this.svg.selectAll(".circle_" + id).data(this.data[id]);

        circles.enter().append("circle")
            .attr("class", "circle_" + id)
            .attr(this.pointAttributes[id])
            .attr("clip-path", "url(#" + this.clipId + ")" );
        
        var counter = 0;
        circles.each(function(d) { if(this.xAccessor[id](d) > this.xScale.domain()[0] && this.xAccessor[id](d) < this.xScale.domain()[1]) counter++; }.bind(this));
        var scaleFactor = this.getScaleFactor(0.1, counter);
        var position = 0;

        circles 
            .attr("cx", function(d) { return this.margin.left + this.xScale(this.xAccessor[id](d)) }.bind(this))
            .attr("cy", function(d) { return this.margin.top + this.yScale(this.yAccessor[id](d)) }.bind(this))
            .attr("info", function(d) { return this.numberFormat(this.xAccessor[id](d)) + "<br/>" + this.numberFormat(this.yAccessor[id](d)) }.bind(this))
            .attr("visibility", function(d, i){
                var t;
                if((i % scaleFactor == 0) && this.xScale(this.xAccessor[id](d)) >= position) {
                    t = "visible";
                    position = this.xScale(this.xAccessor[id](d)) + 5;
                } else {
                    t = "hidden";
                }
                return t;
                }.bind(this))
            .on("mouseover", this.mouseOverPoint.bind(this))
            .on("mouseout", this.mouseOutPoint.bind(this));

       circles.exit().remove();
    }
}

/** 
 * Adds the specified data set to this graph (must be followed by display() or displayAll())
 * @param id A string that distinguishes this data set from all others on this graph
 * @param data The data to be added to this graph
 * @param pointAttributes A dictionary of html attributes to add to each displayed point
 * @param mouseEvents A dictionary of mouse events to occur when mouse is over a data point
 */
safe_graph_timeplot.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
    data.sort(function(a, b){ return a.time - b.time });
    this.data[id] = data;
    this.pointAttributes[id] = pointAttributes;
    this.toDisplay[id] = true;
    this.updateDataFeatures();
}

/**
 * Gets the scale factor for points on the graph. This number essentially means
 * plot every x-th point.
 *
 * @param pointsPerPixel How many points to plot per pixel (recommended 10)
 * @param size The number of points being plotted
 * @return the scale factor as an integer
 */
safe_graph_timeplot.prototype.getScaleFactor = function(pointsPerPixel, size) {
    return Math.ceil((size / this.width) * pointsPerPixel);
}
/**----------------------------------
 * A graph for scatterplots 
 *----------------------------------
 *
 * This object uses given data with a specified format to create
 * a graph with confidence intervals. The data given should be 
 * of the form [{xValue:__, medianValue:__, highDevValue:__, lowDevValue:__}, ...] 
 */

safe.graph.scatterplot = safe_graph_scatterplot;

// This plot inherits from the cartesian plot
safe_graph_scatterplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_scatterplot.prototype.constructor = safe_graph_scatterplot;

/**
 * Creates a new lineplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_scatterplot(margins, width, height, onSelectedCallBack, parentElement){
    safe.graph.cartesian.apply(this, [margins, width, height, onSelectedCallBack, parentElement]); 
    this.numberFormat = d3.format(".2f");
    return this;
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_scatterplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".dataPoint_" + id).remove();
        return;
    }

    points = this.svg.selectAll(".dataPoint_" + id).data(this.data[id]);
    points.enter()
        .append("circle")
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("class", "dataPoint_" + id)
        .attr(this.pointAttributes[id])
        .on("mouseover", this.mouseOverPoint.bind(this))
        .on("mouseout", this.mouseOutPoint.bind(this));

    points
        .attr("cx", function (d) { return this.margin.left + this.xScale(this.xAccessor[id](d)) }.bind(this))
        .attr("cy", function (d) { return this.margin.top + this.yScale(this.yAccessor[id](d)) }.bind(this))
        .attr("info", function(d) { return this.numberFormat(this.xAccessor[id](d)) + "<br/>" + this.numberFormat(this.yAccessor[id](d)) }.bind(this));

	points.exit().remove();	
}

/**--------------------------------------
 * A graph for time series
 * -------------------------------------
 *
 * This graph take data with {time:__, value:__} pairs and
 * graphs them as a time series
 */

safe.graph.timeplot = safe_graph_timeplot;

// This plot inherits from the cartesian plot
safe_graph_timeplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_timeplot.prototype.constructor = safe_graph_timeplot;

/**
 * Creates a new lineplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 * @param displayPoints If false or undefined, points are not displayed on lines. Otherwise, points are displayed
 *
 * @return The newly created object
 * */
function safe_graph_timeplot(margins, width, height, selectedCallBack, parentDomId, displayPoints){
    safe.graph.cartesian.apply(this, [margins, width, height, selectedCallBack, parentDomId]);

    this.displayPoints = displayPoints == undefined || displayPoints != false ? true : displayPoints;

    this.line = d3.svg.line()
        .x(function(d) { return this.margin.left + this.xScale(this.xAccessor[this.currentId](d)) }.bind(this) )
        .y(function(d) { return this.margin.top + this.yScale(this.yAccessor[this.currentId](d)) }.bind(this) );

    this.numberFormat = d3.format(".2f");
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_timeplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".circle_" + id).remove();
        this.svg.selectAll(".path_" + id).remove();
        return;
    }
    this.currentId = id;

    var paths = this.svg.selectAll(".path_" + id).data([this.data[id]]);

    paths.enter().append("path");

    paths
        .attr("d", function(d){ return this.line(d) }.bind(this))
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("class", "path_" + id)
        .attr({"stroke-width":"1", "stroke":this.pointAttributes[id].stroke });

    paths.exit().remove();

    if(this.displayPoints == true){
        var circles = this.svg.selectAll(".circle_" + id).data(this.data[id]);

        circles.enter().append("circle")
            .attr("class", "circle_" + id)
            .attr(this.pointAttributes[id])
            .attr("clip-path", "url(#" + this.clipId + ")" );
        
        var counter = 0;
        circles.each(function(d) { if(this.xAccessor[id](d) > this.xScale.domain()[0] && this.xAccessor[id](d) < this.xScale.domain()[1]) counter++; }.bind(this));
        var scaleFactor = this.getScaleFactor(0.1, counter);
        var position = 0;

        circles 
            .attr("cx", function(d) { return this.margin.left + this.xScale(this.xAccessor[id](d)) }.bind(this))
            .attr("cy", function(d) { return this.margin.top + this.yScale(this.yAccessor[id](d)) }.bind(this))
            .attr("info", function(d) { return this.numberFormat(this.xAccessor[id](d)) + "<br/>" + this.numberFormat(this.yAccessor[id](d)) }.bind(this))
            .attr("visibility", function(d, i){
                var t;
                if((i % scaleFactor == 0) && this.xScale(this.xAccessor[id](d)) >= position) {
                    t = "visible";
                    position = this.xScale(this.xAccessor[id](d)) + 5;
                } else {
                    t = "hidden";
                }
                return t;
                }.bind(this))
            .on("mouseover", this.mouseOverPoint.bind(this))
            .on("mouseout", this.mouseOutPoint.bind(this));

       circles.exit().remove();
    }
}

/** 
 * Adds the specified data set to this graph (must be followed by display() or displayAll())
 * @param id A string that distinguishes this data set from all others on this graph
 * @param data The data to be added to this graph
 * @param pointAttributes A dictionary of html attributes to add to each displayed point
 * @param mouseEvents A dictionary of mouse events to occur when mouse is over a data point
 */
safe_graph_timeplot.prototype.addDataSet = function(id, data, xAccessor, yAccessor, pointAttributes){
    data.sort(function(a, b){ return xAccessor(a) - xAccessor(b) });
    safe.graph.cartesian.prototype.addDataSet.apply(this, [id, data, xAccessor, yAccessor, pointAttributes]);
}

/**
 * Gets the scale factor for points on the graph. This number essentially means
 * plot every x-th point.
 *
 * @param pointsPerPixel How many points to plot per pixel (recommended 10)
 * @param size The number of points being plotted
 * @return the scale factor as an integer
 */
safe_graph_timeplot.prototype.getScaleFactor = function(pointsPerPixel, size) {
    return Math.ceil((size / this.width) * pointsPerPixel);
}
safe.selector.factor_chooser = safe_selector_factor_chooser;

function safe_selector_factor_chooser(exp_id, callBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = callBack;
    this.parent = safe.tools.d3Object(parentElement);

    this.dSelf = this.parent
        .append("div")
        .attr("class", "col-xs-3 navbar-header navbar-static-top affix")
        .style("outline", "solid 2px black")
        .style("margin-left", "0.25cm")
        .style("background", "white")
        .style("cursor", "move")
        .on("click", this.onMouseClick.bind(this))

    this.dSelf.append("label")
        .text("Select Graph to View")
        .attr("class", "h4 text-center col-xs-12")
        .style("pointer-events", "none");

	this.dSelf
		.append("button")
        .attr("class", "col-xs-6 btn btn-default")
		.text("Add Factor Graph")
		.on("click", this.addFactorGraph.bind(this));

    var div = this.dSelf.append("div")
        .attr("class", "col-xs-6");

    this.selectFactor = new safe.selector.factor(this.exp_id, undefined, div);
    this.selectFactor.readInitialData(this.initComponents.bind(this));
}

safe_selector_factor_chooser.prototype.initComponents = function(){
    this.selectFactor.initComponents();
}

safe_selector_factor_chooser.prototype.addFactorGraph = function(){
	this.callBack(this.getSelectedFactor());
}

safe_selector_factor_chooser.prototype.getSelectedFactor = function(){
	return this.selectFactor.getSelectedFactor();
}

safe_selector_factor_chooser.prototype.getFactorNumber = function(){
    return Object.keys(this.selectFactor.factorList).length
}

safe_selector_factor_chooser.prototype.onMouseMove = function(drag){
    if(drag == true){
        d3.select("body")
          .on("mousemove", function(){
              this.dSelf
                  .style("left", (d3.event.clientX - this.dSelf[0][0].offsetWidth/2 ) + "px")
                  .style("top", (d3.event.clientY - this.dSelf[0][0].offsetHeight/2 ) + "px");
               }.bind(this))
          .on("mouseup", function(){ this.onMouseMove(false) }.bind(this));
       
    }else{
        d3.select("body").on("mousemove", undefined);
    }
}

safe_selector_factor_chooser.prototype.onMouseClick= function(){
    if( d3.event.target == this.dSelf[0][0] ){
        if(this.mouseDrag == undefined || this.mouseDrag == false){
            this.mouseDrag = true;
            this.onMouseMove(true);
        }else{
            this.mouseDrag = false;
            this.onMouseMove(false);
        }
    }
}
/**-------------------------------------
 * A full selector object
 * -------------------------------------
 *
 * An object prototype which includes all selection capabilities
 * currently in place.
 */
safe.selector.full = safe_selector_full;

/**
 * Creates a new full selector object
 * @param exp_id The experiment id with which this selector is to be associated
 * @param selectionCallBack A function to be called when a metric is selected
 * @param applyFilterCallBack A function to be called when a new filter is created
 * @param parementElement The dom id, dom element, or d3 object to which this selector
 *        should be added
 */
function safe_selector_full(exp_id, selectionCallBack, applyFilterCallBack, parentElement, callBack){
        this.exp_id = exp_id;
        this.dParent = safe.tools.d3Object(parentElement);
        this.selectionCallBack = selectionCallBack;
        this.applyFilterCallBack = applyFilterCallBack;
        
        this.dSelf = this.dParent.append("div")
            .attr("class", "well container")
            .style("width", "95%");

        this.levelRangeLeft = document.createElement("div");
        this.levelRangeRight = document.createElement("div");

        this.hider = new safe.tools.hider(this.dSelf, this.dParent);
        this.selectFactor = new safe.selector.factor(this.exp_id, this.factorSelectChange.bind(this), this.dSelf);
        this.selectLevelMin = new safe.selector.level(this.exp_id, undefined, this.levelRangeLeft);
        this.selectLevelMax = new safe.selector.level(this.exp_id, undefined, this.levelRangeRight);
        this.selectDesignPoint = new safe.selector.designpoint(this.exp_id, this.designPointSelectChange.bind(this), this.dSelf);
        this.selectRun = new safe.selector.run(this.exp_id, this.runSelectChange.bind(this), this.dSelf);
        this.selectMetric = new safe.selector.metric(this.exp_id, this.selectElementChange.bind(this), this.dSelf);
        this.selectResults = new safe.selector.results(this.dSelf);

        this.selectFactor.readInitialData(function(){
            this.selectLevelMin.readInitialData(function(){
                this.selectLevelMax.readInitialData(function(){
                    this.selectMetric.readInitialData(function(){
                        this.initComponents();
                        if(callBack){ callBack() }
                    }.bind(this));
                }.bind(this));
            }.bind(this));
        }.bind(this));
}
      
/**
 * Initializes the dom components for this object
 */
safe_selector_full.prototype.initComponents = function(){
    this.hider.initComponents();

    // Initialize the factor list
    this.selectFactor.initComponents();
    this.selectLevelMin.initComponents();
    this.selectLevelMax.initComponents();

    this.dSelf[0][0].appendChild(this.levelRangeLeft);
    this.dSelf[0][0].appendChild(this.levelRangeRight);
    d3.select(this.levelRangeLeft).attr("class", "col-xs-6").style("padding", "0px");
    d3.select(this.levelRangeRight).attr("class", "col-xs-6").style("padding", "0px");
     
    // Initialize the apply selection button
    this.dSelf
        .append("button")
        .attr("class", "btn btn-default col-xs-12")
        .text("Apply Selection Filter")
        .on("click", this.applyDesignFilter.bind(this));

    this.selectDesignPoint.initComponents();
    this.selectRun.initComponents();
    this.selectMetric.initComponents();
    this.selectResults.initComponents();

    this.factorSelectChange();
    this.updateDesignPoints();
}

/**
 * Called when the selected factor is changed
 */
safe_selector_full.prototype.factorSelectChange = function(){

    var factor_id = this.selectFactor.getSelectedFactor().id;

    this.selectLevelMin.updateLevels(factor_id);
    this.selectLevelMax.updateLevels(factor_id);

    this.selectLevelMax.eSelect.selectedIndex = this.selectLevelMax.eSelect.options.length - 1;
}

/**
 * Called when the "Apply Filter" button is clicked
 */
safe_selector_full.prototype.applyDesignFilter = function(){

    var factor = this.selectFactor.getSelectedFactor();
    var min = this.selectLevelMin.getMinLevel(factor.id);
    var max = this.selectLevelMax.getMaxLevel(factor.id);

    if(this.getMinLevel() > this.getMaxLevel() ){
        alert("The factor's minimum level cannot be greater than the factor's maximum level.");
        return;
    }

    var parameter = {"name":factor.name, "factor_id":factor.id, "min_value":this.getMinLevel(), "max_value":this.getMaxLevel() };
    if(this.getMinLevel() > min || this.getMaxLevel() < max ){
        this.selectDesignPoint.addParameter(factor.id, parameter);
    }else{
        this.removeDesignFilter(factor.id);
    }

    this.updateDesignPoints();
    this.applyFilterCallBack(parameter);
}

/**
 * Removes the specified filter from the list of filters
 * @param factor_id The id of the factor with which the filter is associated
 */
safe_selector_full.prototype.removeDesignFilter = function(factor_id){
    this.selectDesignPoint.removeParameter(factor_id);

    this.updateDesignPoints();
}

/**
 * Updates the listed design points based on filters and information in the database
 */
safe_selector_full.prototype.updateDesignPoints = function(){
    this.selectDesignPoint.updateDesignPoints(this.designPointSelectChange.bind(this));
}

/**
 * Called when the selected design point changes
 */
safe_selector_full.prototype.designPointSelectChange = function(){
    this.selectRun.updateRuns(this.selectDesignPoint.getSelectedDesignPoint(), function(){
        this.selectMetric.setDistinctionId(this.getSelectedRun());
        this.selectMetric.updateSelectedMetrics();  
    }.bind(this));
}

/**
 * Called when the selected run is changed
 */
safe_selector_full.prototype.runSelectChange = function(){
    this.selectMetric.setDistinctionId(this.getSelectedRun());
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Adds a metric to the list of displayed metrics and associates a color with it
 * @param run_id The id of the run with which the metric is associated
 * @param metric_id The id of the metric being displayed
 * @param color The color to be associated with the metric
 */
safe_selector_full.prototype.addDisplayed = function(run_id, metric_id, color){
    this.selectMetric.setMetricColor(run_id, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Returns the parameters being used to determine selectable design points
 * @return The parameters being used to determine selectable design points, form:
 *         [{"factor_name": <factor_name>, "min_value":<min_value>, "max_value":<max_value>}, ...]
 */
safe_selector_full.prototype.getDesignParameters = function(){
    return this.selectDesignPoint.getSelectionParameters();
}

/**
 * Removes a metric from the list of displayed metrics
 * @param run_id The id of the run with which the metric is associated
 * @param metric_id The id of the metric to be removed
 */
safe_selector_full.prototype.removeDisplayed = function(run_id, metric_id){
    this.selectMetric.setMetricColor(run_id, metric_id, undefined);
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Called when a metric is selected from the metric selector component
 */
safe_selector_full.prototype.selectElementChange = function(){
    var metric_id = d3.event.target.getAttribute("metric_id");
    var metric = this.selectMetric.getMetric(metric_id);
    if(this.selectionCallBack){ this.selectionCallBack(this.getSelectedRun(), metric) }
}

/**
 * Returns the currently selected factor
 * @return The curretnly selected factor
 */
safe_selector_full.prototype.getSelectedFactor = function(){
    return this.selectFactor.getSelectedFactor();
}

/**
 * Returns the currently selected minimum level
 * @return The curretnly selected minimum level
 */
safe_selector_full.prototype.getMinLevel = function(){
    return this.selectLevelMin.getSelectedLevel();
}

/**
 * Returns the currently selected maximum level
 * @return The currently selected maximum level
 */
safe_selector_full.prototype.getMaxLevel = function(){
    return this.selectLevelMax.getSelectedLevel();
}

/**
 * Returns the currently selected design point
 * @return the currently selected design point
 */
safe_selector_full.prototype.getSelectedDesignPoint = function(){
    return this.selectDesignPoint.getSelectedDesignPoint();
}

/**
 * Returns the currently selected run
 * @return the currently selected run
 */
safe_selector_full.prototype.getSelectedRun = function(){
    return this.selectRun.getSelectedRun();
}

/**
 * Returns the parameters currently being used to filter results
 * @return filter parameters of the form [{<parameter_name>:<parameter_value>}, ...]
 */
safe_selector_full.prototype.getResultsParameters = function(){
    return this.selectResults.getResultsParameters();
}
/**-------------------------------------
 * A histogram options object
 * -------------------------------------
 *
 * An object prototype which allows the selection of various
 * setting for a currently selected histogram 
 */

safe.selector.histogram_options = safe_selector_histogram_options;

/* Include functionality of the options selector */
safe_selector_histogram_options.prototype = Object.create(safe.selector.options.prototype);
safe_selector_histogram_options.constructor = safe_selector_histogram_options;


/**
 * Creates a new selection object
 * @param titleElement The dom element that holds the graph's title 
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_histogram_options(titleElement,  parentElement){
    this.plotList = {};
    safe.selector.options.apply(this, [titleElement, parentElement]);
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_histogram_options.prototype.initComponents = function(){
    safe.selector.options.prototype.initComponents.apply(this);

    this.dSelf.append("label")
        .attr("class", "col-xs-12")
        .text("Bin Number:");

    var binsDropdown = this.dSelf.append("div")
        .attr("class", "col-xs-6")
        .style("padding", "0px")
            .append("select")
            .attr("class", "form-control")
            .on("change", this.binsDropdownChange.bind(this));

    binsDropdown.selectAll("option").data(["sqrt(n)", "log n"])
        .enter()
        .append("option")
        .text(function(d) { return d });

    var binNumber = this.dSelf.append("div")
        .attr("class", "col-xs-6")
        .style("padding", "0px")
            .append("input")
            .attr("class", "form-control")
            .attr("type", "number")
            .attr("min", 1)
            .on("change", this.binNumberChange.bind(this));

    this.dSelf.append("label")
        .attr("class", "col-xs-6")
        .text("Bin Width:");
    var binWidth = this.dSelf.append("div")
        .attr("class", "col-xs-6")
        .style("padding", "0px")
            .append("input")
            .attr("class", "form-control")
            .attr("type", "number")
            .on("change", this.binWidthChange.bind(this));

    this.frequency = false;
    var freqButton = this.dSelf.append("button")
        .attr("class", "col-xs-12 btn btn-default")
        .text("Frequency/Probability")
        .on("click", this.setSelectedFrequency.bind(this)); 

    this.binsDropdown = binsDropdown[0][0];
    this.frequencyButton = freqButton[0][0];
    this.binNumber = binNumber[0][0];
    this.binWidth = binWidth[0][0];
}

/**
 * Sets the label text boxes based on the currently selected graph
 */
safe_selector_histogram_options.prototype.setLabels = function(){
    safe.selector.options.prototype.setLabels.apply(this);
    if(this.frequency === true){ 
        this.selectedGraph.setYAxisLabel("Frequency") 
    }else { 
        this.selectedGraph.setYAxisLabel("Probability") 
    }

    this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
}
        
/**
 * Sets whether the graph is displaying by frequency or percentage
 */
safe_selector_histogram_options.prototype.setSelectedFrequency = function(){
    this.frequency = !this.frequency;
    if(this.selectedGraph == undefined){ return }
    this.selectedGraph.setFrequency(this.frequency);

    if(this.frequency === true){ 
        this.selectedGraph.setYAxisLabel("Frequency") 
    }else { 
        this.selectedGraph.setYAxisLabel("Probability") 
    }

    this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
    this.selectedGraph.display();

    d3.values(this.plotList).forEach(function(d){
        d.setFrequency(this.frequency);
        d.setYAxisLabel(this.yAxisLabel.value);
        d.display();
    }.bind(this));
}

/**
 * Adds a graph to the list that is currently displayed.
 * This is required because some option changes must be 
 * reflected accross all displayed graphs
 */
safe_selector_histogram_options.prototype.addGraph = function(id, graph){
    this.plotList[id] = graph;
    if(!this.selectedGraph){ return }

    this.plotList[id].setBinNumber(this.selectedGraph.getBinNumber());
    if(this.frequency === true){ 
        this.plotList[id].setYAxisLabel("Frequency") 
    }else { 
        this.plotList[id].setYAxisLabel("Probability") 
    }
    this.plotList[id].display();
}

/**
 * Called when the bins dropdown changes
 */
safe_selector_histogram_options.prototype.binsDropdownChange = function(){
    if(this.selectedGraph == undefined || this.selectedGraph.getData() == undefined){ return }

    var select = this.binsDropdown;
    var selected = select.options[select.selectedIndex].value;
    if(selected == "sqrt(n)"){
        this.binNumber.value = Math.ceil(Math.sqrt(this.selectedGraph.getData().length));
        this.binNumberChange();
    }else if(selected == "log n"){
        this.binNumber.value = Math.ceil(1 + 3.3 * Math.log(this.selectedGraph.getData().length));     // Sturges' Rule
        this.binNumberChange();
    }
}

/**
 * Sets the currently selected graph
 * @param graph The graph object that has been selected 
 */
safe_selector_histogram_options.prototype.setSelectedGraph = function(graph){
    safe.selector.options.prototype.setSelectedGraph.apply(this, [graph]);
    this.binNumber.value = this.selectedGraph.getBinNumber();
    this.binWidth.value = this.selectedGraph.getBinWidth();
    this.frequency = this.selectedGraph.getFrequency();
}

/**
 * Called when the selected number of bins has changed
 */
safe_selector_histogram_options.prototype.binNumberChange = function(){
    if(this.selectedGraph.getData() == undefined){ return }
    var binNumber = parseInt(this.binNumber.value);
    if(binNumber < 1){ 
        alert("Must have at least one bin.");
        this.binNumber.value = 1;
        binNumber = 1;
    }
    this.selectedGraph.setBinNumber(binNumber);
    this.binNumber.value = this.selectedGraph.getBinNumber();
    this.binWidth.value = this.selectedGraph.getBinWidth();
    this.selectedGraph.display();

    d3.values(this.plotList).forEach(function(d){
        d.setBinNumber(this.selectedGraph.getBinNumber());
        d.display();
    }.bind(this));
}

/**
 * Called when the selected width of the bins has changed
 */
safe_selector_histogram_options.prototype.binWidthChange = function(){
    if(this.selectedGraph.getData() == undefined){ return }
    var binWidth = parseFloat(this.binWidth.value);
    if(binWidth <= 0){ 
        alert("Bin width must be greater than 0.");
        this.binWidth.value = 1;
        binWidth = 1;
    }
    this.selectedGraph.setBinWidth(binWidth);
    this.binNumber.value = this.selectedGraph.getBinNumber();
    this.binWidth.value = this.selectedGraph.getBinWidth();
    this.selectedGraph.display();

    d3.values(this.plotList).forEach(function(d){
        d.setBinNumber(this.selectedGraph.getBinNumber());
        d.display();
    }.bind(this));
}
/**-------------------------------------
 * A selector for the lineplot graphs
 * -------------------------------------
 * 
 * A selector which allows the selection of one level for each
 * factor except one. That factor is the one to be varied as
 * others remain constant
 */
safe.selector.lineplot = safe_selector_lineplot;

/**
 * Create a new selector object
 *
 * @param exp_id The id of the experiment currently being observed
 * @param factor The factor that is to be varied as others remain constant
 * @param metricSelectCallBack The function to be called when a metric is selected
 * @param parentElement The dom element, d3 object, or dom id of the element
 *          to which this element should be appended
 */
function safe_selector_lineplot(exp_id, factor, metricSelectCallBack, parentElement){
	this.exp_id = exp_id;
	this.factor = factor;
	this.metricSelectCallBack = metricSelectCallBack;
    this.dParent = safe.tools.d3Object(parentElement);

	this.dSelf = this.dParent.append("div")
        .attr("class", "container well")
        .style("width", "95%");

    var hider = new safe.tools.hider(this.dSelf, this.dParent);
    hider.initComponents();

    this.levelList = {};
    this.factorList = {};
    this.parameters = {};
    this.selectMetric = new safe.selector.metric(this.exp_id, this.metricSelectChange.bind(this), this.dSelf);

    this.selectMetric.readInitialData(function(){
        this.readInitialData(this.initComponents.bind(this));
    }.bind(this));
}

/**
 * Reads the initial data required for proper operation
 * @param callBack A function to be called after the read is complete
 */
safe_selector_lineplot.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/levels/", function(error, data){
        if(error) { console.warn(error) }

        Object.keys(data).forEach(function(factor_id){
            if(factor_id == this.factor.id){ return }
            this.factorList[factor_id] = data[factor_id];
            this.levelList[factor_id] = data[factor_id].levels;
            this.parameters[data[factor_id].name] = data[factor_id].levels[0];
        }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

/**
 * Initializes the dom elmenets of this object
 */
safe_selector_lineplot.prototype.initComponents = function(){
    var selectors = this.dSelf.selectAll(".factor").data(Object.keys(this.levelList)).enter()
        .append("div")
        .attr("class", "col-xs-12 factor");

    selectors.append("label")
        .attr("class", "col-xs-6")
        .text(function(d) { return this.factorList[d].name + " : " }.bind(this));

    selectors
        .append("select")
        .attr("class", "form-control")
        .attr("factor_id", function(d){ return d })
        .on("change", this.levelSelectChange.bind(this))
            .selectAll("option").data(function(d){ return this.levelList[d] }.bind(this))
            .enter()
            .append("option")
            .text(function(d){ return d });

    this.dSelf.append("div").attr("class", "clearfix");

    this.selectMetric.initComponents();
    this.selectMetric.setDistinctionId(getKeyString(this.parameters));
}

/**
 * Called when a selected level is changed
 */
safe_selector_lineplot.prototype.levelSelectChange = function(){
    var select = d3.event.target;
    var level = select.options[select.selectedIndex].value;
    var factor_name = this.factorList[select.getAttribute("factor_id")].name;
    this.parameters[factor_name] = level;
    this.selectMetric.setDistinctionId(getKeyString(this.parameters));
}

/**
 * Called when a metric is selected
 */
safe_selector_lineplot.prototype.metricSelectChange = function(metric){
    this.metricSelectCallBack(this.parameters, metric);
}

/**
 * Set a metric as being selected 
 * @param keyString A string that destinguishes one set of metrics from another
 *          An example could be a run id
 * @param metric_id The id of the metric that is to be set as selected
 * @param color The color to be used in indicating this metric as selected
 */
safe_selector_lineplot.prototype.addDisplayed = function(keyString, metric_id, color){
    this.selectMetric.setMetricColor(keyString, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Set a metric as being unselected
 * @param keyString A string that destinguishes one set of metrics from another
 *          An example could be a run id
 * @param metric_id The id of the metric that is to be set as unselected
 */
safe_selector_lineplot.prototype.removeDisplayed = function(keyString, metric_id){
    this.selectMetric.setMetricColor(keyString, metric_id, undefined);
    this.selectMetric.updateSelectedMetrics();
}

/**-------------------------------------
 * A qqplot selector object 
 * -------------------------------------
 *
 * An object prototype which allows selections for the qqplot grpah
 */
safe.selector.qqplot_select = safe_selector_qqplot_select;

/**
 * Creates a new qq plot selector object
 * @param exp_id The experiment id with which this selector is to be associated
 * @param selectionCallBack A function to be called when a metric is selected
 * @param applyFilterCallBack A function to be called when a new filter is created
 * @param parementElement The dom id, dom element, or d3 object to which this selector
 *        should be added
 */
function safe_selector_qqplot_select(exp_id, selectionCallBack, applyFilterCallBack, parentElement){
        this.exp_id = exp_id;
        this.dParent = safe.tools.d3Object(parentElement);
        this.selectionCallBack = selectionCallBack;
        this.applyFilterCallBack = applyFilterCallBack;
        
        this.dSelf = this.dParent.append("div").attr("class", "qqplot_select");

        this.selectAxis = new safe.selector.axis(this.axisSelectChange.bind(this), this.dSelf);
        this.selectFactor = new safe.selector.factor(this.exp_id, this.factorSelectChange.bind(this), this.dSelf);
        this.selectLevelMin = new safe.selector.level(this.exp_id, undefined, this.dSelf);
        this.selectLevelMax = new safe.selector.level(this.exp_id, undefined, this.dSelf);
        this.selectDesignPoint = new safe.selector.designpoint(this.exp_id, this.designPointSelectChange.bind(this), this.dSelf);
        this.selectRun = new safe.selector.run(this.exp_id, this.runSelectChange.bind(this), this.dSelf);
        this.selectMetric = new safe.selector.metric(this.exp_id, this.selectElementChange.bind(this), this.dSelf);
        this.selectResults = new safe.selector.results(this.dSelf);
        this.selectFile = new safe.selector.file(this.dSelf);

        this.selectFactor.readInitialData(function(){
            this.selectLevelMin.readInitialData(function(){
                this.selectLevelMax.readInitialData(function(){
                    this.selectMetric.readInitialData(this.initComponents.bind(this))
                }.bind(this));
            }.bind(this));
        }.bind(this));
}
      
/**
 * Initializes the dom components for this object
 */
safe_selector_qqplot_select.prototype.initComponents = function(){
    // Initialize the axis selection
    this.selectAxis.initComponents();

    // Initialize the factor list
    this.selectFactor.initComponents();
    this.selectLevelMin.initComponents();
    this.selectLevelMax.initComponents();
     
    // Initialize the apply selection button
    this.dSelf
        .append("button")
        .attr("class", "apply_selection")
        .text("Apply Selection Filter")
        .on("click", this.applyDesignFilter.bind(this));

    this.selectDesignPoint.initComponents();
    this.selectRun.initComponents();
    this.selectMetric.initComponents();
    this.selectResults.initComponents();
    this.selectFile.initComponents();

    this.factorSelectChange();
    this.updateDesignPoints();
}

/**
 * Called when the selected axis is changed
 */
safe_selector_qqplot_select.prototype.axisSelectChange = function(){
    this.selectMetric.setDistinctionId(this.getSelectedAxis() + "_" + this.getSelectedRun());
    this.selectMetric.updateSelectedMetrics();  
}

/**
 * Called when the selected factor is changed
 */
safe_selector_qqplot_select.prototype.factorSelectChange = function(){

    var factor_id = this.selectFactor.getSelectedFactor().id;

    this.selectLevelMin.updateLevels(factor_id);
    this.selectLevelMax.updateLevels(factor_id);

    this.selectLevelMax.eSelect.selectedIndex = this.selectLevelMax.eSelect.options.length - 1;
}

/**
 * Called when the "Apply Filter" button is clicked
 */
safe_selector_qqplot_select.prototype.applyDesignFilter = function(){

    var factor = this.selectFactor.getSelectedFactor();
    var min = this.selectLevelMin.getMinLevel(factor.id);
    var max = this.selectLevelMax.getMaxLevel(factor.id);

    if(this.getMinLevel() > this.getMaxLevel() ){
        alert("The factor's minimum level cannot be greater than the factor's maximum level.");
        return;
    }

    var parameter = {"name":factor.name, "factor_id":factor.id, "min_value":this.getMinLevel(), "max_value":this.getMaxLevel() };
    if(this.getMinLevel() > min || this.getMaxLevel() < max ){
        this.selectDesignPoint.addParameter(factor.id, parameter);
    }else{
        this.removeDesignFilter(factor.id);
    }

    this.updateDesignPoints();
    this.applyFilterCallBack(parameter);
}

/**
 * Removes the specified filter from the list of filters
 * @param factor_id The id of the factor with which the filter is associated
 */
safe_selector_qqplot_select.prototype.removeDesignFilter = function(factor_id){
    this.selectDesignPoint.removeParameter(factor_id);

    this.updateDesignPoints();
}

/**
 * Updates the listed design points based on filters and information in the database
 */
safe_selector_qqplot_select.prototype.updateDesignPoints = function(){
    this.selectDesignPoint.updateDesignPoints(this.designPointSelectChange.bind(this));
}

/**
 * Called when the selected design point changes
 */
safe_selector_qqplot_select.prototype.designPointSelectChange = function(){
    this.selectRun.updateRuns(this.selectDesignPoint.getSelectedDesignPoint(), function(){
        this.selectMetric.setDistinctionId(this.getSelectedAxis() + "_" + this.getSelectedRun());
        this.selectMetric.updateSelectedMetrics();  
    }.bind(this));
}

/**
 * Called when the selected run is changed
 */
safe_selector_qqplot_select.prototype.runSelectChange = function(){
    this.selectMetric.setDistinctionId(this.getSelectedAxis() + "_" + this.getSelectedRun());
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Adds a metric to the list of displayed metrics and associates a color with it
 * @param run_id The id of the run with which the metric is associated
 * @param metric_id The id of the metric being displayed
 * @param color The color to be associated with the metric
 */
safe_selector_qqplot_select.prototype.addDisplayed = function(axis, run_id, metric_id, color){
    this.selectMetric.setMetricColor(axis + "_" + run_id, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Returns the parameters being used to determine selectable design points
 * @return The parameters being used to determine selectable design points, form:
 *         [{"factor_name": <factor_name>, "min_value":<min_value>, "max_value":<max_value>}, ...]
 */
safe_selector_qqplot_select.prototype.getDesignParameters = function(){
    return this.selectDesignPoint.getSelectionParameters();
}

/**
 * Removes a metric from the list of displayed metrics
 * @param run_id The id of the run with which the metric is associated
 * @param metric_id The id of the metric to be removed
 */
safe_selector_qqplot_select.prototype.removeDisplayed = function(axis, run_id, metric_id){
    this.selectMetric.setMetricColor(axis + "_" + run_id, metric_id, "white");
    this.selectMetric.updateSelectedMetrics();
}

/**
 * Called when a metric is selected from the metric selector component
 */
safe_selector_qqplot_select.prototype.selectElementChange = function(){
    var metric_id = d3.event.target.getAttribute("metric_id");
    var metric = this.selectMetric.getMetric(metric_id);
    if(this.selectionCallBack){ this.selectionCallBack(this.getSelectedAxis(), this.getSelectedRun(), metric) }
}

/**
 * Returns the currently selected axis
 * @return The currently selected axis
 */
safe_selector_qqplot_select.prototype.getSelectedAxis = function(){
    return this.selectAxis.getSelectedAxis();
}

/**
 * Returns the currently selected factor
 * @return The curretnly selected factor
 */
safe_selector_qqplot_select.prototype.getSelectedFactor = function(){
    return this.selectFactor.getSelectedFactor();
}

/**
 * Returns the currently selected minimum level
 * @return The curretnly selected minimum level
 */
safe_selector_qqplot_select.prototype.getMinLevel = function(){
    return this.selectLevelMin.getSelectedLevel();
}

/**
 * Returns the currently selected maximum level
 * @return The currently selected maximum level
 */
safe_selector_qqplot_select.prototype.getMaxLevel = function(){
    return this.selectLevelMax.getSelectedLevel();
}

/**
 * Returns the currently selected design point
 * @return the currently selected design point
 */
safe_selector_qqplot_select.prototype.getSelectedDesignPoint = function(){
    return this.selectDesignPoint.getSelectedDesignPoint();
}

/**
 * Returns the currently selected run
 * @return the currently selected run
 */
safe_selector_qqplot_select.prototype.getSelectedRun = function(){
    return this.selectRun.getSelectedRun();
}

/**
 * Returns the parameters currently being used to filter results
 * @return filter parameters of the form [{<parameter_name>:<parameter_value>}, ...]
 */
safe_selector_qqplot_select.prototype.getResultsParameters = function(){
    return this.selectResults.getResultsParameters();
}

/**--------------------------
 * Window Buttons Object
 *--------------------------
 *
 * An object which adds button that allow and element
 * to be hidden, displayed, or removed from the page
 */

safe.tools.window_buttons = safe_tools_window_buttons;

/**
 * Create a new window buttons object
 * @param element The element with which the buttons are to be associated
 */
function safe_tools_window_buttons(element){
    this.element = safe.tools.d3Object(element);
}

/**
 * Initializes the buttons components within the element
 */
safe_tools_window_buttons.prototype.initComponents = function(){
    this.displayed = this.element.append("div")
        .style({"width": "100%", "padding": "0px" });

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.remove.bind(this));

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-minus-sign")
        .style({"cursor": "pointer", "padding-right": "0.2cm" })
        .on("click", this.toggleHidden.bind(this));

    var domElement = this.element[0][0];
    var newDiv = document.createElement("div");
    var hidden = domElement.parentNode.insertBefore(newDiv, domElement);

    this.hidden = d3.select(newDiv)
        .attr("class", "well container")
        .style({"width": "95%", "display": "none"});

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.remove.bind(this));

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-plus-sign")
        .style({"cursor": "pointer", "padding-right": "0.2cm"})
        .on("click", this.toggleHidden.bind(this));
}

/**
 * Toggles the element between hidden and displayed
 */
safe_tools_window_buttons.prototype.toggleHidden = function(){
    var temp = this.element.style("display");
    this.element.style("display", this.hidden.style("display"));
    this.hidden.style("display", temp);
}

/**
 * Removes the element from the display.
 */
safe_tools_window_buttons.prototype.remove = function(){
    this.element.remove();
    this.hidden.remove();
}
//-------------------------------------
// Hider Button
//-------------------------------------
//
// An object that can be added to an html element
// which creates minimize and maximize button which
// control whether the element is fully displayed
// or hidden.

safe.tools.hider = safe_tools_hider;

/**
 * Create a new hider object
 * @param element The element to be hidden
 */
function safe_tools_hider(element){
    this.element = safe.tools.d3Object(element);
}

/**
 * Initialize the hider components within the element
 */
safe_tools_hider.prototype.initComponents = function(){
    this.displayed = this.element.append("div")
        .style({"width": "100%", "padding": "0px" });

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-minus-sign")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.toggleHidden.bind(this));

    this.displayed.append("label").text(" Hide");

    var domElement = this.element[0][0];
    var newDiv = document.createElement("div");
    var hidden = domElement.parentNode.insertBefore(newDiv, domElement);

    this.hidden = d3.select(newDiv)
        .attr("class", "well container")
        .style({"width": "95%", "display": "none"});

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-plus-sign")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm"})
        .on("click", this.toggleHidden.bind(this));

    this.hidden.append("label").text("Show");
}

/**
 * Toggles whether the element is hidden or displayed
 */
safe_tools_hider.prototype.toggleHidden = function(){
    var temp = this.element.style("display");
    this.element.style("display", this.hidden.style("display"));
    this.hidden.style("display", temp);
}
