/**----------------------------------
 * A graph for scatterplots 
 *----------------------------------
 *
 * This object uses given data with a specified format to create
 * a graph with confidence intervals. The data given should be 
 * of the form [{xValue:__, medianValue:__, highDevValue:__, lowDevValue:__}, ...] 
 */

safe.graph.scatterplot = safe_graph_scatterplot;

// This plot inherits from the cartesian plot
safe_graph_scatterplot.prototype = Object.create(safe.graph.cartesian.prototype);
safe_graph_scatterplot.prototype.constructor = safe_graph_scatterplot;

/**
 * Creates a new lineplot object
 * @param margins The padding to be placed around the graphing area {top: __, bottom:__, right:__, left:__}
 * @param width The pixel width of the entire graph including graphing area and margins
 * @param selectedCallBack The function to be called when the user clicks on this graph
 *          This value can be "undefined" in which case nothing will happen (will be called "selectedCallBack(this)")
 * @param parentElement The element to which this graph should be added. This can be
 *          a dom element, an html id, or a d3 object.
 *
 * @return The newly created object
 * */
function safe_graph_scatterplot(margins, width, height, onSelectedCallBack, parentElement){
    safe.graph.cartesian.apply(this, [margins, width, height, onSelectedCallBack, parentElement]); 
    this.numberFormat = d3.format(".2f");
    return this;
}

/**
 * Displays the specified data set in the form associated with line graphs
 * @param id The string id for the desired data set specified in "addDataSet"
 */
safe_graph_scatterplot.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".dataPoint_" + id).remove();
        return;
    }

    points = this.svg.selectAll(".dataPoint_" + id).data(this.data[id]);
    points.enter()
        .append("circle")
        .attr("clip-path", "url(#" + this.clipId + ")" )
        .attr("class", "dataPoint_" + id)
        .attr(this.pointAttributes[id])
        .on("mouseover", this.mouseOverPoint.bind(this))
        .on("mouseout", this.mouseOutPoint.bind(this));

    points
        .attr("cx", function (d) { return this.margin.left + this.xScale(this.xAccessor[id](d)) }.bind(this))
        .attr("cy", function (d) { return this.margin.top + this.yScale(this.yAccessor[id](d)) }.bind(this))
        .attr("info", function(d) { return this.numberFormat(this.xAccessor[id](d)) + "<br/>" + this.numberFormat(this.yAccessor[id](d)) }.bind(this));

	points.exit().remove();	
}

