/**-------------------------------------
 * A Default Infobox Object
 * -------------------------------------
 *
 * The prototype of an object which holds lines of information which
 * can be removed by the user.
 */

safe.infobox.default = safe_infobox_default;

/**
 * The constructor of the information object
 *
 * @param removeCallBack The function to be called when an element
 *        is selected for removal. Called as "removeCallBack(key)"
 *        where key is the key given thorugh "addElement"
 * @param parentElement The element to which this infobox should be
 *        added. It can be and element id, dom element, or d3 object.
 */
function safe_infobox_default(removeCallBack, parentElement){
    this.removeCallBack = removeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    
    this.dSelf = this.dParent
        .append("div")
        .attr("class", "container well col-xs-12")
        .style("width", "100%");

    this.elements = {};
    this.returnAttributes = {};
}

/**
 * Adds an element to the infobox
 * @param key A unique identifier for this element
 * @param elementString The informational string to be displayed
 * @param returnAttributes A value (or object) to be returned along 
 *        with an element's key when the element is removed.
 */
safe_infobox_default.prototype.addElement = function(key, elementString, returnAttributes){
    this.elements[key] = elementString;
    this.returnAttributes[key] = returnAttributes;
    this.updateElements();
}

/**
 * Removes an element from the infobox
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_default.prototype.removeElement = function(key){
    console.log("Deleting: ", key);
    delete this.elements[key];
    delete this.returnAttributes[key];
    this.updateElements();
}

/**
 * Called when an element is selected for removal by the user
 * @param key The key associated with the element through "addElement"
 */
safe_infobox_default.prototype.removeSelected = function(key){
    this.removeCallBack(this.returnAttributes[key]);
}

/**
 * Updates the elements being displayed in the infobox
 * to those currently associated with it.
 */
safe_infobox_default.prototype.updateElements = function(){
    console.log(this.elements);
    this.dSelf.selectAll("div").remove();

    var parameters = this.dSelf.selectAll("div").data(Object.keys(this.elements))
        .enter()
        .append("div")
        .attr("class", "col-xs-12")

    parameters.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle col-xs-1")
        .style("cursor", "pointer")
        .on("click", function(d){ this.removeSelected(d) }.bind(this));

    parameters
        .append("div")
        .attr("class", "select_label")
        .text(function(d) { return this.elements[d] }.bind(this));
}
