/**-------------------------------------
 * An x-axis selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of axis 
 * in addition to the default selection options
 */

safe.selector.axis = safe_selector_axis;

/**
 * Creates a new selection object
 * @param changeCallBack A function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_axis(changeCallBack, parentElement){
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.xAxis = safe.constants.X_AXIS;
    this.yAxis = safe.constants.Y_AXIS;
    this.data = [{"name":"X Axis", "value":this.xAxis},
                 {"name":"Y Axis", "value":this.yAxis}];
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_axis.prototype.initComponents = function(){
    this.dSelf = this.dParent
        .append("div");

    this.dSelect = this.dSelf
        .append("select")
        .attr("class", "form-control")
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];

    this.dSelect.selectAll("option").data(this.data)
        .enter()
        .append("option")
        .attr("axis", function(d){ return d.value })
        .text(function(d){ return d.name });
}

/**
 * Called when there is a change in the currently selected axis.
 */
safe_selector_axis.prototype.selectChange = function(){
    this.callBack(this.getSelectedAxis());
}

/**
 * Returns the design point that is currently selected
 * @return The currently selected axis 
 */
safe_selector_axis.prototype.getSelectedAxis = function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("axis");
}
