/**-------------------------------------
 * A results selector object 
 * -------------------------------------
 *
 * An object prototype which allows the selection of filtering options
 * for results that are returned from the database
 */
safe.selector.results = safe_selector_results;

/**
 * Creates a new selection object
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added
 */
function safe_selector_results(parentElement){
    this.exp_id = exp_id;
    this.dParent = safe.tools.d3Object(parentElement);
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_results.prototype.initComponents = function(){

    this.dSelect = this.dParent.append("div");
    this.eSelect = this.dSelect[0][0];

    var data = [{name: "Filter Step:"},
                {name: "Max Results:"},
                {name: "Moving Average:"},
                {name: "Outlier Filter:"}];

    var lines = this.dSelect.selectAll(".key_line").data(data);

    lines = lines.enter().append("div")
        .attr("class", "col-xs-12")
        .style("padding", "0px");

    lines
        .append("label")
        .attr("class", "col-xs-6")
        .text(function(d) { return d.name });

    var inputs = lines
        .append("div")
        .attr("class", "col-xs-6")
            .append("input")
            .attr("type", "number")
            .attr("min", 0)
            .attr("value", 0)
            .attr("class", "form-control");

    this.clusterSize = inputs[0][0];
    this.maxResults = inputs[0][1];
    this.denoiseWindowSize = inputs[0][2];
    this.outlierFilter = inputs[0][3];
}

/**
 * Returns the currently selected cluster size option
 * @return The currently selected cluster size option
 */
safe_selector_results.prototype.getClusterSize = function(){
    if (this.clusterSize.value == ""){ this.clusterSize.value = 0 }
    return this.clusterSize.value;
}

/**
 * Returns the currently selected maximum results setting
 * @return The curretnly selected maximum results setting
 */
safe_selector_results.prototype.getMaxResults = function(){
    if (this.maxResults.value == ""){ this.maxResults.value = 0 }
    return this.maxResults.value;
}

/**
 * Returns the currently selected denoise window setting
 * @return The curretnly selected denoise window setting
 */
safe_selector_results.prototype.getDenoiseWindowSize = function(){
    if (this.denoiseWindowSize.value == ""){ this.denoiseWindowSize.value = 0 }
    return this.denoiseWindowSize.value;
}

/**
 * Returns the currently selected outlier filter setting
 * @return The curretnly selected outlier filter setting
 */
safe_selector_results.prototype.getFilterOutliers = function(){
    if (this.outlierFilter.value == "" || parseFloat(this.outlierFilter.value) < 0){ this.outlierFilter.value = 0 }
    return this.outlierFilter.value;
}

/**
 * Returns an object of all results parameters
 * @return An object of all results parameters in the form
 *      {<parameter_name>:<parameter_value>, ...}
 */
safe_selector_results.prototype.getResultsParameters = function(){
    return {"filter_step": this.getClusterSize(),
            "max_results":this.getMaxResults(),
            "denoise_window":this.getDenoiseWindowSize(),
            "filter_outliers":this.getFilterOutliers()};
}
