/**--------------------------
 * Window Buttons Object
 *--------------------------
 *
 * An object which adds button that allow and element
 * to be hidden, displayed, or removed from the page
 */

safe.tools.window_buttons = safe_tools_window_buttons;

/**
 * Create a new window buttons object
 * @param element The element with which the buttons are to be associated
 */
function safe_tools_window_buttons(element){
    this.element = safe.tools.d3Object(element);
}

/**
 * Initializes the buttons components within the element
 */
safe_tools_window_buttons.prototype.initComponents = function(){
    this.displayed = this.element.append("div")
        .style({"width": "100%", "padding": "0px" });

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.remove.bind(this));

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-minus-sign")
        .style({"cursor": "pointer", "padding-right": "0.2cm" })
        .on("click", this.toggleHidden.bind(this));

    var domElement = this.element[0][0];
    var newDiv = document.createElement("div");
    var hidden = domElement.parentNode.insertBefore(newDiv, domElement);

    this.hidden = d3.select(newDiv)
        .attr("class", "well container")
        .style({"width": "95%", "display": "none"});

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-remove-circle")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.remove.bind(this));

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-plus-sign")
        .style({"cursor": "pointer", "padding-right": "0.2cm"})
        .on("click", this.toggleHidden.bind(this));
}

/**
 * Toggles the element between hidden and displayed
 */
safe_tools_window_buttons.prototype.toggleHidden = function(){
    var temp = this.element.style("display");
    this.element.style("display", this.hidden.style("display"));
    this.hidden.style("display", temp);
}

/**
 * Removes the element from the display.
 */
safe_tools_window_buttons.prototype.remove = function(){
    this.element.remove();
    this.hidden.remove();
}
