#!/usr/bin/env python

from twisted.internet.protocol import Protocol, ProcessProtocol, Factory
from twisted.protocols.basic import Int32StringReceiver
from twisted.internet import reactor, protocol
from twisted.internet.error import ProcessDone, ProcessTerminated
import json
import os
import sys
import base64
import struct
import sys
import time
import signal
STDIN = 0
STDOUT = 1
STDERR = 2
SAMPLES = 3

class PipeProtocol(ProcessProtocol):
  """Protocol responsible to connect NS3 <--> Sim. Client manager (worker)"""
  def __init__(self, connection, proc_id):
    self.connection=connection
    self.id = proc_id
    self.done = False
    self.buffer = {'id': proc_id, 'data': [], 'type':'result'}
    self.buffer_size = 0

    #self.transport.disconnecting = False #Hack to make Int32StringReceiver work
   
  def connectionMade(self):
    #Little hack to use the Int32String protocol with pipes
    self.parser = Int32StringReceiver()
    self.parser.makeConnection(None)
    self.parser.stringReceived = self.stringReceived
    print("Connection made!")

  def childDataReceived(self, childFD, data):
    """Handler for data received from simulator"""
    if childFD == STDOUT:
      sys.stdout.write(data) #STD.ouT
    if childFD == STDERR:
      sys.stderr.write(data) #ERR
    if childFD == SAMPLES:
      self.parser.dataReceived(data)

  def processExited(self, status):
    """Handler for when underlying process has exited"""
    self.done = True
    if isinstance(status.value, ProcessDone):
      #Finished on it's own
      print("Process exited OK!!!")
      success=True
      #Send any new data we've received since last buffer dump
      self.connection.sendString(json.dumps(self.buffer))
      self.buffer = {'id': self.id, 'data': [], 'type':'result'}
    elif status.value.signal == signal.SIGTERM:
      #Killed because we've reached the necessary precision
      print("PROCESS KILLED")
      success = True
      print("Sent String DPFINISHED "+str(self.id))          
    else:
      #There was another error that caused the process to end
      print("Process exited WITH ERROR: "+str(status))
      success = False      
    #let server know we've finished the current design_point
    self.connection.sendString(json.dumps({"type":"dp_finished", "success":success, 'id': self.id}))
    #request a new design_point
    self.connection.sendString(json.dumps({"type": "dp_request", 'id' : self.id})) 
      
      
  def processEnded(self, reason):
    print("ENDED!")

  def stringReceived(self, raw_data):
    data = json.loads(raw_data)
    handlers={
          "new_metric":self._handleNewMetric,
          "result":self._handleResult
        }
    #Demux message to correct handler function
    if data['type'] in handlers.keys():
      handlers[data['type']](data)
    else:
      raise Exception("Invalid mesage received: "+raw_data)
    
  def _handleResult(self,data):    
    if not self.done:
      data.pop('type')
      self.buffer['data'].append(data)
      self.buffer_size += 1
      if self.buffer_size >= 1000:
        self.connection.sendString(json.dumps(self.buffer))
        self.buffer = {'id': self.id, 'data': [], 'type':'result'}
        self.buffer_size = 0        


  def _handleNewMetric(self,data):
    if not self.done:
      data['id'] = self.id
      self.connection.sendString(json.dumps(data))

'''if __name__ == "__main__":
  print("Trying to connect pipe...")
  programPath = str(os.path.abspath(os.path.join(os.getcwd(),"testPipe")))
  reactor.spawnProcess(PipeProtocol("bu"), programPath, [os.path.basename(programPath)], None, 
      childFDs={STDIN: 'w', STDOUT: 'r', STDERR: 'r', SAMPLES: 'r'})
  reactor.run()'''
